Package: voice
Type: Package
Title: Speaker Recognition, Voice Analysis and Mood Inference via Music
        Theory
Version: 0.5.6
Date: 2026-05-13
Authors@R: person("Zabala", "Filipe J.", email = "filipezabala@gmail.com", role = c("cre","aut"))
Maintainer: Zabala Filipe J. <filipezabala@gmail.com>
Description: Provides tools for audio data analysis, including feature extraction, pitch detection, and speaker identification. Designed for voice research and signal processing applications.
License: GPL-3
Depends: R (>= 4.1.0)
Imports: arrangements, dplyr, ggplot2, htmltools, httr, httr2, R.utils,
        reticulate, seewave, tabr, tibble, tidyselect, tuneR, wrassp,
        zoo
Suggests: gm, knitr, tidyverse
URL: https://github.com/filipezabala/voice
BugReports: https://github.com/filipezabala/voice/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-13 19:42:06 UTC; filipezabala
Author: Zabala Filipe J. [cre, aut]
Repository: CRAN
Date/Publication: 2026-05-13 20:00:02 UTC
Built: R 4.5.2; ; 2026-05-13 22:41:11 UTC; unix

Package: voice
Type: Package
Title: Speaker Recognition, Voice Analysis and Mood Inference via Music
        Theory
Version: 0.5.6
Date: 2026-05-13
Authors@R: person("Zabala", "Filipe J.", email = "filipezabala@gmail.com", role = c("cre","aut"))
Maintainer: Zabala Filipe J. <filipezabala@gmail.com>
Description: Provides tools for audio data analysis, including feature extraction, pitch detection, and speaker identification. Designed for voice research and signal processing applications.
License: GPL-3
Depends: R (>= 4.1.0)
Imports: arrangements, dplyr, ggplot2, htmltools, httr, httr2, R.utils,
        reticulate, seewave, tabr, tibble, tidyselect, tuneR, wrassp,
        zoo
Suggests: gm, knitr, tidyverse
URL: https://github.com/filipezabala/voice
BugReports: https://github.com/filipezabala/voice/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-13 19:42:06 UTC; filipezabala
Author: Zabala Filipe J. [cre, aut]
Repository: CRAN
Date/Publication: 2026-05-13 20:00:02 UTC
Built: R 4.6.0; ; 2026-05-13 20:25:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export(assign_notes)
export(audio_time)
export(check_chords)
export(cut_audio)
export(diarize)
export(duration)
export(embed_audio)
export(embed_video)
export(enrich_rttm)
export(expand_model)
export(extract_features)
export(feat_summary)
export(get_bit)
export(get_dur)
export(get_left)
export(get_right)
export(get_samp.rate)
export(get_tbeg)
export(get_tdur)
export(interp)
export(interp_df)
export(interp_mc)
export(is_mono)
export(mp3)
export(mp4)
export(notes)
export(notes_freq)
export(piano_plot)
export(png)
export(read_rttm)
export(rm0)
export(smooth_df)
export(splitw)
export(spn2abc)
export(tag)
export(transcribe)
export(write_list)
import(reticulate)
importFrom(dplyr,"%>%")
importFrom(dplyr,mutate_each)
importFrom(dplyr,select)
importFrom(stats,median)
importFrom(stats,quantile)

# `voice`

General tools for voice analysis. 

The `voice` package is being developed to be an easy-to-use set of tools to deal with audio analysis in R. It provides a free and user-friendly toolkit for audio analysis, enabling researchers to extract, tag, and analyze voice data efficiently. It supports the extraction of audio features, enrichment of structured datasets with audio summaries, and automatic identification of spoken segments—while introducing novel features. It also allows audio analysis based on musical theory, associating frequencies with musical notes arranged in a score via [`gm`](https://github.com/flujoo/gm) package. 

## Basic installation
`devtools::install_github('filipezabala/voice')`

More details may be found at 
- https://filipezabala.com/voicegnette/
- https://cran.r-project.org/package=voice 

## Issues
If you want to contribute, report bugs or request new features, use the 'Issues' tab on Github.

# `voice`

General tools for voice analysis. 

The `voice` package is being developed to be an easy-to-use set of tools to deal with audio analysis in R. It provides a free and user-friendly toolkit for audio analysis, enabling researchers to extract, tag, and analyze voice data efficiently. It supports the extraction of audio features, enrichment of structured datasets with audio summaries, and automatic identification of spoken segments—while introducing novel features. It also allows audio analysis based on musical theory, associating frequencies with musical notes arranged in a score via [`gm`](https://github.com/flujoo/gm) package. 

The package has been tested extensively since 2019, including:

- Real-world applications: Dozens of uses, e.g. sex prediction from voice features and speaker diarization in audiobooks.
- Validation: Successful tests on open datasets and LibriVox recordings.

If you want to contribute, report bugs or request new features, use the 'Issues' tab on Github.


## 0. Basic installation
```{r, eval=FALSE}
# Development version from GitHub
install.packages(c('devtools','tidyverse'))
devtools::install_github('filipezabala/voice')

# Stable version from CRAN
install.packages('voice')
```

If you wish to perform a full installation, proceed to Section 4.


### 0.1 For Windows Users
If you're compiling R packages from source, you may need to install [RTools](https://cran.r-project.org/bin/windows/Rtools/), a collection of Windows-specific build tools for R.

### 0.2 For macOS Users
If you're compiling packages, ensure you have [Xcode Command Line Tools](https://mac.install.guide/commandlinetools/) installed. You also may need [macOS tools](https://cran.r-project.org/bin/macosx/tools/).

```{bash, eval=FALSE}
# Install Xcode on MacOS
xcode-select --install
```
More details may be found at https://filipezabala.com/voicegnette/.

## 1. Extract features
### 1.1 Load packages and audio files
```{r, message=FALSE, warning=FALSE}
# packs
library(voice)
library(tidyverse)

# get path to audio file
wavDir <- list.files(system.file('extdata', package = 'wrassp'),
                     pattern = glob2rx('*.wav'), full.names = TRUE)
```

### 1.2 Extract features
```{r, message=FALSE, warning=FALSE}
# minimal usage
M <- voice::extract_features(wavDir)
glimpse(M)
```


## 2. Tag
```{r, message=FALSE, warning=FALSE}
# creating Extended synthetic data
E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3), wav_path = wavDir)
E

# minimal usage
voice::tag(E)

# canonical data
voice::tag(E, groupBy = 'subject_id')
```


## 3. Visualization
### 3.1 Get audio
```{r, message=FALSE, warning=FALSE}
url0 <- 'https://github.com/filipezabala/voiceAudios/raw/refs/heads/main/wav/doremi.wav'
download.file(url0, paste0(tempdir(), '/doremi.wav'), mode = 'wb')
```

You may use the command `voice::embed_audio(url0)` if you wish to show a play 
button when compiling an .Rmd file. See https://github.com/mccarthy-m-g/embedr 
for more details about `embed_audio()` related functions.


### 3.2 Media data
```{r, message=FALSE, warning=FALSE}
M <- voice::extract_features(tempdir())
summary(M)
```

### 3.3 Plot
```{r, message=FALSE, warning=FALSE, fig.width=7.5, fig.height=4}
voice::piano_plot(M, 0) # f0
voice::piano_plot(M, 0:1) # f0 + f1
```

### 3.4 Assign notes
```{r, message=FALSE, warning=FALSE}
(f0_spn <- voice::assign_notes(M, fmt = 0, min_points = 22, min_percentile = .85)) # f0
(f1_spn <- voice::assign_notes(M, fmt = 1, min_points = 22, min_percentile = .85)) # f1
```

### 3.5 Sheet music
Must have [MuseScore](https://musescore.org/en) and [gm](https://flujoo.github.io/gm/articles/gm.html).

#### 3.5.1 Notes sequence of f0
```{r, message=FALSE, warning=FALSE}
library(gm)
line_0 <- gm::Line(as.character(f0_spn))
m0 <- gm::Music() +
  gm::Meter(4, 4) +
  line_0
gm::show(m0, to = c('score', 'audio'))
```


#### 3.5.2 Notes sequences of f0 and f1
```{r, message=FALSE, warning=FALSE}
line_0 <- gm::Line(as.character(f0_spn))
line_1 <- gm::Line(as.character(f1_spn))
m1 <- gm::Music() +
  gm::Meter(4, 4) +
  line_0 + line_1
gm::show(m1, to = c('score', 'audio'))
```

<!-- #### 3.6.3 Chords sequence using f0, f1 and f2  -->
<!-- ```{r, message=FALSE, warning=FALSE} -->
<!-- line_0 <- gm::Line(as.character(f0_spn)) -->
<!-- line_1 <- gm::Line(as.character(f1_spn)) -->
<!-- line_2 <- gm::Line(as.character(f2_spn)) -->
<!-- m2 <- gm::Music() +  -->
<!--   gm::Meter(4, 4) +  -->
<!--   line_0 + line_1 + line_2 -->
<!-- gm::show(m2, to = c('score', 'audio')) -->
<!-- ``` -->


## 4. Advanced installation
Python-based functions `diarize` and `extract_features` (when the latter is inferring `f0_praat` and `fmt_praat` features) require a configured Python environment.

### 4.1 Ubuntu
The following steps are used to fully configure `voice` on Ubuntu 24.04 LTS (Noble Numbat). Reports of inconsistencies are welcome.

<!-- > Without the following Python items 3 and 10, you may run all the functions except `diarize` and `extract_features` calling `f0_praat` and `fmt_praat` features. These functions run respectively [pyannote-audio](https://github.com/pyannote/pyannote-audio) and [Parselmouth](https://github.com/YannickJadoul/Parselmouth). -->

#### 4.1.1. [Curl](https://curl.se/)
Command line tool and library for transferring data with URLs.
```bash
# installing dependencies
sudo apt-get update
sudo apt-get install -y libssl-dev autoconf libtool make
# installing curl
sudo apt install curl
# verify installation
curl --version
```

<!-- #### 4.1.2. [Git](https://git-scm.com/) -->
<!-- Git is a free and open source distributed version control system. -->
<!-- ```bash -->
<!-- sudo apt-get update -->
<!-- sudo apt-get install git-all -->
<!-- ``` -->

<!-- #### 4.1.3. [pip](https://pypi.org/project/pip/) -->
<!-- pip is the package installer for Python.  -->
<!-- ```bash -->
<!-- sudo apt-get update -->
<!-- sudo apt-get install python3-pip -->
<!-- pip3 --version -->
<!-- ``` -->

#### 4.1.2. [ffmpeg](https://ffmpeg.org/)
ffmpeg is a cross-platform solution to record, convert and stream audio and video.
```bash
sudo apt-get update
sudo apt-get install ffmpeg
```

#### 4.1.3. Audio drivers and extra packages
```bash
sudo apt-get update
sudo apt-get install portaudio19-dev libasound2-dev libfontconfig1-dev libmagick++-dev libxml2-dev libharfbuzz-dev libfribidi-dev libgdal-dev cmake cmake-doc ninja-build
```

#### 4.1.4. [MuseScore](https://musescore.org/)
MuseScore is an open source notation software.

```bash
sudo add-apt-repository ppa:mscore-ubuntu/mscore-stable
sudo apt-get update
sudo apt-get install musescore
```

#### 4.1.5. [R](https://www.r-project.org)
R is a free software environment for statistical computing and graphics. To find out your Ubuntu distribution use `lsb_release -a` at terminal.    
```bash
sudo sh -c 'echo "deb https://cloud.r-project.org/bin/linux/ubuntu focal-cran40/" >> /etc/apt/sources.list.d/cran.list'
sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys E084DAB9 
sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys 51716619E084DAB9
gpg -a --export E084DAB9 | sudo apt-key add -

sudo add-apt-repository ppa:c2d4u.team/c2d4u4.0+

sudo apt-get update && sudo apt-get upgrade
sudo apt-get install r-base r-base-dev
```

#### 4.1.6. [RStudio](https://posit.co/downloads)
RStudio is an Integrated Development Environment (IDE) for R. Check for updates [here](https://posit.co/downloads).
```bash
sudo apt-get update
sudo apt-get install gdebi-core
wget https://download1.rstudio.org/electron/jammy/amd64/rstudio-2025.05.0-496-amd64.deb
sudo gdebi rstudio-2025.05.0-496-amd64.deb
```

#### 4.1.9. R packages
"Packages are the fundamental units of reproducible R code." [Hadley Wickham and Jennifer Bryan](https://r-pkgs.org/). The installation may take several minutes. At terminal run:
```bash
sudo R
```

Running R as super user paste the following, row by row:
```r
packs <- c('audio','reticulate','R.utils','seewave','tidyverse','tuneR','wrassp')
install.packages(packs, dep = TRUE)
update.packages(ask = FALSE)
devtools::install_github('egenn/music')
devtools::install_github('flujoo/gm')
```
To configure the `gm` package.
```r
usethis::edit_r_environ()
```

Add the line `MUSESCORE_PATH=/usr/bin/mscore` to `/root/.Renviron` file. To exit use `:wq` at VI. Save and restart the R/RStudio session.


#### 4.1.10. [Miniconda](https://docs.conda.io/en/latest/miniconda.html)
Miniconda is a free minimal installer for [conda](https://docs.conda.io/), an open source package, dependency and environment management system for any language—Python, R, Ruby, Lua, Scala, Java, JavaScript, C/ C++, FORTRAN and more, that runs on Windows, macOS and Linux.   
Follow the instructions at https://docs.conda.io/en/latest/miniconda.html. 

At terminal:
```bash
cd ~/Downloads/
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-Linux-x86_64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-Linux-x86_64.sh
```
Do you accept the license terms? [yes|no] `yes`.

Miniconda3 will now be installed into this location: /home/user/miniconda3 [ENTER]

You can undo this by running `conda init --reverse $SHELL`? `yes`

Do you wish the installer to initialize Miniconda3 by running conda init? `yes`.

Close and reopen terminal.

```bash
conda update -n base -c defaults conda
```

The following packages will be INSTALLED/REMOVED/UPDATED/DOWNGRADED:... Proceed ([y]/n)? `y`

```bash
conda create -n pyvoice python=3.12
```

The following (NEW) packages will be downloaded/INSTALLED:... Proceed ([y]/n)? `y`

```bash
conda activate pyvoice
pip install -r https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt
```


### 4.2 MacOS
The following steps are used to fully configure `voice` on [MacOS Sonoma and Tahoe](https://www.apple.com/os/macos/). Reports of inconsistencies are welcome.

<!-- Hardware   -->
<!--  . MacBook Air (13-inch, 2020)   -->
<!--  . Chip Apple M1 -->
<!--  . Memory 8GB LPDDR4 -->

#### 4.2.1. [Homebrew](https://brew.sh/)
Install Homebrew, 'The Missing Package Manager for macOS (or Linux)' and remember to `brew doctor` eventually. At terminal (`command + space 'terminal'`) run:
```bash
/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"
```

#### 4.2.2. [wget](https://www.gnu.org/software/wget/)
GNU Wget is a free software package for retrieving files using HTTP, HTTPS, FTP and FTPS, the most widely used Internet protocols. It is a non-interactive commandline tool, so it may easily be called from scripts, cron jobs, terminals without X-Windows support, etc.
```bash
brew install wget
```

#### 4.2.3. [Python](https://www.python.org/)
Python is a programming language that integrate systems. According to [this](https://github.com/Homebrew/homebrew-core/issues/62911) post, it is recommended to install Python 3.8 and 3.9 and make it consistent.
```bash
brew install python@3.12
python3 --version 
pip3 --version
```

#### 4.2.4. [ffmpeg](https://ffmpeg.org/)
ffmpeg is a cross-platform solution to record, convert and stream audio and video. The installation may take several minutes.
```bash
brew install ffmpeg
```

#### 4.2.5. [XQuartz](https://www.xquartz.org/)
The XQuartz project is an open-source effort to develop a version of the [X.Org X Window System](https://www.x.org/wiki/) that runs on macOS.

 - Download and run https://github.com/XQuartz/XQuartz/releases/download/XQuartz-2.8.5/XQuartz-2.8.5.pkg
 - Will take around 320 MB of disk space   
 - Send XQuartz-2.8.5.dmg to Trash    


#### 4.2.6. [MacPorts](https://guide.macports.org/chunked/installing.macports.html)
Follow the instructions from https://guide.macports.org/chunked/installing.macports.html.

<!-- ```bash -->
<!-- curl -O https://distfiles.macports.org/MacPorts/MacPorts-2.10.7.tar.bz2 -->
<!-- tar xf MacPorts-2.10.7.tar.bz2 -->
<!-- cd MacPorts-2.10.7 -->
<!-- ./configure -->
<!-- make -->
<!-- sudo make install -->
<!-- cd ../; rm -rf MacPorts-2.10.7* -->
<!-- sudo nano /etc/paths -->
<!-- ``` -->
<!-- Add the following lines at the end of the file. -->
<!-- ```bash -->
<!-- /opt/local/bin -->
<!-- /opt/local/sbin -->
<!-- ``` -->
<!-- Use Ctrl+O and return to save the file and Ctrl+X to close nano editor. Reboot your terminal and use -->
<!-- ```bash -->
<!-- sudo port -v selfupdate -->
<!-- sudo port upgrade outdated -->
<!-- ``` -->

#### 4.2.7. [tcllib](https://ports.macports.org/port/tcllib/)

```bash
sudo port selfupdate && sudo port upgrade tcllib
sudo port install tcllib
```

#### 4.2.8. [MuseScore](https://musescore.org/)
MuseScore is an open source notation software.

 - Download and run https://musescore.org/en/download/musescore.dmg  
 - Drag MuseScore 4 to Applications folder   
 - Will take around 320 MB of disk space   
 - Unmount MuseScore-4.5.2 virtual disk and send MuseScore-Studio-4.5.2.251141402.dmg to Trash   


#### 4.2.9. [R](https://www.r-project.org)
R is a free software environment for statistical computing and graphics.  

 - Download and run the pkg file according to you architecture from https://cloud.r-project.org/bin/macosx/
 - Will take around 180 MB of disk space   


#### 4.2.10. [RStudio](https://posit.co/downloads)
RStudio is an Integrated Development Environment (IDE) for R.

 - Download and run https://download1.rstudio.org/electron/macos/RStudio-2025.05.0-496.dmg
 - Drag RStudio to Applications folder   
 - Will take around 770 MB of disk space   
 - Unmount RStudio virtual disk and send RStudio-2025.05.0-496.dmg to Trash   
 - Type `command + space 'rstudio'`   
 - Tools > Global Options... > Appearance > Merbivore (Restart required)   


#### 4.2.11. R packages
"Packages are the fundamental units of reproducible R code." [Hadley Wickham and Jennifer Bryan](https://r-pkgs.org/). Type `command + space 'terminal'`   
```bash
sudo R
```

Running R as super user paste the following, one line at a time.
```r
packs <- c('audio','reticulate','R.utils','seewave','tidyverse','tuneR','wrassp')
install.packages(packs, dep = TRUE)
update.packages(ask = FALSE)
devtools::install_github('egenn/music')
devtools::install_github('flujoo/gm')
```

#### 4.2.12. [Miniconda](https://docs.conda.io/en/latest/miniconda.html)
Miniconda is a free minimal installer for [conda](https://docs.conda.io/), an open source package, dependency and environment management system for any language—Python, R, Ruby, Lua, Scala, Java, JavaScript, C/ C++, FORTRAN and more, that runs on Windows, macOS and Linux. 

For 64-bit version use

```bash
cd ~/Downloads
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-MacOSX-x86_64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-MacOSX-x86_64.sh
```

For M1 version use

```bash
cd ~/Downloads
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-MacOSX-arm64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-MacOSX-arm64.sh
```

In order to continue the installation process, please review the license
agreement. Please, press ENTER to continue `ENTER`.

You can undo this by running `conda init --reverse $SHELL`? `yes`

Close and reopen terminal.

```bash
export PATH="~/miniconda3/bin:$PATH"
conda update -n base -c defaults conda
```

The following packages will be INSTALLED/REMOVED/UPDATED/DOWNGRADED:... Proceed ([y]/n)? `y`

```bash
conda create -n pyvoice python=3.12
```
The following (NEW) packages will be downloaded/INSTALLED:... Proceed ([y]/n)? `y`

Close and reopen terminal.

```bash
conda activate base
conda activate pyvoice
pip install -r https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt
```

## 5. Diarize

```{r}
# download
url0 <- 'https://github.com/filipezabala/voiceAudios/raw/main/wav/sherlock0.wav'
wavDir <- normalizePath(tempdir())
download.file(url0, paste0(wavDir, '/sherlock0.wav'), mode = 'wb')
```
Diarization can be performed to detect speaker segments (i.e., 'who spoke when').
```{r}
# diarize
voice::diarize(fromWav = wavDir, toRttm = wavDir, token = 'YOUR_TOKEN')
```

The `voice::diarize()` function creates Rich Transcription Time Marked (RTTM)[^rttm] files, space-delimited text files containing one turn per line defined by NIST - National Institute of Standards and Technology. The RTTM files can be read using `voice::read_rttm()`.

[^rttm]: See Appendix	C at https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf.

```{r}
# read_rttm
(rttm <- voice::read_rttm(wavDir))
```

Finally, the audio waves can be automatically segmented.
```{r}
# split audio wave
voice::splitw(fromWav = wavDir, fromRttm = wavDir, to = wavDir)
dir(wavDir, pattern = '.[Ww][Aa][Vv]$')
```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
ini0 <- Sys.time()

## ----eval=FALSE---------------------------------------------------------------
# # Development version from GitHub
# install.packages('devtools')
# devtools::install_github('filipezabala/voice')
# 
# # Stable version from CRAN
# install.packages('voice')

## ----message=FALSE, warning=FALSE---------------------------------------------
# packs
library(voice)
library(tidyverse)

# get path to audio file
wavDir <- list.files(system.file('extdata', package = 'wrassp'),
                     pattern = glob2rx('*.wav'), full.names = TRUE)

## ----message=FALSE, warning=FALSE---------------------------------------------
# minimal usage
M <- voice::extract_features(wavDir)
glimpse(M)

## ----message=FALSE, warning=FALSE---------------------------------------------
# creating Extended synthetic data
E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3), wav_path = wavDir)
E

# minimal usage
voice::tag(E)

# canonical data
voice::tag(E, groupBy = 'subject_id')

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# url0 <- 'https://github.com/filipezabala/voiceAudios/raw/refs/heads/main/wav/doremi.wav'
# download.file(url0, paste0(tempdir(), '/doremi.wav'), mode = 'wb')
# voice::embed_audio(url0) # See https://github.com/mccarthy-m-g/embedr for more details.

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# M <- voice::extract_features(tempdir())
# summary(M)

## ----message=FALSE, warning=FALSE, fig.width=7.5, fig.height=4, eval=FALSE----
# voice::piano_plot(M, 0) # f0
# voice::piano_plot(M, 0:1) # f0 + f1

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# (f0_spn <- voice::assign_notes(M, fmt = 0, min_points = 22, min_percentile = .85)) # f0
# (f1_spn <- voice::assign_notes(M, fmt = 1, min_points = 22, min_percentile = .85)) # f1

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# library(gm)
# line_0 <- gm::Line(as.character(f0_spn))
# m0 <- gm::Music() +
#   gm::Meter(4, 4) +
#   line_0
# gm::show(m0, to = c('score', 'audio'))

## ----message=FALSE, warning=FALSE, eval=FALSE---------------------------------
# line_0 <- gm::Line(as.character(f0_spn))
# line_1 <- gm::Line(as.character(f1_spn))
# m1 <- gm::Music() +
#   gm::Meter(4, 4) +
#   line_0 + line_1
# gm::show(m1, to = c('score', 'audio'))

## ----eval=FALSE---------------------------------------------------------------
# # download
# url0 <- 'https://github.com/filipezabala/voiceAudios/raw/main/wav/sherlock0.wav'
# wavDir <- normalizePath(tempdir())
# download.file(url0, paste0(wavDir, '/sherlock0.wav'), mode = 'wb')

## ----eval=FALSE---------------------------------------------------------------
# # diarize
# voice::diarize(fromWav = wavDir, toRttm = wavDir, token = 'YOUR_TOKEN')

## ----eval=FALSE---------------------------------------------------------------
# # read_rttm
# (rttm <- voice::read_rttm(wavDir))

## ----eval=FALSE---------------------------------------------------------------
# # split audio wave
# voice::splitw(fromWav = wavDir, fromRttm = wavDir, to = wavDir)
# dir(wavDir, pattern = '.[Ww][Aa][Vv]$')

#' Expand model
#'
#' @description Expand model given \code{y} and \code{x} variables.
#' @param y The Y variable.
#' @param x The X variables.
#' @param k Number of additive components.
#' @return A \code{char} vector containing the expanded models.
#' @examples
#' library(voice)
#'
#' expand_model('y', LETTERS[1:4], 1)
#' expand_model('y', LETTERS[1:4], 2)
#' expand_model('y', LETTERS[1:4], 3)
#' expand_model('y', LETTERS[1:4], 4)
#'
#' # multiple models using apply functions
#' nx <- 10 # number of X variables to be used
#' models <- lapply(1:nx, expand_model, y = 'y', x = LETTERS[1:nx])
#' names(models) <- 1:nx
#' models
#' sum(sapply(models, length)) # total of models
#' @export
expand_model <- function(y, x, k){

  y <- paste0(y, ' ~ ')
  comb <- utils::combn(x, k)

  if(k == 1){
    models <- unlist(lapply(y, paste0, comb))
  }

  if(k > 1){
    nc <- ncol(comb)
    pred <- vector('list', nc)
    for(j in 1:nc){
      pred[[j]] <- paste0(comb[,j], collapse = ' + ')
    }
    models <- unlist(lapply(y, paste0, pred))
  }
  return(models)
}

#'  Get bit rate
#'
#' @description Get bit rate from WAV file.
#' @param x Wave object from `tuneR::readWave`.
#' @return Integer indicating the bit rate from a WAV file.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern <- glob2rx('*.wav'), full.names = TRUE)
#'
#' rw <- tuneR::readWave(path2wav[1])
#' voice::get_bit(rw)
#'
#' rwl <- lapply(path2wav, tuneR::readWave)
#' sapply(rwl, voice::get_bit)
#' @export
get_bit <- function(x){
  return(x@bit)
}

#' These functions are sourced from the `embedr` package by Michael McCarthy, under
#' MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
#' This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
#' See https://github.com/mccarthy-m-g/embedr for more details.
#'
#' Example Media Files
#'
#' Example media files included with embedr.
#'
#'  - `mp3`: MP3 audio
#'  - `mp4`: MP4 video
#'  - `png`: PNG thumbnail
#'
#' @rdname media-files
#' @name media-files
#' @aliases mp3 mp4 png
#' @export mp3 mp4 png
delayedAssign('mp3', system.file("media", "audio-vignette.mp3", package = "voice"))
delayedAssign('mp4', system.file("media", "video-vignette.mp4", package = "voice"))
delayedAssign('png', system.file("media", "thumbnail-vignette.png", package = "voice"))

#' Convert SPN to ABC
#'
#' @description Convert SPN to standard octave.
#' @param x A vector containing a note in SPN (Scientific Pitch Notation).
#' @param to_lower Logical. Should the string be lower case? Default: \code{FALSE}.
#' @param spacing Logical. Should the strin return spaces between notes? Default: \code{TRUE}.
#' @examples
#' library(voice)
#' spn2abc('C4')
#' spn2abc('C5')
#' spn2abc('C4', to_lower = TRUE)
#' spn2abc(c('C4','D#7','E2'))
#' spn2abc(c('C4','D#7','E2'), to_lower = TRUE)
#' spn2abc(c('C4','D#7','E2'), spacing = FALSE)
#' spn2abc(c('C4','D#7','E2'), to_lower = TRUE, spacing = FALSE)
#' @references https://en.wikipedia.org/wiki/Scientific_pitch_notation
#'
#' https://en.wikipedia.org/wiki/ABC_notation
#' @export

#TODO: Program drop_rep_seq, #' @param drop_rep_seq Logical. Should sequences of same note be replaced by one single note? Default: \code{FALSE}.
spn2abc <- function(x, to_lower = FALSE, spacing = TRUE){
  freqs <- c(voice::notes_freq()$spn.lo, voice::notes_freq()$spn.hi[108])
  lev <- c('C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#', 'A', 'A#', 'B')
  octa <- base::strsplit(as.character(x), '[0-9]')
  if(to_lower){
    octa <- factor(unlist(tolower(octa)), levels = tolower(lev))
    if(spacing){
      return(octa)
    } else{
      return(paste0(octa, collapse = ''))
    }
  } else{
    octa <- factor(unlist(octa), levels = lev)
    if(spacing){
      return(octa)
    } else{
      return(paste0(octa, collapse = ''))
    }
  }
}

#'  Time duration
#'
#' @description Get time duration from a data frame in RTTM standard.
#' @param x A data frame in RTTM standard. See 'voice::read_rttm'.
#' @return Numeric vector containing the time duration in seconds.
#' @examples
#' library(voice)
#'
#' url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
#' download.file(url0, destfile = paste0(tempdir(), '/sherlock0.rttm'))
#'
#' rttm <- voice::read_rttm(tempdir())
#' (gtd <- voice::get_tdur(rttm$sherlock0.rttm))
#' class(gtd)
#' @export
get_tdur <- function(x){
  return(x$tdur)
}

#' Inperpolate data frames
#'
#' @description Interpolate data frames using multicore, compressing to \code{compact.to} fraction. May remove zeros.
#' @param x A data frame.
#' @param compact.to Proportion of remaining points after interpolation. If equals to 1 and keep.zeros = TRUE, the original vector is presented.
#' @param id The identification column. Default: \code{colname} of the first column of \code{x}.
#' @param colnum A \code{char} vector indicating the numeric colnames. If \code{NULL}, uses the columns of the \code{numeric} class.
#' @param drop.x Logical. Drop columns containing .x? Default: \code{TRUE}.
#' @param drop.zeros Logical. Drop repeated zeros or keep 1 zero per null set? Default: \code{FALSE}.
#' @param to.data.frame Logical. Should return a data frame? If \code{FALSE} returns a list. Default: \code{TRUE}.
#' @param round.off Number of decimal places of the interpolated \code{y}. Default: \code{NULL}.
#' @param weight Vector of weights with same length of \code{y}. Default: \code{NULL}.
#' @param mc.cores The number of cores to mclapply. Default: \code{1}.
#' @return A data frame of interpolated values with nrow near to \code{compact.to*length(x)}.
#' @importFrom dplyr %>%
#' @importFrom dplyr mutate_each
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' # getting Media data frame via lean call
#' M <- extract_features(dirname(path2wav), features = c('f0','fmt'),
#' mc.cores = 1, verbose = FALSE)
#'
#' \donttest{
#' (cM.df <- interp_df(M[,-(1:2)], 0.1, mc.cores = 1))
#' (cM.df2 <- interp_df(M[,-(1:2)], 0.1, drop.x = FALSE, mc.cores = 1))
#'
#' dim(M)
#' dim(cM.df)
#' dim(cM.df2)
#' (cM.list <- interp_df(M[,-(1:2)], 0.1, to.data.frame = FALSE, mc.cores = 1))
#' }
#' @seealso \code{interp}, \code{interp_mc}
#' @export
interp_df <- function(x, compact.to, id = colnames(x)[1], colnum = NULL,
                      drop.x = TRUE, drop.zeros = FALSE,
                      to.data.frame = TRUE, round.off = NULL, weight = NULL,
                      mc.cores = 1){
  ini <- Sys.time()

  # numeric columns
  if(is.null(colnum)){
    is.num <- unlist(lapply(x, class)) == 'numeric'
    colnum <- colnames(x[,is.num])
  }

  # non-numeric columns
  colnon <- setdiff(colnames(x), colnum)

  # split numeric columns by id
  snum <- split(x[,colnum], x[,id])

  # split non-numeric columns id
  snon <- split(x[,colnon], x[,id])

  # original lengths by id
  lv <- table(x[,id])

  # vector and length of distinct id's
  nlv <- names(lv)
  nid <- length(nlv)

  # compact lengths by id
  n <- ceiling(compact.to*lv)
  cs <- c(0, cumsum(n))

  # interolationg numeric variables
  cn.li <- lapply(snum, voice::interp_mc, compact.to = compact.to,
                  drop.zeros = drop.zeros, to.data.frame = to.data.frame,
                  round.off = round.off, weight = weight, mc.cores = mc.cores)

  # transforming in dataframe
  cn.df <- lapply(cn.li, as.data.frame)
  # cn.df <- bind_cols(cn.li)

  # compact list
  li <- vector('list', length = nid)
  for(i in 1:nid){
    index <- 1:n[i]
    cn.li[[i]] <- snon[[i]][index,]
    even <- seq(2, ncol(cn.df[[i]]), by = 2)
    index2 <- sort(union(even, even-!drop.x))
    cn.df.temp <- cn.df[[i]][,index2]
    if(is.numeric(cn.df.temp)){ # dealing with unitary features
      cn.df.temp <- dplyr::as_tibble(cn.df.temp)
      names(cn.df.temp) <- colnames(x[colnum])
    }
    cn.li[[i]] <- dplyr::bind_cols(cn.li[[i]], cn.df.temp)
    names(cn.li)[i] <- nlv[i]
  }
  if(drop.x){
    cn <- lapply(cn.li, colnames)[[1]]
    cn <- base::strsplit(cn, '[.]')
    cn <- as.data.frame(cn)[1,]
    cn.li <- lapply(cn.li, stats::setNames, cn)
  }

  # compact dataframe
  if(to.data.frame){
    cn.li <- do.call(dplyr::bind_rows, cn.li)
  }

  return(cn.li)
}

#' Split Wave
#'
#' @description Split WAV files either in \code{fromWav} directory or using (same names) RTTM files/subdirectories as guidance.
#' @param slice_duration The slices duration in seconds. Default: \code{NULL}, so RTTM file(s) must be given.
#' @param fromWav Either WAV file or directory containing WAV files.
#' @param fromRttm Either RTTM file or directory containing RTTM files. Default: \code{NULL}.
#' @param toSplit A directory to write generated files. Default: \code{NULL}.
#' @param autoDir Logical. Must the directories tree be created? Default: \code{FALSE}. See 'Details'.
#' @param subDir Logical. Must the splitted files be placed in subdirectories? Default: \code{FALSE}.
#' @param output Character string, the class of the object to return, either 'wave' or 'list'.
#' @param filesRange The desired range of directory files (default: \code{NULL}, i.e., all files). Must be TRUE only if \code{fromWav} is a directory.
#' @param full.names Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (default: \code{TRUE}) Used by \code{base::list.files}.
#' @param recursive Logical. Should the listing recursively into directories? (default: \code{FALSE}) Used by \code{base::list.files}. Inactive if \code{fromWav} is a file.
#' @param silence.gap The silence gap (in seconds) between adjacent words in a keyword. Rows with \code{tdur <= silence.gap} are removed. (default: \code{0.5})
#' @return Splited audio files according to the correspondent RTTM file(s). See '\code{voice::diarize}'.
#' @details When \code{autoDir = TRUE}, the following directories are created: \code{'../mp3'},\code{'../rttm'}, \code{'../split'} and \code{'../musicxml'}. Use \code{getwd()} to find the parent directory \code{'../'}.
#' @examples
#' \dontrun{
#' library(voice)
#'
#' # Using RTTM files as refernece
#' urlWav <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/wav/sherlock0.wav'
#' destWav <- paste0(tempdir(), '/sherlock0.wav')
#' download.file(urlWav, destfile = destWav)
#'
#' urlRttm <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
#' destRttm <- paste0(tempdir(), '/sherlock0.rttm')
#' download.file(urlRttm, destfile = destRttm)
#'
#' splitDir <- paste0(tempdir(), '/split')
#' dir.create(splitDir)
#'
#' # Splitting via RTTM
#' splitw(destWav, fromRttm = destRttm, toSplit = splitDir)
#' dir(splitDir)
#'
#' # Splitting with slice_duration of 5 seconds
#' splitw(destWav, slice_duration = 5, toSplit = splitDir)
#' splitw(destWav, slice_duration = 2, toSplit = splitDir)
#'
#' }
#' @seealso \code{voice::diarize}
#' @export
splitw <- function(fromWav,
                   slice_duration = NULL,
                   fromRttm = NULL,
                   toSplit = NULL,
                   autoDir = FALSE,
                   subDir = FALSE,
                   output = 'wave',
                   filesRange = NULL,
                   full.names = TRUE,
                   recursive = FALSE,
                   silence.gap = 0.5){

  # time processing
  pt0 <- proc.time()

  if(is.null(slice_duration) & is.null(fromRttm)){
    stop('Either splitby or fromRttm must be given!')
  }

  # checking if is either a file or a directory
  if(utils::file_test('-f', fromWav)){
    wavDir <- dirname(fromWav)
    wavFiles <- fromWav
  } else{
    wavDir <- fromWav
  }

  if(autoDir){
    ss <- unlist(strsplit(wavDir, '/'))
    parDir <- paste0(ss[-length(ss)], collapse ='/')
    mp3Dir <- paste0(parDir, '/mp3')
    rttmDir <- paste0(parDir, '/rttm')
    splitDir <- paste0(parDir, '/split')
    mxmlDir <- paste0(parDir, '/musicxml')
    # ifelse(!dir.exists(parDir), dir.create(parDir), 'Directory exists!')
    # ifelse(!dir.exists(wavDir), dir.create(wavDir), 'Directory exists!')
    ifelse(!dir.exists(mp3Dir), dir.create(mp3Dir), 'Directory exists!')
    ifelse(!dir.exists(rttmDir), dir.create(rttmDir), 'Directory exists!')
    ifelse(!dir.exists(splitDir), dir.create(splitDir), 'Directory exists!')
    ifelse(!dir.exists(mxmlDir), dir.create(mxmlDir), 'Directory exists!')
  }

  if(is.null(fromRttm) & exists('rttmDir')){
    fromRttm <- rttmDir
  }

  # rttm
  if(!is.null(fromRttm)){
    if(utils::file_test('-f', fromRttm)){
      rttmDir <- dirname(fromRttm)
      rttmFiles <- fromRttm
    } else{
      rttmDir <- fromRttm
      rttmFiles <- list.files(fromRttm, pattern = '[[:punct:]][rR][tT][tT][mM]$',
                              full.names = full.names, recursive = recursive)
    }
  }

  # split
  if(is.null(toSplit)){
    toSplit <- splitDir
  }

  # wav
  if(utils::file_test('-d', fromWav)){
    wavFiles <- list.files(fromWav, pattern = '[[:punct:]][wW][aA][vV]$',
                           full.names = full.names, recursive = recursive)
  }

  # filtering by fileRange
  if(!is.null(filesRange)){
    fullRange <- 1:length(wavFiles)
    filesRange <- base::intersect(fullRange, filesRange)
    wavFiles <- wavFiles[filesRange]
    get1 <- function(x){x[1]}
    splunctWav <- strsplit(basename(wavFiles), '[.]')
    splunctRttm <- strsplit(basename(rttmFiles), '[.]')
    nameWav <- sapply(splunctWav, get1)
    nameRttm <- sapply(splunctRttm, get1)
    rttmFiles <- rttmFiles[nameWav == nameRttm]
  }

  # number of WAV files to be extracted
  nWav <- length(wavFiles)

  # reading WAV
  audio <- sapply(wavFiles, tuneR::readWave)

  if(!is.null(fromRttm)){
    # reading RTTM
    rttm <- lapply(rttmFiles, utils::read.table)
    colnames <- c('type','file','chnl','tbeg','tdur',
                  'ortho','stype','name','conf','slat')
    rttm <- lapply(rttm, stats::setNames, colnames)

    # silence.gap
    keep.row <- function(x){
      kr <- x[x$tdur > silence.gap,]
      return(kr)
    }
    rttm <- lapply(rttm, keep.row)

  # time beginning, duration and ending
  tbeg <- lapply(rttm, voice::get_tbeg)
  tdur <- lapply(rttm, voice::get_tdur)
  tend <- Map('+', tbeg, tdur)
  } else{
    # getting duration in seconds
    audio_dur <- sapply(audio, voice::get_dur)

    # splitby is a scalar
    if(length(slice_duration) == 1){
      tbeg <- vector('list', 1)
      tbeg[[1]] <- seq(0, audio_dur, slice_duration)
      tdur <- vector('list', 1)
      tdur[[1]] <- c(rep(slice_duration, length(tbeg[[1]])-1),
                     audio_dur%%slice_duration)
      tend <- Map('+', tbeg, tdur)
    }
    # else{ # splitby is a vector
    #
    # }
  }

  # defining break points
  breaks <- Map('c', tbeg, tend)
  breaks <- lapply(breaks, sort)
  nbreaks <- sapply(breaks, length)
  index <- lapply(nbreaks, seq, from = 2, by = 2)

  # audio information
  freq <- sapply(audio, voice::get_samp.rate)

  # split.audio - Do parallel/vectorized?
  split.audio <- function(x, index, breaks, freq){
    splaudio <- vector('list', length(x))
    for(i in 1:length(x)){
      splaudio[[i]] <- vector('list', length(index[[i]]))
      for(j in index[[i]]){
        splaudio[[i]][[j/2]] <- x[[i]][(freq[i]*breaks[[i]][j-1]):(freq[i]*breaks[[i]][j])]
      }
    }
    return(splaudio)
  }
  sa <- split.audio(x=audio, index=index, breaks=breaks, freq=freq)

  # splitted audio information
  bitSpl <- lapply(unlist(sa), voice::get_bit)
  freqSpl <- lapply(unlist(sa), voice::get_samp.rate)
  leftSpl <- lapply(unlist(sa), voice::get_left)
  rightSpl <- lapply(unlist(sa), voice::get_right)

  # writing output as a list
  if(output == 'list'){
    if(!is.null(toSplit)){
      timest <- format(Sys.time(), "%Y-%m-%d_%H-%M-%S")
      voice::write_list(x = sa, path = paste0(toSplit,'/list_', timest, '.txt'))
    }
    return(sa)
  }

  # writing output as wave
  if(output == 'wave'){
    fileName <- do.call(rbind, strsplit(basename(wavFiles), '[.]'))
    fileNameSplit <- vector('list', length(nbreaks))
    # k <- 0
    audioWave <- vector('list', sum(nbreaks)/2)
    for(i in 1:length(nbreaks)){
      for(j in (nbreaks[i]/2)){
        fileNameSplit[[i]] <- paste0(fileName[i,1], '_split_', 1:j, '.',
               fileName[i,2])
      }
    }

    # save files
    if(subDir){
      pathNameSplit <- lapply(fileName[,1], function(x) paste0(toSplit, '/', x))
      dc <- function(x) ifelse(!dir.exists(x), dir.create(x), 'Directory exists!')
      lapply(pathNameSplit, dc)
      for(i in 1:length(pathNameSplit)){
        pathNameSplit[[i]] <- paste0(pathNameSplit[[i]], '/', fileNameSplit[[i]])
      }
    } else{
      pathNameSplit <- lapply(fileNameSplit, function(x) paste0(toSplit, '/', x))
    }
    for(i in 1:length(audio)){
      for(j in 1:length(sa[[i]])){
        try(tuneR::writeWave(sa[[i]][[j]], filename = pathNameSplit[[i]][j]))
      }
    }
  }

  # total time
  t0 <- proc.time()-pt0
  cat('TOTAL TIME', t0[3], 'SECONDS\n\n')
}

#' Extract audio features
#' @description Extracts features from WAV audio files.
#' @param x A vector containing either files or directories of audio files in WAV format.
#' @param features Vector of features to be extracted. (Default: \code{'f0','fmt','gain'}). Available features: \code{'f0','f0_mhs','f0_praat','fmt','fmt_praat','zcr','rms','gain','rfc','ac','cep','dft','css','lps','mfcc','df','pf','rf','rcf','rpf'}.
#' @param filesRange The desired range of directory files (Default: \code{NULL}, i.e., all files). Should only be used when all the WAV files are in the same folder.
#' @param sex \code{= <code>} set sex specific parameters where <code> = \code{'f'}[emale], \code{'m'}[ale] or \code{'u'}[nknown] (Default: \code{'u'}). Used as 'gender' by \code{wrassp::ksvF0}, \code{wrassp::forest} and \code{wrassp::mhsF0}.
#' @param windowShift \code{= <dur>} set analysis window shift to <dur>ation in ms (Default: \code{5.0}). Used by \code{wrassp::ksvF0}, \code{wrassp::forest}, \code{wrassp::mhsF0}, \code{wrassp::zcrana}, \code{wrassp::rfcana}, \code{wrassp::acfana}, \code{wrassp::cepstrum}, \code{wrassp::dftSpectrum}, \code{wrassp::cssSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param numFormants \code{= <num>} <num>ber of formants (Default: \code{8}). Used by \code{wrassp::forest}.
#' @param numcep Number of Mel-frequency cepstral coefficients (cepstra) to return (Default: \code{12}). Used by \code{tuneR::melfcc}.
#' @param dcttype Type of DCT used. \code{'t1'} or \code{'t2'}, \code{'t3'} for HTK \code{'t4'} for feacalc (Default: \code{'t2'}). Used by \code{tuneR::melfcc}.
#' @param fbtype Auditory frequency scale to use: \code{'mel'}, \code{'bark'}, \code{'htkmel'}, \code{'fcmel'} (Default: \code{'mel'}). Used by \code{tuneR::melfcc}.
#' @param resolution \code{= <freq>} set FFT length to the smallest value which results in a frequency resolution of <freq> Hz or better (Default: \code{40.0}). Used by \code{wrassp::cssSpectrum}, \code{wrassp::dftSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param usecmp Logical. Apply equal-loudness weighting and cube-root compression (PLP instead of LPC) (Default: \code{FALSE}). Used by \code{tuneR::melfcc}.
#' @param mc.cores Number of cores to be used in parallel processing. (Default: \code{1})
#' @param full.names Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (Default: \code{TRUE}) Used by \code{base::list.files}.
#' @param recursive Logical. Should the listing recursively into directories? (Default: \code{FALSE}) Used by \code{base::list.files}.
#' @param check.mono Logical. Check if the WAV file is mono. (Default: \code{TRUE})
#' @param stereo2mono (Experimental) Logical. Should files be converted from stereo to mono? (Default: \code{TRUE})
#' @param overwrite (Experimental) Logical. Should converted files be overwritten? If not, the file gets the suffix \code{_mono}. (Default: \code{FALSE})
#' @param freq Frequency in Hz to write the converted files when \code{stereo2mono=TRUE}. (Default: \code{44100})
#' @param round.to Number of decimal places to round to. (Default: \code{NULL})
#' @param verbose Logical. Should the running status be showed? (Default: \code{FALSE})
#' @param pycall Python call. See \url{https://github.com/filipezabala/voice} for details.
#' @return A Media data frame containing the selected features.
#' @details The feature 'df' corresponds to 'formant dispersion' (df2:df8) by
#' Fitch (1997), 'pf' to formant position' (pf1:pf8) by Puts, Apicella & Cárdena
#' (2011), 'rf' to 'formant removal' (rf1:rf8) by Zabala (2025), 'rcf' to
#' 'formant cumulated removal' (rcf2:rcf8) by Zabala (2025) and 'rpf' to
#' 'formant position removal' (rpf2:rpf8) by Zabala (2025).
#' The \code{'fmt_praat'} feature may take long time processing. The following
#' features may contain a variable number of columns: \code{'cep'}, \code{'dft'},
#' \code{'css'} and \code{'lps'}.
#' On Windows, this function may occasionally cause the RStudio session to crash.
#' @references Levinson N. (1946). The Wiener (root mean square) error criterion in filter design and prediction. Journal of Mathematics and Physics, 25(1-4), 261–278. (\doi{10.1002/SAPM1946251261})
#'
#' Durbin J. (1960). “The fitting of time-series models.” Revue de l’Institut International de Statistique, pp. 233–244. (\url{https://www.jstor.org/stable/1401322})
#'
#' Cooley J.W., Tukey J.W. (1965). “An algorithm for the machine calculation of complex Fourier series.” Mathematics of computation, 19(90), 297–301. (\url{https://www.ams.org/journals/mcom/1965-19-090/S0025-5718-1965-0178586-1/})
#'
#' Wasson D., Donaldson R. (1975). “Speech amplitude and zero crossings for automated identification of human speakers.” IEEE Transactions on Acoustics, Speech, and Signal Processing, 23(4), 390–392. (\url{https://ieeexplore.ieee.org/document/1162690})
#'
#' Allen J. (1977). “Short term spectral analysis, synthesis, and modification by discrete Fourier transform.” IEEE Transactions on Acoustics, Speech, and Signal Processing, 25(3), 235– 238. (\url{https://ieeexplore.ieee.org/document/1162950})
#'
#' Schäfer-Vincent K. (1982). "Significant points: Pitch period detection as a problem of segmentation." Phonetica, 39(4-5), 241–253. (\doi{10.1159/000261665} )
#'
#' Schäfer-Vincent K. (1983). "Pitch period detection and chaining: Method and evaluation." Phonetica, 40(3), 177–202. (\doi{10.1159/000261691})
#'
#' Ephraim Y., Malah D. (1984). “Speech enhancement using a minimum-mean square error short-time spectral amplitude estimator.” IEEE Transactions on acoustics, speech, and signal processing, 32(6), 1109–1121. (\url{https://ieeexplore.ieee.org/document/1164453})
#'
#' Delsarte P., Genin Y. (1986). “The split Levinson algorithm.” IEEE transactions on acoustics, speech, and signal processing, 34(3), 470–478. (\url{https://ieeexplore.ieee.org/document/1164830})
#'
#' Jackson J.C. (1995). "The Harmonic Sieve: A Novel Application of Fourier Analysis to Machine Learning Theory and Practice." Technical report, Carnegie-Mellon University Pittsburgh PA Schoo; of Computer Science.
#'
#' Fitch, W.T. (1997) "Vocal tract length and formant frequency dispersion correlate with body size in rhesus macaques." J. Acoust. Soc. Am. 102, 1213 – 1222. (\doi{10.1121/1.421048})
#'
#' Boersma P., van Heuven V. (2001). Praat, a system for doing phonetics by computer. Glot. Int., 5(9/10), 341–347. (\url{https://www.fon.hum.uva.nl/paul/papers/speakUnspeakPraat_glot2001.pdf})
#'
#' Ellis DPW (2005). “PLP and RASTA (and MFCC, and inversion) in Matlab.” Online web resource.
#'
#' Puts, D.A., Apicella, C.L., Cardenas, R.A. (2012) "Masculine voices signal men's threat potential in forager and industrial societies." Proc. R. Soc. B Biol. Sci. 279, 601–609. (\doi{10.1098/rspb.2011.0829})
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' # minimal usage
#' M1 <- extract_features(path2wav)
#' M2 <- extract_features(dirname(path2wav))
#' identical(M1,M2)
#' table(basename(M1$wav_path))
#'
#' # limiting filesRange
#' M3 <- extract_features(path2wav, filesRange = 3:6)
#' table(basename(M3$wav_path))
#' @export
extract_features <- function(x,
                             features = c('f0', 'fmt', 'gain'), # F0, formants and gain
                             filesRange = NULL,
                             sex = 'u',
                             windowShift = 10,
                             numFormants = 8,
                             numcep = 12,
                             dcttype = c('t2', 't1', 't3', 't4'),
                             fbtype = c('mel', 'htkmel', 'fcmel', 'bark'),
                             resolution = 40,
                             usecmp = FALSE,
                             mc.cores = 1,
                             full.names = TRUE,
                             recursive = FALSE,
                             check.mono = FALSE,
                             stereo2mono = FALSE,
                             overwrite = FALSE,
                             freq = 44100,
                             round.to = NULL,
                             verbose = FALSE,
                             pycall = '~/miniconda3/envs/pyvoice/bin/python'){

  # time processing
  pt0 <- proc.time()

  # checking if x is composed by files or directories
  if(utils::file_test('-f', x[1])){
    wavDir <- lapply(x, dirname)
    wavDir <- do.call(rbind, wavDir)
    wavDir <- unique(wavDir)
    wavFiles <- x
  } else{
    wavDir <- unique(x)
    nWavDir <- length(wavDir)
    wavFiles <- lapply(wavDir, list.files, pattern = '[[:punct:]][wW][aA][vV]$',
                       full.names = full.names, recursive = recursive)
    wavFiles <- do.call(rbind, as.list(unlist(wavFiles)))
    # wavFiles <- do.call(rbind, wavFiles)
  }

  # filtering by filesRange
  if(!is.null(filesRange)){
    fullRange <- 1:length(wavFiles)
    filesRange <- base::intersect(fullRange, filesRange)
    wavFiles <- wavFiles[filesRange]
  }

  # number of wav files to be extracted
  nWav <- length(wavFiles)

  # checking mono/stereo (GO PARALLEL?)
  if(check.mono){
    mono <- sapply(wavFiles, voice::is_mono)
    which.stereo <- which(!mono)
    ns <- length(which.stereo)
    if(sum(which.stereo)){
      if(verbose){
        cat('The following', ns, 'audio files are stereo and must be converted to mono: \n',
            paste0(names(mono[which.stereo]), sep = '\n'), '\n')
      }
      if(stereo2mono){
        audio <- sapply(wavFiles[which.stereo], tuneR::readWave)
        new.mono <- sapply(audio, tuneR::mono)
        n <- sapply(wavFiles[which.stereo], nchar)
        for(i in 1:ns){
          if(overwrite){
            seewave::savewav(new.mono[[i]], f=freq,
                             filename = wavFiles[which.stereo][i])
          }
          else if(!overwrite){
            new.name <- substr(wavFiles[which.stereo][i], 1, n[i]-4)
            new.name <- paste0(new.name, '_mono.wav')
            seewave::savewav(new.mono[[i]], f=freq, filename = new.name)
          }
        }
      }
    }
  }

  # checking lpa features
  f <- vector(length = 3)
  f[1] <- sum(features == 'rms')
  f[2] <- sum(features == 'gain')
  f[3] <- sum(features == 'rfc')

  # list of features
  features2 <- base::setdiff(features, c('df','pf','rf','rcf','rpf'))
  nFe <- length(features2)
  ifelse(sum(f) == 0, ind1 <- 0, ind1 <- 1)
  ifelse(sum(features == 'mfcc'), ind2 <- 0, ind2 <- 1)
  features.list.temp <- vector('list', nFe-sum(f)+ind1+ind2)
  features.list <- vector('list', nFe)
  length.list <- vector('list', nFe)
  i.temp <- 0
  i <- 0

  # 1. F0 analysis of the signal via Schafer-Vincent (1983), wrassp::ksvF0
  if('f0' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::ksvF0,
                                                       gender = sex,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'f0'
    names(features.list)[i] <- 'f0'
    names(length.list)[i] <- 'f0'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 2. F0 analysis of the signal via Jackson (1995) [Michel’s (M)odified (H)armonic (S)ieve algorithm], wrassp::mhsF0
  if('f0_mhs' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::mhsF0,
                                                       toFile = FALSE,
                                                       gender = sex,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'f0_mhs'
    names(features.list)[i] <- 'f0_mhs'
    names(length.list)[i] <- 'f0_mhs'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }


  # 3. F0 analysis of the signal via Boersma (1993)
  if('f0_praat' %in% features){
    i.temp <- i.temp+1
    i <- i+1

    # setting environment
    reticulate::use_condaenv(pycall, required = TRUE)
    parselmouth <- reticulate::import('parselmouth')

    # setting structure
    names(features.list.temp)[i.temp] <- 'f0_praat'
    names(features.list)[i] <- 'f0_praat'
    names(length.list)[i] <- 'f0_praat'
    features.list.temp[[i.temp]] <- vector('list', nWav)

    # F0 extraction
    for(j in 1:nWav){
      snd <- parselmouth$Sound(wavFiles[j])
      pitch <- snd$to_pitch(time_step = windowShift/1000)
      interval <- seq(pitch$start_time, pitch$end_time, windowShift/1000)

      f0_praat_temp <- sapply(interval, pitch$get_value_at_time)
      f0_praat_temp[is.nan(f0_praat_temp)] <- NA
      features.list.temp[[i.temp]][[j]] <- f0_praat_temp
    }

    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]], length))
  }

  # 4. Formant estimation (f1:f8) via wrassp::forest
  if('fmt' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::forest,
                                                       gender = sex,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       numFormants = numFormants,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'fmt'
    names(features.list)[i] <- 'fmt'
    names(length.list)[i] <- 'fmt'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 5. Formant estimation (f1_praat:f8_praat) via Burg algorithm
  if('fmt_praat' %in% features){
    i.temp <- i.temp+1
    i <- i+1

    # setting environment
    reticulate::use_condaenv(pycall, required = TRUE)
    parselmouth <- reticulate::import('parselmouth')

    # setting structure
    names(features.list.temp)[i.temp] <- 'fmt_praat'
    names(features.list)[i] <- 'fmt_praat'
    names(length.list)[i] <- 'fmt_praat'
    features.list.temp[[i.temp]] <- vector('list', nWav)

    # Formant extraction
    for(j in 1:nWav){
      # formants extraction
      snd <- parselmouth$Sound(wavFiles[j])
      fmt <- snd$to_formant_burg(time_step = windowShift/1000,
                                 max_number_of_formants = numFormants)
      interval <- seq(fmt$start_time, fmt$end_time, windowShift/1000)

      #TODO: build with apply
      fmt_praat_temp <- matrix(nrow = length(interval), ncol = numFormants)
      colnames(fmt_praat_temp) <- paste0('fmt_', 1:numFormants)
      for(k in 1:numFormants){
        for(l in 1:length(interval)){
          fmt_praat_temp[l,k] <- fmt$get_value_at_time(formant_number = as.integer(k),
                                                       time = interval[l])
        }
      }
      fmt_praat_temp[is.nan(fmt_praat_temp)] <- NA
      features.list.temp[[i.temp]][[j]] <- fmt_praat_temp
    }

    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]], nrow))
  }

  # 6. Analysis of the averages of the short-term positive and negative (Z)ero-(C)rossing (R)ates
  if('zcr' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::zcrana,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'zcr'
    names(features.list)[i] <- 'zcr'
    names(length.list)[i] <- 'zcr'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }


  # 7. (L)inear (P)rediction (A)nalysis [rms, gain, rfc]
  if('rms' %in% features | 'gain' %in% features | 'rfc' %in% features){
    i.temp <- i.temp+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::rfcana,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'lpa'

    if('rms' %in% features){
      i <- i+1
      names(features.list)[i] <- 'rms'
      names(length.list)[i] <- 'rms'
      length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                        wrassp::numRecs.AsspDataObj))
    }

    if('gain' %in% features){
      i <- i+1
      names(features.list)[i] <- 'gain'
      names(length.list)[i] <- 'gain'
      # features.list[[i]] <- dplyr::tibble()
      length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                        wrassp::numRecs.AsspDataObj))
    }

    if('rfc' %in% features){
      i <- i+1
      names(features.list)[i] <- 'rfc'
      names(length.list)[i] <- 'rfc'
      length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                        wrassp::numRecs.AsspDataObj))
    }
  }

  # 8. Analysis of short-term (A)uto(C)orrelation function
  if('ac' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::acfana,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'ac'
    names(features.list)[i] <- 'ac'
    names(length.list)[i] <- 'ac'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 9. Short-term (CEP)stral analysis
  if('cep' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::cepstrum,
                                                       toFile = FALSE,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'cep'
    names(features.list)[i] <- 'cep'
    names(length.list)[i] <- 'cep'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 10. Short-term (DFT) spectral analysis
  if('dft' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::dftSpectrum,
                                                       toFile = FALSE,
                                                       resolution = resolution,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'dft'
    names(features.list)[i] <- 'dft'
    names(length.list)[i] <- 'dft'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 11. (C)epstral (S)moothed version of dft(S)pectrum
  if('css' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::cssSpectrum,
                                                       toFile = FALSE,
                                                       resolution = resolution,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'css'
    names(features.list)[i] <- 'css'
    names(length.list)[i] <- 'css'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 12. (L)inear (P)redictive (S)moothed version of dftSpectrum
  if('lps' %in% features){
    i.temp <- i.temp+1
    i <- i+1
    features.list.temp[[i.temp]] <- parallel::mclapply(wavFiles,
                                                       wrassp::lpsSpectrum,
                                                       toFile = FALSE,
                                                       resolution = resolution,
                                                       windowShift = windowShift,
                                                       mc.cores = mc.cores)
    names(features.list.temp)[i.temp] <- 'lps'
    names(features.list)[i] <- 'lps'
    names(length.list)[i] <- 'lps'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]],
                                      wrassp::numRecs.AsspDataObj))
  }

  # 13. Mel-Frequency Cepstral Coefficients (MFCC)
  rWave <- parallel::mclapply(wavFiles, tuneR::readWave, mc.cores = mc.cores)
  i.temp <- i.temp+1
  features.list.temp[[i.temp]] <- parallel::mclapply(rWave, tuneR::melfcc,
                                                     wintime = windowShift/1000,
                                                     hoptime = windowShift/1000,
                                                     dcttype = dcttype,
                                                     numcep = numcep,
                                                     fbtype = fbtype,
                                                     usecmp = usecmp,
                                                     mc.cores = mc.cores)
  names(features.list.temp)[i.temp] <- 'mfcc'

  if('mfcc' %in% features){
    i <- i+1
    names(features.list)[i] <- 'mfcc'
    names(length.list)[i] <- 'mfcc'
    length.list[[i]] <- unlist(lapply(features.list.temp[[i.temp]], nrow))
  }

  # creating tibbles at features.list
  features.list <- lapply(features.list, dplyr::tibble)

  # minimum length
  n_min <- apply(dplyr::bind_rows(length.list), 1, min)

  # concatenating
  for(j in 1:nWav){ # upgrade: use bind_rows, foreach

    # time processing
    pt1 <- proc.time()

    if('f0' %in% features){
      f0_temp <- as.matrix(features.list.temp$f0[[j]]$F0[1:n_min[j]], ncol = 1)
      features.list$f0 <- rbind(features.list$f0, f0_temp)
    }

    if('f0_mhs' %in% features){
      f0_mhs_temp <- as.matrix(features.list.temp$f0_mhs[[j]]$pitch[1:n_min[j],],
                               ncol = 1)
      features.list$f0_mhs <- rbind(features.list$f0_mhs, f0_mhs_temp)
    }

    if('f0_praat' %in% features){
      f0_praat_temp <- as.matrix(features.list.temp$f0_praat[[j]][1:n_min[j]],
                                 ncol = 1)
      features.list$f0_praat <- rbind(features.list$f0_praat, f0_praat_temp)
    }

    if('fmt' %in% features){
      fmt_temp <- as.matrix(features.list.temp$fmt[[j]]$fm[1:n_min[j],],
                           ncol = numFormants)
      features.list$fmt <- rbind(features.list$fmt, fmt_temp)
    }

    if('fmt_praat' %in% features){
      fmt_praat_temp <- as.matrix(features.list.temp$fmt_praat[[j]][1:n_min[j],],
                                  ncol = numFormants)
      features.list$fmt_praat <- rbind(features.list$fmt_praat, fmt_praat_temp)
    }

    if('zcr' %in% features){
      zcr_temp <- as.matrix(features.list.temp$zcr[[j]]$zcr[1:n_min[j],],
                            ncol = 1)
      features.list$zcr <- rbind(features.list$zcr, zcr_temp)
    }

    if('rms' %in% features){
      rms_temp <- as.matrix(features.list.temp$lpa[[j]]$rms[1:n_min[j],],
                            ncol = 1)
      features.list$rms <- rbind(features.list$rms, rms_temp)
    }

    if('gain' %in% features){
      gain_temp <- as.matrix(features.list.temp$lpa[[j]]$gain[1:n_min[j],],
                             ncol = 1)
      features.list$gain <- rbind(features.list$gain, gain_temp)
    }

    if('rfc' %in% features){
      rfc_temp <- as.matrix(features.list.temp$lpa[[j]]$rfc[1:n_min[j],],
                            ncol = 19)
      features.list$rfc <- rbind(features.list$rfc, rfc_temp)
    }

    if('ac' %in% features){
      ac_temp <- as.matrix(features.list.temp$ac[[j]]$acf[1:n_min[j],],
                           ncol = 20)
      features.list$ac <- rbind(features.list$ac, ac_temp)
    }

    if('cep' %in% features){
      cep_temp <- as.matrix(features.list.temp$cep[[j]]$cep[1:n_min[j],],
                            ncol = 257)
      features.list$cep <- rbind(features.list$cep, cep_temp)
    }

    if('dft' %in% features){
      dft_temp <- as.matrix(features.list.temp$dft[[j]]$dft[1:n_min[j],],
                            ncol = 257)
      features.list$dft <- rbind(features.list$dft, dft_temp)
    }

    if('css' %in% features){
      css_temp <- as.matrix(features.list.temp$css[[j]]$css[1:n_min[j],],
                            ncol = 257)
      features.list$css <- rbind(features.list$css, css_temp)
    }

    if('lps' %in% features){
      lps_temp <- as.matrix(features.list.temp$lps[[j]]$lps[1:n_min[j],],
                            ncol = 257)
      features.list$lps <- rbind(features.list$lps, lps_temp)
    }

    if('mfcc' %in% features){
      features.list$mfcc <- rbind(features.list$mfcc,
                                  features.list.temp$mfcc[[j]][1:n_min[j],])
    }

    if(verbose){
      cat('PROGRESS', paste0(round(j/nWav*100,2),'%'), '\n')
    }
    t1 <- proc.time()-pt1
    if(verbose){
      cat('FILE', j, 'OF', nWav, '|', t1[3], 'SECONDS\n\n')
    }
  }

  # id, using smaller length: n_min
  id <- tibble::enframe(rep(wavFiles, n_min), value = 'wav_path',
                        name = NULL)

  # colnames
  if('f0' %in% features){
    colnames(features.list$f0) <- 'f0'
  }

  if('f0_mhs' %in% features){
    colnames(features.list$f0_mhs) <- paste0('f0_mhs')
  }

  if('f0_praat' %in% features){
    colnames(features.list$f0_praat) <- paste0('f0_praat')
  }

  if('fmt' %in% features){
    colnames(features.list$fmt) <- paste0('f', 1:ncol(features.list$fmt))
  }

  if('fmt_praat' %in% features){
    colnames(features.list$fmt_praat) <- paste0('f', 1:ncol(features.list$fmt_praat), '_praat')
  }

  if('zcr' %in% features){
    colnames(features.list$zcr) <- paste0('zcr', 1:ncol(features.list$zcr))
  }

  if('rms' %in% features){
    colnames(features.list$rms) <- paste0('rms')
  }

  if('gain' %in% features){
    colnames(features.list$gain) <- paste0('gain')
  }

  if('rfc' %in% features){
    colnames(features.list$rfc) <- paste0('rfc', 1:ncol(features.list$rfc))
  }

  if('ac' %in% features){
    colnames(features.list$ac) <- paste0('acf', 1:ncol(features.list$ac))
  }

  if('cep' %in% features){
    colnames(features.list$cep) <- paste0('cep', 1:ncol(features.list$cep))
  }

  if('dft' %in% features){
    colnames(features.list$dft) <- paste0('dft', 1:ncol(features.list$dft))
  }

  if('css' %in% features){
    colnames(features.list$css) <- paste0('css', 1:ncol(features.list$css))
  }

  if('lps' %in% features){
    colnames(features.list$lps) <- paste0('lps', 1:ncol(features.list$lps))
  }

  if('mfcc' %in% features){
    colnames(features.list$mfcc) <- paste0('mfcc', 1:ncol(features.list$mfcc))
  }

  # as data frame
  dat <- dplyr::bind_cols(id, features.list)

  # rounding
  if(!is.null(round.to)){
    dat[-1] <- round(dat[-1], round.to)
  }

  # replacing 0 by NA
  dat[-1][sapply(dat[-1], R.utils::isZero)] <- NA

  # 14. Df - Formant Dispersion by Fitch (1997)
  if('f0' %in% features & 'fmt' %in% features & 'df' %in% features){
    if(numFormants >= 2) {dat$df2 <- (dat$f2-dat$f1)/1}
    if(numFormants >= 3) {dat$df3 <- (dat$f3-dat$f1)/2}
    if(numFormants >= 4) {dat$df4 <- (dat$f4-dat$f1)/3}
    if(numFormants >= 5) {dat$df5 <- (dat$f5-dat$f1)/4}
    if(numFormants >= 6) {dat$df6 <- (dat$f6-dat$f1)/5}
    if(numFormants >= 7) {dat$df7 <- (dat$f7-dat$f1)/6}
    if(numFormants >= 8) {dat$df8 <- (dat$f8-dat$f1)/7}
  }

  # TODO: in scale check if columns are not constant (degenerated random variables)
  # Scaling
  if('f0' %in% features & 'fmt' %in% features &
     ('pf' %in% features | 'rf' %in% features |
      'rcf' %in% features | 'rpf' %in% features)){
    cn <- paste0('f', 0:numFormants)
    f_sc <- sapply(dat[,cn], scale)
  }

  # 15. Pf - Formant Position by Puts, Apicella & Cárdenas (2011)
  if('f0' %in% features & 'fmt' %in% features & 'pf' %in% features){
    if(numFormants >= 1) {dat$pf1 <- f_sc[,'f1']}
    if(numFormants >= 2) {dat$pf2 <- rowMeans(f_sc[,c('f1','f2')], na.rm = TRUE)}
    if(numFormants >= 3) {dat$pf3 <- rowMeans(f_sc[,c('f1','f2','f3')], na.rm = TRUE)}
    if(numFormants >= 4) {dat$pf4 <- rowMeans(f_sc[,c('f1','f2','f3','f4')], na.rm = TRUE)}
    if(numFormants >= 5) {dat$pf5 <- rowMeans(f_sc[,c('f1','f2','f3','f4','f5')], na.rm = TRUE)}
    if(numFormants >= 6) {dat$pf6 <- rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6')], na.rm = TRUE)}
    if(numFormants >= 7) {dat$pf7 <- rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6','f7')], na.rm = TRUE)}
    if(numFormants >= 8) {dat$pf8 <- rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6','f7','f8')], na.rm = TRUE)}
  }

  # 16. Rf - Formant Removal by Zabala (2023)
  if('f0' %in% features & 'fmt' %in% features & 'rf' %in% features){
    if(numFormants >= 1) {dat$rf1 <- f_sc[,'f0']-f_sc[,'f1']}
    if(numFormants >= 2) {dat$rf2 <- f_sc[,'f0']-f_sc[,'f2']}
    if(numFormants >= 3) {dat$rf3 <- f_sc[,'f0']-f_sc[,'f3']}
    if(numFormants >= 4) {dat$rf4 <- f_sc[,'f0']-f_sc[,'f4']}
    if(numFormants >= 5) {dat$rf5 <- f_sc[,'f0']-f_sc[,'f5']}
    if(numFormants >= 6) {dat$rf6 <- f_sc[,'f0']-f_sc[,'f6']}
    if(numFormants >= 7) {dat$rf7 <- f_sc[,'f0']-f_sc[,'f7']}
    if(numFormants >= 8) {dat$rf8 <- f_sc[,'f0']-f_sc[,'f8']}
  }

  # 17. RCf - Formant Cumulated Removal by Zabala (2023)
  if('f0' %in% features & 'fmt' %in% features & 'rcf' %in% features){
    # if(numFormants >= 1) {dat$rcf1 <- f_sc[,'f0']-f_sc[,'f1']} # equivalent to Rf1 and RPf1
    if(numFormants >= 2) {dat$rcf2 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2')], na.rm = TRUE)}
    if(numFormants >= 3) {dat$rcf3 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3')], na.rm = TRUE)}
    if(numFormants >= 4) {dat$rcf4 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3','f4')], na.rm = TRUE)}
    if(numFormants >= 5) {dat$rcf5 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3','f4','f5')], na.rm = TRUE)}
    if(numFormants >= 6) {dat$rcf6 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3','f4','f5','f6')], na.rm = TRUE)}
    if(numFormants >= 7) {dat$rcf7 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3','f4','f5','f6','f7')], na.rm = TRUE)}
    if(numFormants >= 8) {dat$rcf8 <- f_sc[,'f0']-rowSums(f_sc[,c('f1','f2','f3','f4','f5','f6','f7','f8')], na.rm = TRUE)}
  }

  # 18. RPf - Formant Position Removal by Zabala (2023)
  if('f0' %in% features & 'fmt' %in% features & 'rpf' %in% features){
    # if(numFormants >= 1) {dat$rpf1 <- f_sc[,'f0']-dat$pf1} # equivalent to Rf1 and RCf1
    if(numFormants >= 2) {dat$rpf2 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2')], na.rm = TRUE)}
    if(numFormants >= 3) {dat$rpf3 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3')], na.rm = TRUE)}
    if(numFormants >= 4) {dat$rpf4 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3','f4')], na.rm = TRUE)}
    if(numFormants >= 5) {dat$rpf5 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3','f4','f5')], na.rm = TRUE)}
    if(numFormants >= 6) {dat$rpf6 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6')], na.rm = TRUE)}
    if(numFormants >= 7) {dat$rpf7 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6','f7')], na.rm = TRUE)}
    if(numFormants >= 8) {dat$rpf8 <- f_sc[,'f0']-rowMeans(f_sc[,c('f1','f2','f3','f4','f5','f6','f7','f8')], na.rm = TRUE)}
  }

  # creating ids
  idf <- lapply(n_min, seq, 1)
  dat <- dplyr::bind_cols(section_seq = 1:nrow(dat),
                          section_seq_file = unlist(lapply(idf, rev)), dat)

  # total time
  t0 <- proc.time()-pt0
  if(verbose){
    cat('TOTAL TIME', t0[3], 'SECONDS\n\n')
  }

  # return dat
  return(dat)
}

#' Duration of sequences
#'
#' @param x A vector containing symbols and \code{NA}.
#' @param windowShift Window shift to duration in ms (default: 5.0).
#' @return A data frame with duration in number of lines/ocurrences (\code{dur_line}), milliseconds (\code{dur_ms}) and proportional (\code{dur_prop}).
#' @examples
#' library(voice)
#' duration(letters)
#' duration(c('a','a','a',letters,'z'))
#'
#' nts <- c('NA','NA','A3','A3','A3','A3','A#3','B3','B3','C4','C4','C4','C4',
#' 'C4','C4','C#4','C4','C4','C4','B3','A#3','NA','NA','NA','NA','NA','NA','NA',
#' 'NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','D4','D4','D4','C#4',
#' 'C#4','C#4','C4','C4','B3','B3','A#3','A#3','A3','A3','G3','G#3','G3','F#3')
#' duration(nts)
#' @export
duration <- function(x, windowShift = 5){
  if(!is.factor(x)){
    x <- factor(x, levels = unique(x))
  }
  if(sum(is.na(x))){
    x <- factor(x, levels = c(levels(x), NA), exclude = NULL)
  }
  n <- length(x)
  m <- match(x,x)
  d <- diff(m)
  dur_line <- as.numeric(table(cumsum(abs(d))))

  if(x[1] != x[2]){
    dur_line <- c(1, dur_line)
  } else {
    dur_line[1] <- dur_line[1]+1
    }

  pos <- m[d != 0]
  if(d[1] == 0){
    pos <- c(pos, n)
  }

  note <- factor(x[pos], levels = levels(x))
  dur_ms <- dur_line*windowShift
  dur_prop <- dur_line/sum(dur_line)
  dur <- data.frame(note = note, dur_line = dur_line,
                    dur_ms = dur_ms, dur_prop = dur_prop)

  return(dur)
}

#' These functions are sourced from the `embedr` package by Michael McCarthy, under
#' MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
#' This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
#' See https://github.com/mccarthy-m-g/embedr for more details.
#'
#' Check if URL exists
#'
#' Given a character string, returns a logical vector indicating
#' whether a request for a specific URL responds without error.
#'
#' @param x A character vector.
#' @return `TRUE` if the URL responds without error, otherwise `FALSE`.
url.exists <- function(x) {
  tryCatch(!httr::http_error(x), error=function(e) FALSE)
}

#' Match string for URL prefix
#'
#' Given a character vector, returns a logical vector indicating
#' which elements have a URL scheme.
#'
#' @param x A character vector.
is.url <- function(x) {
  grepl("www.|http://|https://|ftp://|file://", x)
}


#' Return strings without a URL scheme
#'
#' Given a character vector, returns a logical indicating whether the
#' paths in the vector point to existing local files.
#'
#' @param x A character vector.
is.local <- function(x) {
  # check if paths have a URL scheme
  paths <- is.url(x)
  # name paths
  names(paths) <- x
  # remove paths with a URL scheme
  paths <- paths[(paths %in% FALSE)]
  # check which paths exist
  paths.exist <- file.exists(names(paths))
  # name path.exists
  names(paths.exist) <- names(paths)
  # return result or error
  if (length(paths) == 0) {
    # default to NULL if no path names without a URL scheme are present
    NULL
  } else if (!all(paths.exist)) {
    # remove paths that exist
    paths.exist <- paths.exist[(paths.exist %in% FALSE)]
    # notify user of the paths causing the error
    if (length(paths.exist) > 1) {
      stop("The files: ", paste(names(paths.exist), sep = ", ",
                                collapse = ", "), " do not exist. Please use valid filepaths.")
    } else {
      stop("The file: ", paste0(names(paths.exist)), " does not exist. ",
           "Please use a valid filepath.")
    }
  } else all(file.exists(names(paths))) # return TRUE
}

#' Return strings with a URL scheme
#'
#' Given a character vector, returns a logical indicating whether the
#' URLs in the vector respond without error.
#'
#' @param x A character vector.
is.hosted <- function(x) {
  # check if paths have a URL scheme
  paths <- is.url(x)
  # name paths
  names(paths) <- x
  # remove paths with a URL scheme
  paths <- paths[!(paths %in% FALSE)]
  # check which paths exist
  paths.exist <- sapply(names(paths), url.exists)
  # name path.exists
  names(paths.exist) <- names(paths)
  # return result or error
  if (length(paths) == 0) {
    # default to NULL if no path names without a URL scheme are present
    NULL
  } else if (!all(paths.exist)) {
    # remove paths that exist
    paths.exist <- paths.exist[(paths.exist %in% FALSE)]
    # notify user of the paths causing the error
    if (length(paths.exist) > 1) {
      stop("The URLs: ", paste(names(paths.exist), sep = ", ",
                               collapse = ", "), " do not exist. Please use valid URLs.")
    } else {
      stop("The URL: ", paste0(names(paths.exist)), " does not exist. ",
           "Please use a valid URL.")
    }
  } else all(sapply(names(paths), url.exists)) # return TRUE
}

#' Match string for audio suffix
#'
#' Given a character vector, returns a logical vector indicating
#' which elements have a valid audio file extension.
#'
#' @param x A character vector.
is.audio <- function(x) {
  # check if paths have a valid media file extension
  paths <- grepl(".mp3|.ogg|.wav", x)
  # name paths
  names(paths) <- x
  # check if all paths have valid media file extensions
  if (!all(grepl(".mp3|.ogg|.wav", x))) {
    # remove paths with a valid media file extension
    paths <- paths[(paths %in% FALSE)]
    # notify user of the paths causing the error
    if (length(paths) > 1) {
      stop(paste(names(paths), sep = ", ", collapse = ", "),
           " do not end with a valid audio file extension; ",
           "valid extensions are .mp3, .ogg, and .wav.")
    } else {
      stop(names(paths), " does not end with a valid audio file extension; ",
           "valid extensions are .mp3, .ogg, and .wav.")
    }
  } else all(grepl(".mp3|.ogg|.wav", x)) # return TRUE
}

#' Match string for video suffix
#'
#' Given a character vector, returns a logical vector indicating
#' which elements have a valid video file extension.
#'
#' @param x A character vector.
is.video <- function(x) {
  # check if paths have a valid media file extension
  paths <- grepl(".mp4|.webm|.ogg", x)
  # name paths
  names(paths) <- x
  # check if all paths have valid media file extensions
  if (!all(grepl(".mp4|.webm|.ogg", x))) {
    # remove paths with a valid media file extension
    paths <- paths[(paths %in% FALSE)]
    # notify user of the paths causing the error
    if (length(paths) > 1) {
      stop(paste(names(paths), sep = ", ", collapse = ", "),
           " do not end with a valid video file extension; ",
           "valid extensions are .mp4, .webm, and .ogg.")
    } else {
      stop(names(paths), " does not end with a valid video file extension; ",
           "valid extensions are .mp4, .webm, and .ogg.")
    }
  } else all(grepl(".mp4|.webm|.ogg", x)) # return TRUE
}

#' Compress zeros.
#'
#' @description Transforms \code{n} sets of \code{m>n} zeros (alternated with sets of non zeros) into \code{n} sets of \code{n} zeros.
#' @param y A vector or time series.
#' @return Vector with n zeros.
#' @examples
#' library(voice)
#'
#' (v0 <- c(1:20,rep(0,10)))
#' (r0 <- rm0(v0))
#' length(v0)
#' length(r0)
#' sum(v0 == 0)
#'
#' (v1 <- c(rep(0,10),1:20))
#' (r1 <- rm0(v1))
#' length(r1)
#'
#' (v2 <- rep(0,10))
#' (r2 <- rm0(v2))
#' length(r2)
#'
#' (v3 <- c(0:10))
#' (r3 <- rm0(v3))
#' length(r3)
#'
#' (v4 <- c(rep(0,10), 1:10, rep(0,5), 10:20, rep(0,10)))
#' (r4 <- rm0(v4))
#' length(r4)
#' sum(v4 == 0)
#' @export
rm0 <- function(y){

  if(sum(y^2, na.rm = TRUE) == 0){ # null vector
    return(0)
  }

  is.zero <- y == 0 # is zero?
  if(sum(is.zero, na.rm = TRUE) == 0){ # no zeros, returns original vector
    return(y)
  }

  ly <- length(y)
  first.zero <- is.zero[1] # is the First position a Zero?
  diz <- diff(is.zero) # Difference of Is.Zero

  w0 <- which(is.zero) # positions containing zeros
  lw0 <- length(w0) # number of (positions containing) zeros

  d0 <- diff(w0) # position of the changes (F-T or T-F) in v0 == 0
  change <- d0 != 1
  n0 <- sum(change, na.rm = TRUE)+1 # number of sets with 0's

  if(sum(is.zero) == n0){ # number of zeros equals to number of sets with 0's
    return(y)
  }

  lv <- ly - lw0 + n0 # length of compacted vector, after cleaning 0's
  v <- rep(0, lv) # compact vector

  wdn0 <- which(diz != 0) # position of the last change (F-T or T-F in y==0)
  dwdn0 <- c(wdn0[1], diff(wdn0)) # sizes of sets with zeros and non-zeros, alternated
  ld0 <- length(dwdn0)
  dwdn1 <- rep(1, ld0) # vector of ones

  if(ld0 == 1 & first.zero){
    suppressWarnings(v[2:lv] <- y[(dwdn0[1]+1):ly])
  }

  if(ld0 == 1 & !first.zero){
    suppressWarnings(v[1:dwdn0[1]] <- y[1:dwdn0[1]])
  }

  if(ld0 > 1){
    odd <- seq(1, ld0, by = 2) # odd numbers 1:length(dwdn0)
    dwdn1[first.zero+odd] <- dwdn0[first.zero+odd]

    cs0 <- cumsum(dwdn0)
    cs1 <- cumsum(dwdn1)

    suppressWarnings(v[1:cs1[1]] <- y[1:cs0[1]])

    for(i in 2:length(cs0)){ # the loop works fine even with the warning (gets the last position, ok)
      suppressWarnings(v[(cs1[i-1]+1):cs1[i]] <- y[(cs0[i-1]+1):cs0[i]])
    }
  }
  return(v)
}

#'  Get left channel
#'
#' @description Get left channel from WAV file.
#' @param x Wave object from `tuneR::readWave`.
#' @return Numeric vector indicating the left channel from a WAV file.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern <- glob2rx('*.wav'), full.names = TRUE)
#'
#' rw <- tuneR::readWave(path2wav[1])
#' l <- voice::get_left(rw)
#' head(l)
#' length(l)
#' @export
get_left <- function(x){
  return(x@left)
}

#' Verify if an audio is mono
#'
#' @usage is_mono(x)
#' @param x Path to WAV audio file.
#' @return Logical. `TRUE` indicates a mono (one-channel) file. `FALSE` indicates a non-mono (two-channel) file.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' is_mono(path2wav[1])
#' sapply(path2wav, is_mono)
#' @export
is_mono <- function(x){
  audio <- tuneR::readWave(x, 1, 2)
  is.mono <- tuneR::nchannel(audio) == 1
  return(is.mono)
}

#' These functions are sourced from the `embedr` package by Michael McCarthy, under
#' MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
#' This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
#' See https://github.com/mccarthy-m-g/embedr for more details.
#'
#' Embed audio in R Markdown documents
#'
#' `embed_audio()` provides a standard way to embed audio in R Markdown
#' documents when the output format is HTML, and to print placeholder text
#' when the output format is not HTML.
#'
#' `embed_audio()` is a wrapper for the HTML5 `<audio>` element that prints
#' HTML `<audio>` code in HTML documents built by R Markdown and placeholder
#' text in non-HTML documents built by R Markdown. This function may be useful
#' for conditional output that depends on the output format. For example, you
#' may embed audio in an R Markdown document when the output format is HTML,
#' and print placeholder text when the output format is LaTeX.
#'
#' The function determines output format using [knitr::is_html_output()]. By
#' default, these formats are considered as HTML formats: `c('markdown',
#' 'epub', 'html', 'html5', 'revealjs', 's5', 'slideous', 'slidy')`.
#'
#' @param src A path or URL to the media file.
#' @param type The type of media file specified in `src`.
#' @param attribute A character vector specifying which attributes to use.
#'   "none" can be used if no attributes are desired.
#' @param id A character string specifying a unique ID for the element.
#'   Can be used by CSS or JavaScript to perform certain tasks for
#'   the element with the specific ID.
#' @param placeholder The placeholder text to use when the output format is
#'   not HTML.
#' @return If `knitr::is_html_output()` is `TRUE`, returns HTML `<audio>` code.
#'   If `knitr::is_html_output()` is `FALSE`, returns placeholder text.
#' @note This function is supposed to be used in R code chunks or inline R code
#'   expressions. You are recommended to use forward slashes (/) as path
#'   separators instead of backslashes in the file paths.
#' @examples
#' # By default, embed_audio() embeds an audio element with playback controls
#' embed_audio(mp3)
#'
#' # To change the attributes of the audio element, use `attribute`
#' embed_audio(mp3, attribute = c("controls", "loop"))
#'
#' # To add placeholder text for non-HTML documents, use `placeholder`
#' embed_audio(mp3, placeholder = "This is placeholder text.")
#'
#' \dontrun{
#' # embed_audio() is intended to be used in R Markdown code chunks or inline
#' # expressions. The following creates and knits an R Markdown document to
#' # HTML and PDF in your current working directory for you to inspect:
#' library(rmarkdown)
#' writeLines(c("# Hello embedr!",
#' "```{r embed-audio, echo=TRUE}",
#' "embed_audio(mp3, placeholder = 'This is placeholder text.')",
#' "```"), "test.Rmd")
#' render("test.Rmd", output_format = c('html_document', 'pdf_document'))
#'
#' # Delete test files created by example code
#' unlink(c("test.Rmd", "test.html", "test.pdf"))
#' }
#' @export
embed_audio <- function(src,
                        type = c("mpeg", "ogg", "wav"),
                        attribute = c("controls", "autoplay", "loop",
                                      "muted", "preload", "none"),
                        id = "",
                        placeholder = "") {
  # check if src has a valid media file extension
  is.audio(src)
  # check if the audio sources exist
  is.local(src)
  is.hosted(src)
  # check whether default values should be used for type or attribute
  if(missing(type)) {type <- "mpeg"}
  if(missing(attribute)) {attribute <- "controls"}
  # evaluate choices
  type <- match.arg(type,
                    c("mpeg", "ogg", "wav"),
                    several.ok = TRUE)
  attribute <- match.arg(attribute,
                         c("controls", "autoplay", "loop",
                           "muted", "preload", "none"),
                         several.ok = TRUE)
  # collapse attribute choices to a character string
  attribute <- paste(attribute, sep = " ", collapse = " ")
  # make attribute empty if "none" is in the character string
  if (all(grepl("none", attribute))) {attribute <- ""}
  # compare length of src and type character vectors
  if (length(src) != length(type)) {
    message("Arguments `src` and `type` are different lengths; ",
            "recycling the shorter vector.")
  }
  # print output
  if (knitr::is_html_output()) {
    # open <audio> tag
    if (missing(id)) {
      audio <- sprintf("<audio %1$s>", attribute)
    } else {
      audio <- sprintf("<audio id='%2$s' %1$s>", attribute, id)
    }
    # create <source> strings from vectors, then collapse to single string
    audio_source <- paste(sprintf("<source src='%1$s' type='audio/%2$s'>",
                                  src, type), sep = "", collapse = "")
    # print the completed HTML <audio> output
    htmltools::HTML(audio, audio_source,
                    "Your browser does not support the audio tag; ",
                    "for browser support, please see: ",
                    "https://www.w3schools.com/tags/tag_audio.asp",
                    "</audio>")
  } else htmltools::HTML(placeholder)
}

#' Embed video in R Markdown documents
#'
#' `embed_video()` provides a standard way to embed video in R Markdown
#' documents when the output format is HTML, and to print placeholder text
#' when the output format is not HTML.
#'
#' `embed_video()` is a wrapper for the HTML5 `<video>` element that prints
#' HTML `<video>` code in HTML documents built by R Markdown and placeholder
#' text in non-HTML documents built by R Markdown. This function may be useful
#' for conditional output that depends on the output format. For example, you
#' may embed video in an R Markdown document when the output format is HTML,
#' and print placeholder text when the output format is LaTeX.
#'
#' The function determines output format using [knitr::is_html_output()]. By
#' default, these formats are considered as HTML formats: `c('markdown',
#' 'epub', 'html', 'html5', 'revealjs', 's5', 'slideous', 'slidy')`.
#'
#' @inheritParams embed_audio
#' @param width The width of the video, in pixels.
#' @param height The height of the video, in pixels.
#' @param thumbnail A path to an image.
#' @return If `knitr::is_html_output()` is `TRUE`, returns HTML `<video>` code.
#'   If `knitr::is_html_output()` is `FALSE`, returns placeholder text.
#' @note This function is supposed to be used in R code chunks or inline R code
#'   expressions. You are recommended to use forward slashes (/) as path
#'   separators instead of backslashes in the file paths.
#' @examples
#' # By default, embed_video() embeds a video element with playback controls
#' embed_video(mp4)
#'
#' # To change the attributes of the video element, use `attribute`
#' embed_video(mp4, attribute = c("controls", "loop"))
#'
#' # To add a thumbnail to the video element, use `thumbnail`
#' embed_video(mp4, thumbnail = png)
#'
#' # To add placeholder text for non-HTML documents, use `placeholder`
#' embed_video(mp4, placeholder = "This is placeholder text.")
#'
#' \dontrun{
#' # embed_video() is intended to be used in R Markdown code chunks or inline
#' # expressions. The following creates and knits an R Markdown document to
#' # HTML and PDF in your current working directory for you to inspect:
#' library(rmarkdown)
#' writeLines(c("# Hello embedr!",
#' "```{r embed-video, echo=TRUE}",
#' "embed_video(mp4, thumbnail = png, placeholder = 'This is placeholder text.')",
#' "```"), "test.Rmd")
#' render("test.Rmd", output_format = c('html_document', 'pdf_document'))
#'
#' # Delete test files created by example code
#' unlink(c("test.Rmd", "test.html", "test.pdf"))
#' }
#' @export
embed_video <- function(src,
                        type = c("mp4", "webm", "ogg"),
                        width = "320",
                        height = "240",
                        attribute = c("controls", "autoplay", "loop",
                                      "muted", "preload", "none"),
                        thumbnail = NULL,
                        id = "",
                        placeholder = "") {
  # check if src has a valid media file extension
  is.video(src)
  # check if the video sources exist
  is.local(src)
  is.hosted(src)
  # check whether default values should be used for type or attribute
  if(missing(type)) {type <- "mp4"}
  if(missing(attribute)) {attribute <- "controls"}
  # evaluate choices
  type <- match.arg(type,
                    c("mp4", "webm", "ogg"),
                    several.ok = TRUE)
  attribute <- match.arg(attribute,
                         c("controls", "autoplay", "loop",
                           "muted", "preload", "none"),
                         several.ok = TRUE)
  # collapse attribute choices to a character string
  attribute <- paste(attribute, sep = " ", collapse = " ")
  # make attribute empty if "none" is in the character string
  if (all(grepl("none", attribute))) {attribute <- ""}
  # compare length of src and type character vectors
  if (length(src) != length(type)) {
    message("Vectors `src` and `type` are different lengths; ",
            "recycling the shorter vector.")
  }
  # print output
  if (knitr::is_html_output()) {
    # decide whether to include thumbnail and id
    if (!missing(thumbnail) & !missing(id)) {
      # thumbnail and id
      video <- sprintf("<video id='%5$s' width='%1$s'
                       height='%2$s' %3$s poster='%4$s'>",
                       width, height, attribute, thumbnail, id)
    } else if (!missing(thumbnail) & missing(id)) {
      # thumbnail
      video <- sprintf("<video width='%1$s' height='%2$s' %3$s poster='%4$s'>",
                       width, height, attribute, thumbnail)
    } else if (missing(thumbnail) & !missing(id)) {
      # id
      video <- sprintf("<video id='%4$s' width='%1$s' height='%2$s' %3$s>",
                       width, height, attribute, id)
    } else {
      # no thumbnail or id
      video <- sprintf("<video width='%1$s' height='%2$s' %3$s>",
                       width, height, attribute)
    }
    # create <source> strings from vectors, then collapse to single string
    video_source <- paste(sprintf("<source src='%1$s' type='video/%2$s'>",
                                  src, type), sep = "", collapse = "")
    # print the completed HTML <video> output
    htmltools::HTML(video, video_source,
                    "Your browser does not support the video tag; ",
                    "for browser support, please see: ",
                    "https://www.w3schools.com/tags/tag_video.asp",
                    "</video>")
  } else htmltools::HTML(placeholder)
}

#' Interpolate vectors using multicore
#'
#' @param y A numeric vector, matrix or data frame.
#' @param compact.to Proportion of remaining points after compression. If equals to 1 and keep.zeros = TRUE, the original vector is presented.
#' @param drop.zeros Logical. Drop repeated zeros? Default: \code{FALSE}.
#' @param to.data.frame Logical. Convert to data frame? Default: \code{FALSE}.
#' @param round.off Number of decimal places of the interpolated \code{y}. Default: \code{NULL}.
#' @param weight Vector of weights with same length of \code{y}. Default: \code{NULL}.
#' @param mc.cores The number of cores to mclapply. Default: \code{1}.
#' @return A list of x and y convoluted values with length near to \code{compact.to*length(y)}.
#' @importFrom dplyr select
#' @importFrom dplyr %>%
#' @examples
#' library(voice)
#' # Same result of interp() function if x is a vector
#' interp(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = FALSE)
#' interp_mc(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = FALSE)
#'
#' interp(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = TRUE)
#' interp_mc(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = TRUE)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' \donttest{
#' # getting Media data frame
#' M <- voice::extract_features(dirname(path2wav), mc.cores = 1, verbose = FALSE)
#'
#' M.num <- M[,-(1:3)]
#' nrow(M.num)
#' cm1 <- interp_mc(M.num, compact.to = 0.1, drop.zeros = TRUE,
#' to.data.frame = FALSE, mc.cores = 1)
#' names(cm1)
#' lapply(cm1$f0, length)
#' }
#' @seealso \code{rm0}, \code{interp}, \code{interp_df}
#' @export
interp_mc <- function(y, compact.to, drop.zeros = FALSE, to.data.frame = FALSE,
                      round.off = NULL, weight = NULL,
                      mc.cores = 1){
  if(is.vector(y)){
    cm <- voice::interp(y, compact.to = compact.to, drop.zeros = drop.zeros,
                        to.data.frame = to.data.frame, round.off = round.off,
                        weight = weight)
  }
  if(is.matrix(y) | is.data.frame(y)){
    cm <- parallel::mclapply(y, voice::interp, compact.to = compact.to,
                             drop.zeros = drop.zeros,
                             to.data.frame = to.data.frame,
                             round.off = round.off, weight = weight,
                             mc.cores = mc.cores)
  }
  return(cm)
}

#' Cut audio vectors
#'
#' @description Cut vectors
#' @param x A vector containing the feature to be cut by \code{byvar}.
#' @param byvar A vector containing the variable to cut by.
#' @examples
#' library(voice)
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#' # Media dataset
#' M <- extract_features(path2wav)
#' cut_audio(M$f0)
#' cut_audio(M$gain, M$f0)
#' @export
cut_audio <- function(x, byvar = x){
    # 1. NA, https://stackoverflow.com/questions/74674411/split-vector-by-each-na-in-r
    i <- !is.na(byvar)
    splitx <- split(x[i], cumsum(!i)[i])
    return(splitx)
}

#'  Get right channel
#'
#' @description Get right channel from WAV file.
#' @param x Wave object from `tuneR::readWave`.
#' @return Numeric vector indicating the right channel from a WAV file.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern <- glob2rx('*.wav'), full.names = TRUE)
#'
#' rw <- tuneR::readWave(path2wav[1])
#' r <- voice::get_right(rw)
#' head(r)
#' length(r)
#' @export
get_right <- function(x){
  return(x@right)
}

#' Tag a data frame with media information
#'
#' @param x An Extended data frame to be tagged with media information. See references.
#' @param groupBy A variable to group the summary measures. The argument must be a character vector. (Default: \code{groupBy = 'wav_path'}).
#' @param wavPath A vector containing the path(s) to WAV files. May be both as \code{dirname} or \code{basename} formats.
#' @param wavPathName A string containing the WAV path name. (Default: \code{wavPathName = 'wav_path'}).
#' @param tags Tags to be added to \code{x}. See Details. (Default: \code{'feat_summary'}).
#' @param sortByGroupBy Logical. Should the function sort the Extended data frame \code{x} by \code{gropuBy}? (Default: \code{sortByGroupBy = TRUE}).
#' @param filesRange The desired range of directory files. Should only be used when all the WAV files are in the same folder. (Default: \code{NULL}, i.e., all files).
#' @param features Vector of features to be extracted. (Default: \code{'f0'}).
#' @param sex \code{= <code>} set sex specific parameters where <code> = \code{'f'}[emale], \code{'m'}[ale] or \code{'u'}[nknown] (default: \code{'u'}). Used as 'gender' by \code{wrassp::ksvF0}, \code{wrassp::forest} and \code{wrassp::mhsF0}.
#' @param windowShift \code{= <dur>} set analysis window shift to <dur>ation in ms (default: 5.0). Used by \code{wrassp::ksvF0}, \code{wrassp::forest}, \code{wrassp::mhsF0}, \code{wrassp::zcrana}, \code{wrassp::rfcana}, \code{wrassp::acfana}, \code{wrassp::cepstrum}, \code{wrassp::dftSpectrum}, \code{wrassp::cssSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param numFormants \code{= <num>} <num>ber of formants (Default: \code{8}). Used by \code{wrassp::forest}.
#' @param numcep Number of Mel-frequency cepstral coefficients (cepstra) to return (Default: \code{12}). Used by \code{tuneR::melfcc}.
#' @param dcttype Type of DCT used. \code{'t1'} or \code{'t2'}, \code{'t3'} for HTK \code{'t4'} for feacalc (Default: \code{'t2'}). Used by \code{tuneR::melfcc}.
#' @param fbtype Auditory frequency scale to use: \code{'mel'}, \code{'bark'}, \code{'htkmel'}, \code{'fcmel'} (Default: \code{'mel'}). Used by \code{tuneR::melfcc}.
#' @param resolution \code{= <freq>} set FFT length to the smallest value which results in a frequency resolution of <freq> Hz or better (Default: \code{40.0}). Used by \code{wrassp::cssSpectrum}, \code{wrassp::dftSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param usecmp Logical. Apply equal-loudness weighting and cube-root compression (PLP instead of LPC) (Default: \code{FALSE}). Used by \code{tuneR::melfcc}.
#' @param mc.cores Number of cores to be used in parallel processing. (Default: \code{1})
#' @param full.names Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (Default: \code{TRUE}) Used by \code{base::list.files}.
#' @param recursive Logical. Should the listing recursively into directories? (Default: \code{FALSE}) Used by \code{base::list.files}.
#' @param check.mono Logical. Check if the WAV file is mono. (Default: \code{TRUE})
#' @param stereo2mono (Experimental) Logical. Should files be converted from stereo to mono? (Default: \code{TRUE})
#' @param overwrite (Experimental) Logical. Should converted files be overwritten? If not, the file gets the suffix \code{_mono}. (Default: \code{FALSE})
#' @param freq Frequency in Hz to write the converted files when \code{stereo2mono=TRUE}. (Default: \code{44100})
#' @param round.to Number of decimal places to round to. (Default: \code{NULL})
#' @param verbose Logical. Should the running status be showed? (Default: \code{FALSE})
#' @return A tibble data frame containing summarized numeric columns using (1) mean, (2) standard deviation, (3) variation coefficient, (4) median, (5) interquartile range and (6) median absolute deviation.
#' @details \code{filesRange} should only be used when all the WAV files are in the same folder.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' # creating Extended synthetic data
#' E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3),
#' wav_path = path2wav)
#' E
#'
#' # minimal usage
#' tag(E)
#'
#' # canonical data
#' tag(E, groupBy = 'subject_id')
#'
#' # limiting filesRange
#' tag(E, filesRange = 3:6)
#'
#' # more features
#' Et <- tag(E, features = c('f0', 'fmt', 'rf', 'rcf', 'rpf', 'rfc', 'mfcc'),
#' groupBy = 'subject_id')
#' Et
#' str(Et)
#' @export
tag <- function(x,
                groupBy = 'wav_path',
                wavPath = unique(x$wav_path),
                wavPathName = 'wav_path',
                tags = c('feat_summary'),
                sortByGroupBy = TRUE,
                filesRange = NULL,
                features = 'f0',
                sex = 'u',
                windowShift = 5,
                numFormants = 8,
                numcep = 12,
                dcttype = c('t2', 't1', 't3', 't4'),
                fbtype = c('mel', 'htkmel', 'fcmel', 'bark'),
                resolution = 40,
                usecmp = FALSE,
                mc.cores = 1,
                full.names = TRUE,
                recursive = FALSE,
                check.mono = FALSE,
                stereo2mono = FALSE,
                overwrite = FALSE,
                freq = 44100,
                round.to = 4,
                verbose = FALSE){

  # sort by groupBy
  if(sortByGroupBy){
    x <- dplyr::arrange(x, groupBy)
  }

  # voice::feat_summary
  if('feat_summary' %in% tags){
    res <- voice::feat_summary(x = x,
                               groupBy = groupBy,
                               wavPath = wavPath,
                               wavPathName = wavPathName,
                               filesRange = filesRange,
                               features = features,
                               sex = sex,
                               windowShift = windowShift,
                               numFormants = numFormants,
                               numcep = numcep,
                               dcttype = dcttype,
                               fbtype = fbtype,
                               resolution = resolution,
                               usecmp = usecmp,
                               mc.cores = mc.cores,
                               full.names = full.names,
                               recursive = recursive,
                               check.mono = check.mono,
                               stereo2mono = stereo2mono,
                               overwrite = overwrite,
                               freq = freq,
                               round.to = round.to,
                               verbose = verbose)
  }

  # # voice::audio_time
  # if('audio_time' %in% tags){
  #   at <- voice::audio_time(wavPath, filesRange = filesRange,
  #                           recursive = recursive)
  #   if(exists('res')){
  #     res <- dplyr::left_join(res, at, by = groupBy)
  #   } else {
  #     res <- at
  #   }
  # }

  # # voice::spoken_prop
  # st <- spoken_time(wavPath, get.id = TRUE, recursive = TRUE)
  # x <- left_join(x, st, by = 'filename')
  # x <- mutate(x, tag_spoken_prop = tag_spoken_time/tag_audio_time)
  #
  #
  # # voice::notes_summary
  # notes_summary(wavPath, get.id = TRUE, i = i)

  return(res)
}

#' Assign notes to frequencies
#'
#' @description Returns a vector of notes for equal-tempered scale, A4 = 440 Hz.
#' @param x Numeric vector of frequencies in Hz.
#' @param method Method of specifying musical pitch. (Default: \code{spn}, i.e., Scientific Pitch Notation).
#' @param moving.average Logical. Must apply moving average? (Default: \code{FALSE}).
#' @param k Integer width of the rolling window used if moving.average is TRUE. (Default: \code{11}).
#' @return A vector containing the notes for equal-tempered scale, A4 = 440 Hz. When `method = 'spn'` the vector is of class 'ordered factor'. When `method = 'octave'` the vector is of class 'factor'.  When `method = 'midi'` the vector is of class 'integer'.
#' @details The symbol '#' is being used to represent a sharp note, the higher
#' in pitch by one semitone on Scientific Pitch Notation (SPN).
#' @references \url{https://en.wikipedia.org/wiki/Scientific_pitch_notation#Table_of_note_frequencies}
#' @seealso \code{notes_freq}
#' @examples
#' library(voice)
#' notes(c(220,440,880))
#' notes(c(220,440,880), method = 'octave')
#' notes(c(220,440,880), method = 'midi')
#' @export
notes <- function(x, method = 'spn', moving.average = FALSE, k = 11){
  if(moving.average){
    x <- zoo::rollmean(x, k)
  }
  x <- as.matrix(x)
  # freq <- voice::notes_freq()$freq
  # distance <- diff(freq)
  # lf <- length(freq)
  # freqhalf <- c(freq[1] - distance[1]/2,
  #               freq[-lf] + distance/2,
  #               freq[lf]+distance[lf-1]/2)
  freqs <- c(voice::notes_freq()$spn.lo,
             voice::notes_freq()$spn.hi[108])
  spn <- voice::notes_freq()$spn[findInterval(x, freqs)]
  if(method == 'spn'){
    return(spn)
  } else if(method == 'midi'){
    midi <- voice::notes_freq()$midi[match(spn, voice::notes_freq()$spn)]
    return(midi)
  } else if(method == 'octave'){
    lev <- c('C','C#','D','D#','E','F','F#','G','G#','A','A#','B')
    octa <- base::strsplit(as.character(spn), '[0-9]')
    octa <- factor(unlist(octa), levels = lev)
    return(octa)
  } else if(method == 'black'){
    black <- notes_freq()$black[match(spn, voice::notes_freq()$spn)]
    return(black)
  } else if(method == 'Black'){
    Black <- notes_freq()$Black[match(spn, voice::notes_freq()$spn)]
    return(Black)
  }
}

#' Interpolate vectors
#'
#' @description Interpolate vactors, compressing to \code{compact.to} fraction. May remove zeros.
#' @param y A vector or time series.
#' @param compact.to Proportion of remaining points after compaction, between (including) 0 and 1. If equals to 1 and keep.zeros = TRUE, the original vector is presented.
#' @param drop.zeros Logical. Drop repeated zeros? Default: \code{FALSE}.
#' @param to.data.frame Logical. Convert to data frame? Default: \code{FALSE}.
#' @param round.off Number of decimal places of the interpolated \code{y} Default: \code{NULL}.
#' @param weight Vector of weights with same length of \code{y}. Default: \code{NULL}.
#' @return A list of interpolated \code{x} and \code{y} values with length near to \code{compact.to*length(y)}.
#' @examples
#' library(voice)
#'
#' v1 <- 1:100
#' (c1 <- interp(v1, compact.to = 0.2))
#' length(c1$y)
#' plot(1:100, type = 'l')
#' points(c1$x, c1$y, col='red')
#'
#' # with weight
#' (c2 <- interp(v1, compact.to = 0.2, weight = rev(v1)))
#' plot(c1$y)
#' points(c2$y, col = 'red')
#'
#' (v2 <- c(1:5, rep(0,10), 1:10, rep(0,5), 10:20, rep(0,10)))
#' length(v2)
#' interp(v2, 0.1, drop.zeros = TRUE, to.data.frame = FALSE)
#' interp(v2, 0.1, drop.zeros = TRUE, to.data.frame = TRUE)
#' interp(v2, 0.2, drop.zeros = TRUE)
#' interp(v2, 0.2, drop.zeros = FALSE)
#'
#' (v3 <- c(rep(0,10), 1:20, rep(0,3)))
#' (c3 <- interp(v3, 1/3, drop.zeros = FALSE, to.data.frame = FALSE))
#' lapply(c3, length)
#' plot(v3, type = 'l')
#' points(c3$x, c3$y, col = 'red')
#'
#' (v4 <- c(rnorm(1:100)))
#' (c4 <- interp(v4, 1/4, round.off = 3))
#' @seealso \code{rm0}, \code{interp_mc}, \code{interp_df}
#' @export
interp <- function(y, compact.to, drop.zeros = FALSE, to.data.frame = FALSE,
                   round.off = NULL, weight = NULL){

  ifelse(drop.zeros, v <- voice::rm0(y), v <- y)
  lv <- length(v)

  # build weight
  if(!is.null(weight)){
    w <- ceiling(weight)
    v <- rep(v, w)
  }

  # interpolating
  cv <- stats::approx(v, n = ceiling(compact.to*lv), rule = 2)

  # round.off
  if(!is.null(round.off)){
    cv <- lapply(cv, round, round.off)
  }

  # data frame
  if(to.data.frame){
    cv <- do.call(cbind, cv)
  }
  return(cv)
}

#' Assign musical notes
#'
#' @description Assign musical notes in Scientific Pitch Notation or other variant. See \code{voice::notes()}. The notes are cut considering f0 to ensure alignment.
#' @param x Media dataset from voice::extract_features().
#' @param fmt Either F0 or formant frequency (in Hz). Default: \code{fmt = 0}.
#' @param min_points Minimum number of points for audio section. Default: \code{min_points = 4}.
#' @param min_percentile Minimum percentile value of gain to be included on the average of \code{fmt}. Default: \code{min_percentile = 0.75}.
#' @param max_na_prop Maximum proportion os NAs on gain sector. Default: \code{max_na_prop = 1}.
#' @importFrom stats median
#' @importFrom stats quantile
#' @examples
#' library(voice)
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#' # Media dataset
#' M <- extract_features(path2wav)
#' assign_notes(M, fmt = 0) # f0
#' assign_notes(M, fmt = 1) # f1
#' assign_notes(M, fmt = 2) # f2
#' @export
assign_notes <- function(x, fmt = 0, min_points = 4,
                         min_percentile = 0.75, max_na_prop = 1) {

  # writing f0/formant name
  fmt_vec <- paste0('f', fmt)

  # cutting audio by f0 to guarantee same number of notes along f0/formants
  fmt_list <- tryCatch(cut_audio(x[, fmt_vec], x[, 'f0']),
                       error = function(e) stop("Error in voice::cut_audio for formant: ", e$message))
  gain_list <- tryCatch(cut_audio(x[, 'gain'], x[, 'f0']),
                        error = function(e) stop("Error in voice::cut_audio for gain: ", e$message))

  # filtering min_points in f0/formant
  fmt_min_points <- sapply(fmt_list, length)
  valid_points_idx <- fmt_min_points >= min_points
  fmt_list <- fmt_list[valid_points_idx]
  gain_list <- gain_list[valid_points_idx]
  fmt_min_points <- fmt_min_points[valid_points_idx]

  # filtering gain == NA
  gain_na_prop <- sapply(lapply(gain_list, is.na), mean)
  valid_na_idx <- gain_na_prop < max_na_prop
  fmt_list <- fmt_list[valid_na_idx]
  gain_list <- gain_list[valid_na_idx]
  fmt_min_points <- fmt_min_points[valid_na_idx]
  gain_na_prop <- gain_na_prop[valid_na_idx]

  # filtering min_percentile in gain
  gain_min_percentile <- lapply(gain_list, quantile, min_percentile, na.rm = TRUE)
  gain_min_percentile <- gain_min_percentile[fmt_min_points >= min_points]

  # calculating median
  l <- length(gain_list)
  notes12et <- vector('list', length = l)
  for (i in 1:l) {
    fltr_gain <- gain_list[[i]] > gain_min_percentile[[i]]
    notes12et[[i]] <- tryCatch(notes(median(fmt_list[[i]][fltr_gain], na.rm = TRUE)),
                               error = function(e) warning(paste("Error in voice::notes for segment", i, ":", e$message), immediate. = TRUE))
  }

  return(unlist(notes12et))
}

#' Returns the total time of audio files in seconds
#'
#' @param x Either a WAV file or a directory containing WAV files.
#' @param filesRange The desired range of directory files (default: \code{NULL}, i.e., all files).
#' @param recursive Logical. Should the listing recursively into directories? (default: \code{FALSE}) Used by \code{base::list.files}.
#' @return A tibble containing file name <chr> and audio time <dbl> in seconds.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern <- glob2rx('*.wav'), full.names = TRUE)
#'
#' # Tibble containing file name and audio time
#' (at <- voice::audio_time(unique(dirname(path2wav))))
#' str(at)
#' @export
audio_time <- function(x, filesRange = NULL, recursive = FALSE){

  # checking if x is a file or directory
  if(utils::file_test('-f', x)){
    wavDir <- dirname(x)
    wavFiles <- x
  } else{
    wavDir <- x
    wavFiles <- dir(wavDir, pattern = '.[Ww][Aa][Vv]$', full.names = TRUE,
                    recursive = recursive)
  }

  # filtering by filesRange
  if(!is.null(filesRange)){
    fullRange <- 1:length(wavFiles)
    filesRange <- base::intersect(fullRange, filesRange)
    wavFiles <- wavFiles[filesRange]
  }

  # file_name (no extension)
  fn <- unlist(strsplit(basename(wavFiles), '.[Ww][Aa][Vv]$'))

  # calculating
  if(length(wavFiles) > 0){
    a <- lapply(wavFiles, tuneR::readWave)
    gl <- lapply(a, voice::get_left)
    sr <- lapply(a, voice::get_samp.rate)
    le <- lapply(gl, length)
    at <- unlist(Map('/', le, sr))
    at <- dplyr::bind_cols(file_name = fn, tag_audio_time = at)
    return(at)
    }
  else{
    cat('NO WAV FILES IN DIRECTORY!')
  }
}

#' Writes a list to a path
#'
#' @param x A list.
#' @param path A full path to file.
#' @return A file named `list.txt` in `path`.
#' @examples
#' \dontrun{
#' library(voice)
#'
#' pts <- list(x = cars[,1], y = cars[,2])
#' listFile <- paste0(tempdir(), '/list.txt')
#' voice::write_list(pts, listFile)
#' file.info(listFile)
#' system(paste0('head ', listFile))
#' }
#' @export
write_list <- function(x, path){
  sink(path)
  print(x)
  sink()
}

#' Features summary
#'
#' @description Returns summary measures of 'voice::extract_features'.
#' @param x An Extended data frame to be tagged with media information.
#' @param groupBy A variable to group the summary measures. The argument must be a character vector. (Default: \code{groupBy = 'wav_path'}).
#' @param wavPath A vector containing the path(s) to WAV files. May be both as \code{dirname} or \code{basename} formats.
#' @param wavPathName A string containing the WAV path name. (Default: \code{wavPathName = 'wav_path'}).
#' @param features Vector of features to be extracted. (Default: \code{'f0'}).
#' @param filesRange The desired range of directory files (default: \code{NULL}, i.e., all files). Should only be used when all the WAV files are in the same folder.
#' @param sex \code{= <code>} set sex specific parameters where <code> = \code{'f'}[emale], \code{'m'}[ale] or \code{'u'}[nknown] (Default: \code{'u'}). Used as 'gender' by \code{wrassp::ksvF0}, \code{wrassp::forest} and \code{wrassp::mhsF0}.
#' @param windowShift \code{= <dur>} set analysis window shift to <dur>ation in ms (Default: \code{5.0}). Used by \code{wrassp::ksvF0}, \code{wrassp::forest}, \code{wrassp::mhsF0}, \code{wrassp::zcrana}, \code{wrassp::rfcana}, \code{wrassp::acfana}, \code{wrassp::cepstrum}, \code{wrassp::dftSpectrum}, \code{wrassp::cssSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param numFormants \code{= <num>} <num>ber of formants (Default: \code{8}). Used by \code{wrassp::forest}.
#' @param numcep Number of Mel-frequency cepstral coefficients (cepstra) to return (Default: \code{12}). Used by \code{tuneR::melfcc}.
#' @param dcttype Type of DCT used. \code{'t1'} or \code{'t2'}, \code{'t3'} for HTK \code{'t4'} for feacalc (Default: \code{'t2'}). Used by \code{tuneR::melfcc}.
#' @param fbtype Auditory frequency scale to use: \code{'mel'}, \code{'bark'}, \code{'htkmel'}, \code{'fcmel'} (Default: \code{'mel'}). Used by \code{tuneR::melfcc}.
#' @param resolution \code{= <freq>} set FFT length to the smallest value which results in a frequency resolution of <freq> Hz or better (Default: \code{40.0}). Used by \code{wrassp::cssSpectrum}, \code{wrassp::dftSpectrum} and \code{wrassp::lpsSpectrum}.
#' @param usecmp Logical. Apply equal-loudness weighting and cube-root compression (PLP instead of LPC) (Default: \code{FALSE}). Used by \code{tuneR::melfcc}.
#' @param mc.cores Number of cores to be used in parallel processing. (Default: \code{1})
#' @param full.names Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (Default: \code{TRUE}). Used by \code{base::list.files}.
#' @param recursive Logical. Should the listing recursively into directories? (Default: \code{FALSE}) Used by \code{base::list.files}.
#' @param check.mono Logical. Check if the WAV file is mono. (Default: \code{TRUE})
#' @param stereo2mono (Experimental) Logical. Should files be converted from stereo to mono? (Default: \code{TRUE})
#' @param overwrite (Experimental) Logical. Should converted files be overwritten? If not, the file gets the suffix \code{_mono}. (Default: \code{FALSE})
#' @param freq Frequency in Hz to write the converted files when \code{stereo2mono=TRUE}. (Default: \code{44100})
#' @param round.to Number of decimal places to round to. (Default: \code{NULL})
#' @param verbose Logical. Should the running status be showed? (Default: \code{FALSE})
#' @return A tibble data frame containing summarized numeric columns using (1) mean, (2) standard deviation, (3) variation coefficient, (4) median, (5) interquartile range and (6) median absolute deviation.
#' @details \code{filesRange} should only be used when all the WAV files are in the same folder.
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' # creating Extended synthetic data
#' E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3),
#' wav_path = path2wav)
#'
#' # minimal usage
#' feat_summary(E)
#'
#' # canonical data
#' feat_summary(E, groupBy = 'subject_id')
#' @export
feat_summary <- function(x,
                         groupBy = 'wav_path',
                         wavPath = unique(x$wav_path),
                         wavPathName = 'wav_path',
                         features = 'f0',
                         filesRange = NULL,
                         sex = 'u',
                         windowShift = 10,
                         numFormants = 8,
                         numcep = 12,
                         dcttype = c('t2', 't1', 't3', 't4'),
                         fbtype = c('mel', 'htkmel', 'fcmel', 'bark'),
                         resolution = 40,
                         usecmp = FALSE,
                         mc.cores = 1,
                         full.names = TRUE,
                         recursive = FALSE,
                         check.mono = FALSE,
                         stereo2mono = FALSE,
                         overwrite = FALSE,
                         freq = 44100,
                         round.to = 4,
                         verbose = FALSE){

  M <- voice::extract_features(wavPath,
                               filesRange = filesRange,
                               features = features,
                               sex = sex,
                               windowShift = windowShift,
                               numFormants = numFormants,
                               numcep = numcep,
                               dcttype = dcttype,
                               fbtype = fbtype,
                               resolution = resolution,
                               usecmp = usecmp,
                               mc.cores = mc.cores,
                               full.names = full.names,
                               recursive = recursive,
                               check.mono = check.mono,
                               stereo2mono = stereo2mono,
                               overwrite = overwrite,
                               freq = freq,
                               round.to = round.to,
                               verbose = verbose)

  # full vector of features
  featFull <- colnames(M[,-(1:3)])

  # normalizing dirnames @ Media
  M_path_name <- normalizePath(dirname(dplyr::pull(M[, wavPathName])))
  M_base_name <- basename(dplyr::pull(M[, wavPathName]))
  M[, wavPathName] <- paste0(M_path_name, '/', M_base_name)

  # normalizing dirnames @ Extended
  x <- dplyr::as_tibble(x)
  if(utils::file_test('-f', dplyr::pull(x[, wavPathName])[1])){
    x_path_name <- normalizePath(dirname(dplyr::pull(x[, wavPathName])))
    x_base_name <- basename(dplyr::pull(x[, wavPathName]))
    x[, wavPathName] <- paste0(x_path_name, '/', x_base_name)
    M <- dplyr::left_join(M, x, wavPathName)
  } else{
    x[, wavPathName] <- normalizePath(dplyr::pull(x[, wavPathName]))
    wav_path_full <- dir(as.data.frame(x)[, wavPathName], full.names = TRUE)
    x_full <- dplyr::tibble(wav_path = dirname(wav_path_full),
                            wav_path_full = wav_path_full) # generalize wavPathName!
    x_full <- dplyr::left_join(x_full, x, by = 'wav_path')
    x_full <- dplyr::transmute(x_full, subject_id = subject_id, wav_path = wav_path_full)
    M <- dplyr::left_join(M, x_full, wavPathName)
  }

  # Variation Coefficient function
  vc <- function(x, na.rm = TRUE){
    return(stats::sd(x, na.rm = na.rm)/mean(x, na.rm = na.rm))
  }

  # complement of featFull
  compFeatFull <- setdiff(colnames(M), featFull)

  # tyding up
  M <- dplyr::select(M, tidyselect::all_of(compFeatFull), tidyselect::all_of(featFull))
  # dplyr::rename(!!wavPathName := tidyselect::ends_with('.y'))

  # group by
  Mg <- M %>%
    dplyr::group_by(.data[[groupBy]])

  # Mean
  M_mean <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), mean, na.rm = TRUE)
  colnames(M_mean)[-1] <- paste0(colnames(M_mean)[-1], '_tag_mean')

  # (Sample) Standard Deviation
  M_sd <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), stats::sd, na.rm = TRUE)
  colnames(M_sd)[-1] <- paste0(colnames(M_sd)[-1], '_tag_sd')

  # (Sample) Variation Coefficient
  M_vc <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), vc, na.rm = TRUE)
  colnames(M_vc)[-1] <- paste0(colnames(M_vc)[-1], '_tag_vc')

  # Median
  M_median <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), stats::median, na.rm = TRUE)
  colnames(M_median)[-1] <- paste0(colnames(M_median)[-1], '_tag_median')

  # InterQuartile Range
  M_iqr <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), stats::IQR, na.rm = TRUE)
  colnames(M_iqr)[-1] <- paste0(colnames(M_iqr)[-1], '_tag_iqr')

  # Median Absolute Deviation
  M_mad <- Mg %>%
    dplyr::summarise_at(dplyr::vars(tidyselect::all_of(featFull)), stats::mad, na.rm = TRUE)
  colnames(M_mad)[-1] <- paste0(colnames(M_mad)[-1], '_tag_mad')

  # left_join
  M_summ <- dplyr::left_join(M_mean, M_sd, by = groupBy)
  M_summ <- dplyr::left_join(M_summ, M_vc, by = groupBy)
  M_summ <- dplyr::left_join(M_summ, M_median, by = groupBy)
  M_summ <- dplyr::left_join(M_summ, M_iqr, by = groupBy)
  M_summ <- dplyr::left_join(M_summ, M_mad, by = groupBy)

  return(M_summ)
}

#' Piano plot
#'
#' @description Piano plot showing the notes in Scientific Pitch Notation.
#' @param dat Data frame or tibble containing the desired frequencies to be plotted.
#' @param num_fmt Number of the desired formant (includes f0 for simplicity). Default: \code{num_fmt = 0}.
#' @param log_freq Logical. Must plot log(frequency)?
#' @param base Logarithm base. Default: \code{exp(1)}.
#' @param color Must the graph be colored overall or by slice? Default: \code{slice}.
#' @examples
#' library(voice)
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#' # Media dataset
#' M <- extract_features(path2wav[1])
#' piano_plot(M, 0)
#' piano_plot(M, 0, color = 'overall')
#' @references https://en.wikipedia.org/wiki/12_equal_temperament
#'
#' https://en.wikipedia.org/wiki/Scientific_pitch_notation
#' @export
piano_plot <- function(dat,
                       num_fmt = 0,
                       log_freq = TRUE,
                       base = exp(1),
                       color = 'slice'){

  # Apply log
  if(log_freq == TRUE){
    lg <- function(x){
      l <- log(x, base = base)
      return(l)
    }
    dat <- dat %>%
      dplyr::mutate(dplyr::across(dplyr::starts_with("f"), lg))
  }

  # Preparing data
  x <- dplyr::pull(dat[,'section_seq_file'])
  fmt_vec <- paste0('f', num_fmt)

  # Dropping first and last NAs
  not_na <- !is.na(dat[,fmt_vec])
  not_na_cum <- apply(not_na, 2, cumsum)
  beg <- which(rowSums(not_na_cum) != 0)[1]
  wh <- which(rowSums(apply(not_na_cum, 2, diff)) != 0)
  end <- wh[length(wh)]+2

  # Non NA
  f0_cut <- voice::cut_audio(dat$f0)
  cs_f0 <- cumsum(!is.na(dat$f0))
  x_cut <- which(as.logical(diff(cs_f0)))+1
  gain_cut <- voice::cut_audio(dat$gain, dat$f0)
  df_long <- tibble::tibble(x_cut, f0_cut = unlist(f0_cut), gain_cut = unlist(gain_cut))

  # gain_color function
  gain_color <- function(x){
    r <- x + abs(min(x, na.rm = TRUE)) + 0.1
    r <- scale(r)
    return(r)
  }
  # Colors by gain/amplitude (overall)
  if(color == 'overall'){
    col_gain <- gain_color(dat$gain)
    # col_gain <- dat$gain + abs(min(dat$gain, na.rm = TRUE)) + 0.1
  }
  # Colors by gain/amplitude (using gain_cut)
  if(color == 'slice'){
    col_gain <- dat$gain
    col_gain[!not_na] <- 0
    col_gain[not_na] <- unlist(lapply(gain_cut, gain_color))
  }

  freqs_lab <- 'Frequency'
  if(log_freq == TRUE){
    freqs_lab <- 'log Frequency'
  }
  # Background figure
  fig <- ggplot2::ggplot(dat, ggplot2::aes(x = x)) +
    ggplot2::xlab('Sample') +
    ggplot2::ylab(freqs_lab) +
    ggplot2::ggtitle(paste0(freqs_lab, ' and notes in SPN'))

  # Adding f0
  if(sum(num_fmt == 0)){
    fig <- fig +
      ggplot2::geom_line(ggplot2::aes(y = .data$f0, col = col_gain), data = dat) +
      # ggplot2::geom_line(ggplot2::aes(y = f0), data = dat) +
      # ggplot2::geom_point(ggplot2::aes(color = df_long$gain_cut)) +
      ggplot2::scale_colour_gradient(low = "blue", high = "red") +
      ggplot2::xlim(beg-10, end)
    # fig
  }

  # Add ingf1
  if(sum(num_fmt == 1)){
    fig <- fig +
      ggplot2::geom_line(ggplot2::aes(y = .data$f1, col = col_gain), data = dat) +
      ggplot2::scale_colour_gradient(low = "blue", high = "red") +
      ggplot2::xlim(beg-10, end)
  }

  # Adding f2
  if(sum(num_fmt == 2)){
    fig <- fig +
      ggplot2::geom_line(ggplot2::aes(y = .data$f2, col = col_gain), data = dat) +
      ggplot2::scale_colour_gradient(low = "blue", high = "red") +
      ggplot2::xlim(beg-10, end)
  }

  # Adding frequency limits
  freqs <- c(voice::notes_freq()$spn.lo,
             voice::notes_freq()$spn.hi[108])
  if(log_freq == TRUE){
    freqs <- log(freqs, base = base)
  }
  fltr <- freqs >= min(dat[,fmt_vec], na.rm = TRUE) &
    freqs <= max(dat[,fmt_vec], na.rm = TRUE)

  fig <- fig +
    ggplot2::geom_hline(yintercept = freqs[fltr], col = 'grey') +
    ggplot2::annotate('text', x = beg-10,
                      y = log(voice::notes_freq()$freq[fltr], base=base),
                      label = voice::notes_freq()$spn[fltr])

  return(fig)
}

utils::globalVariables(c('subject_id', '.data', 'spn.lo', 'spn.hi', 'spn',
                         'midi', 'black', 'Black', 'wavelength', 'unemploy',
                         'pop', 'filename', 'tag_audio_time'))

#'  Enrich RTTM files
#'
#' @description Enrich Rich Transcription Time Marked (RTTM) files obtained from '\code{voice::read_rttm}'.
#' @param listRttm A list containing RTTM files.
#' @param silence.gap The silence gap (in seconds) between adjacent words in a keyword. Rows with \code{tdur <= silence.gap} are removed. (default: \code{0.5})
#' @param as.tibble Logical. Should it return a tibble?
#' @return A list containing either data frames or tibbles obtained from standard RTTM files. See 'voice::read_rttm'.
#' @references \url{https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf}
#' @seealso \code{voice::read_rttm}
#' @examples
#' \donttest{
#' library(voice)
#'
#' url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
#' destfile0 <- paste0(tempdir(), '/sherlock0.rttm')
#' download.file(url0, destfile = destfile0)
#' url1 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock1.rttm'
#' destfile1 <- paste0(tempdir(), '/sherlock1.rttm')
#' download.file(url0, destfile = destfile1)
#'
#' rttm <- voice::read_rttm(dirname(destfile0))
#' (er <- voice::enrich_rttm(rttm))
#' class(er)
#' lapply(er, class)
#' }
#' @export
enrich_rttm <- function(listRttm, silence.gap = 0.5, as.tibble = TRUE){
  fltr <- function(x){
    f <- cumsum(x$tdur > silence.gap)
    f[x$tdur <= silence.gap] <- NA
    f <- cbind(f,x)
    return(f)
  }
  rttm <- lapply(listRttm, fltr)
  rttm <- lapply(rttm, function(x){cbind(1:nrow(x), x)})

  colnames <- c('id', 'id_split', 'type', 'file', 'chnl', 'tbeg', 'tdur',
                'ortho', 'stype', 'name', 'conf', 'slat')
  rttm <- lapply(rttm, stats::setNames, colnames)

  # speakProp <- function(x,y){
  #   sp <- x/y
  # }

  if(as.tibble){
    rttm <- lapply(rttm, dplyr::as_tibble)
  }
  return(rttm)
}

#' Check chords
#'
#' @description Check the sequence of musical notes for chords.
#' @param x A vector containing a sequence of musical notes.
#' @param window Size of window of notes to be checked. Default: \code{3}.
#' @param try_perm Logical. Must try all notes permutations of notes? Default: \code{FALSE}.
#' @examples
#' \dontrun{
#' library(voice)
#' check_chords(c('C','E','G'), window = 3, try_perm = FALSE)
#' check_chords(c('C','E','G'), window = 3, try_perm = TRUE)
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#' M <- extract_features(path2wav)
#' M$gain[is.na(M$f0)] <- NA
#' # assigning notes
#' f0_spn <- assign_notes(M, fmt = 0)
#' check_chords(f0_spn, window = 3, try_perm = FALSE)
#' check_chords(f0_spn, window = 3, try_perm = TRUE)
#' check_chords(f0_spn, window = 4, try_perm = TRUE)
#' }
#' @export
check_chords <- function(x, window = 3, try_perm = FALSE){

  # import Python libs
  pc <- reticulate::import('pychord')
  mi <- reticulate::import('mingus.core.chords')

  # length
  l <- length(x)

  # checking window > l
  if(window > l){
    cat('Window must be less than or equal to the length!\n')
    return(NA)
  }

  # chords lowercase without space (tabr)
  x_lo <- voice::spn2abc(x, to_lower = TRUE) #TODO: , drop_rep_seq = TRUE)

  # chords uppercase with space
  x_up <- voice::spn2abc(x)

  # major/minor by tabr
  if(try_perm){
    # permutations
    perm_lo <- arrangements::permutations(x_lo, window)
    perm_up <- arrangements::permutations(x_up, window)
    nchord <- nrow(perm_lo) #TODO: pre calculate nchord
    # TODO: clean repeated sequences, drop_rep_seq @ spn2abc?
    chrds <- data.frame(i = NA, seq_notes = NA, major = NA, minor = NA,
                        pychord = NA, mingus = NA)
    for(i in 1:nchord){
      fltr_lo <- perm_lo[i,]
      fltr_lo <- paste0(fltr_lo, collapse = '')
      fltr_up <- perm_up[i,]

      chrds[i,1] <- i
      chrds[i,2] <- toupper(fltr_lo)
      chrds[i,3] <- tabr::chord_is_major(fltr_lo)
      chrds[i,4] <- tabr::chord_is_minor(fltr_lo)
      pychord <- pc$find_chords_from_notes(fltr_up)
      if(length(pychord) > 0){
        chrds[i,5] <- paste0(sapply(pychord, as.character), collapse = ', ')
      }
      mingus <- mi$determine(fltr_up)
      if(length(mingus) > 0){
        chrds[i,6] <- paste0(sapply(mingus, as.character), collapse = ', ')
      }
    }
  } else{
    nchord <- l-window+1
    chrds <- data.frame(i = NA, seq_notes = NA,
                        major = NA, minor = NA,
                        pychord = NA, mingus = NA)
    for(i in 1:nchord){
      fltr_lo <- x_lo[i:(window+i-1)]
      fltr_lo <- paste0(fltr_lo, collapse = '')
      fltr_up <- x_up[i:(window+i-1)]

      chrds[i,1] <- i
      chrds[i,2] <- toupper(fltr_lo)
      chrds[i,3] <- tabr::chord_is_major(fltr_lo)
      chrds[i,4] <- tabr::chord_is_minor(fltr_lo)
      pychord <- pc$find_chords_from_notes(fltr_up)
      if(length(pychord) > 0){
        chrds[i,5] <- paste0(sapply(pychord, as.character), collapse = ', ')
      }
      mingus <- mi$determine(fltr_up)
      if(length(mingus) > 0){
        chrds[i,6] <- paste0(sapply(mingus, as.character), collapse = ', ')
      }
    }
  }
  return(chrds)
}

#' Read RTTM files
#'
#' @description Read Rich Transcription Time Marked (RTTM) files in \code{fromRttm} directory.
#' @param fromRttm A directory/folder containing RTTM files.
#' @return A list containing data frames obtained from standard RTTM files. See 'Details'.
#' @details The Rich Transcription Time Marked (RTTM) files are space-delimited text files containing one turn per line defined by NIST - National Institute of Standards and Technology. Each line containing ten fields:
#'
#' \code{type} Type: segment type; should always by SPEAKER.
#'
#' \code{file} File ID: file name; basename of the recording minus extension (e.g., rec1_a).
#'
#' \code{chnl} Channel ID: channel (1-indexed) that turn is on; should always be 1.
#'
#' \code{tbeg} Turn Onset -- onset of turn in seconds from beginning of recording.
#'
#' \code{tdur} Turn Duration -- duration of turn in seconds.
#'
#' \code{ortho} Orthography Field -- should always by <NA>.
#'
#' \code{stype} Speaker Type -- should always be <NA>.
#'
#' \code{name} Speaker Name -- name of speaker of turn; should be unique within scope of each file.
#'
#' \code{conf} Confidence Score -- system confidence (probability) that information is correct; should always be <NA>.
#'
#' \code{slat} Signal Lookahead Time -- should always be <NA>.
#' @references \url{https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf}
#' @seealso \code{voice::enrich_rttm}
#' @examples
#' library(voice)
#'
#' url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
#' download.file(url0, destfile = paste0(tempdir(), '/sherlock0.rttm'))
#' url1 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock1.rttm'
#' download.file(url0, destfile = paste0(tempdir(), '/sherlock1.rttm'))
#'
#' (rttm <- voice::read_rttm(tempdir()))
#' class(rttm)
#' lapply(rttm, class)
#' @export
read_rttm <- function(fromRttm){
  rttmFiles <- list.files(fromRttm, pattern = "[[:punct:]][rR][tT][tT][mM]$",
                          full.names = TRUE)
  if(length(rttmFiles)==0){
    cat('EMPTY DIRECTORY?')
  } else{
    rttm <- lapply(rttmFiles, utils::read.table)
    colnames <- c('type', 'file', 'chnl', 'tbeg', 'tdur',
                  'ortho', 'stype', 'name', 'conf', 'slat')
    rttm <- lapply(rttm, stats::setNames, colnames)
    names(rttm) <- basename(rttmFiles)

    return(rttm)
  }
}

#' Who spoke when?
#'
#' Diarization of WAV audios.
#'
#' @param fromWav Either a file or a directory containing WAV files.
#' @param toRttm A directory to write RTTM files. If the default \code{toRttm = NULL} is used, \code{'./voiceAudios/rttm'} is created and used.
#' @param autoDir Logical. Must the directories tree be created? Default: \code{FALSE}. See 'Details'.
#' @param pycall Python call. See \url{https://github.com/filipezabala/voice} for details.
#' @param token Access token needed to instantiate pretrained speaker diarization
#' pipeline from pyannote.audio. #1 Install pyannote.audio 3.1 with pip install
#' pyannote.audio (already listed @ https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt).
#' #2. Accept \url{https://hf.co/pyannote/segmentation-3.0} user conditions.
#' #3 Accept \url{https://huggingface.co/pyannote/speaker-diarization-3.1} user conditions.
#' #4. Create access token in \url{https://hf.co/settings/tokens}.
#' @return RTTM files in NIST standard. See 'voice::read_rttm'.
#' @details When \code{autoDir = TRUE}, the following directories are created: \code{'../mp3'},\code{'../rttm'}, \code{'../split'} and \code{'../musicxml'}. Use \code{getwd()} to find the parent directory \code{'../'}.
#' @import reticulate
#' @examples
#' \dontrun{
#' library(voice)
#'
#' wavDir <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' voice::diarize(fromWav = unique(dirname(wavDir)),
#' toRttm = tempdir(),
#' token = NULL) # Must enter a token! See documentation.
#'
#' (rttm <- dir(tempdir(), '.[Rr][Tt][Tt][Mm]$', full.names = TRUE))
#' file.info(rttm)
#' }
#' @export
diarize <- function(fromWav, toRttm = NULL, autoDir = FALSE,
                    pycall = '~/miniconda3/envs/pyvoice/bin/python',
                    token = NULL){

  if(is.null(token)){
    stop('Must enter a token!')
  }

  if(autoDir){
    wavDir <- fromWav[1]
    ss <- unlist(strsplit(wavDir, '/'))
    parDir <- paste0(ss[-length(ss)], collapse ='/')
    mp3Dir <- paste0(parDir, '/mp3')
    rttmDir <- paste0(parDir, '/rttm')
    splitDir <- paste0(parDir, '/split')
    mxmlDir <- paste0(parDir, '/musicxml')
    ifelse(!dir.exists(parDir), dir.create(parDir), 'Directory exists!')
    ifelse(!dir.exists(wavDir), dir.create(wavDir), 'Directory exists!')
    ifelse(!dir.exists(mp3Dir), dir.create(mp3Dir), 'Directory exists!')
    ifelse(!dir.exists(rttmDir), dir.create(rttmDir), 'Directory exists!')
    ifelse(!dir.exists(splitDir), dir.create(splitDir), 'Directory exists!')
    ifelse(!dir.exists(mxmlDir), dir.create(mxmlDir), 'Directory exists!')
  }

  if(is.null(toRttm)){
    toRttm <- rttmDir
  }

  #TODO: sort (by some rule) files for extraction

  # process time
  pt0 <- proc.time()
  st0 <- Sys.time()

  reticulate::use_condaenv(pycall, required = TRUE)
  pyannote <- reticulate::import('pyannote.audio')
  pipeline <- pyannote$Pipeline$from_pretrained('pyannote/speaker-diarization-3.1',
                                                use_auth_token = token)

  #TODO: solve 'with closing file handler' issue.

  wavFiles <- dir(fromWav, '.[Ww][Aa][Vv]$', full.names = TRUE)

  for(i in wavFiles){
    diarization <- pipeline(i)
    py <- reticulate::import_builtins()
    rttmFile <- sub('.[Ww][Aa][Vv]$', '.rttm', i)
    rttmBase <- basename(rttmFile)
    rttmTo <- paste0(toRttm, '/', rttmBase)
    # rttmTo <- normalizePath(paste0(toRttm, rttmBase))
    f <- py$open(rttmTo, 'w')
    diarization$write_rttm(f)
    f$close()
  }

  print(Sys.time()-st0)
}

#' Transcribe audio to text
#'
#' @description Transcribe audio to text.
#' @param file Audio file to trinscribe.
#' @param server Default: http://localhost:8080.
#' @param wait Default: \code{TRUE}.
#' @param poll_interval Default: 2.
#' @param timeout Default: 300.
#' @references https://github.com/rishikanthc/scriberr
#' @export
transcribe <- function(
    file,
    server = "http://localhost:8080",
    wait = TRUE,
    poll_interval = 2,
    timeout = 300
) {
  stopifnot(file.exists(file))

  # 1. Upload file
  upload_resp <- httr2::request(paste0(server, "/api/upload")) |>
    httr2::req_body_multipart(file = file) |>
    httr2::req_perform()

  upload_data <- httr2::resp_body_json(upload_resp)
  file_id <- upload_data$id

  # 2. Trigger transcription
  transcribe_resp <- httr2::request(paste0(server, "/api/transcribe")) |>
    httr2::req_body_json(list(file_id = file_id)) |>
    httr2::req_perform()

  job_data <- httr2::resp_body_json(transcribe_resp)
  job_id <- job_data$job_id

  # 3. Poll for result
  if (!wait) return(job_id)

  start_time <- Sys.time()

  repeat {
    status_resp <- httr2::request(
      paste0(server, "/api/transcripts/", job_id)
    ) |>
      httr2::req_perform()

    status_data <- httr2::resp_body_json(status_resp)

    if (status_data$status == "completed") {
      return(list(
        text = status_data$text,
        segments = status_data$segments,
        speakers = status_data$speakers
      ))
    }

    if (difftime(Sys.time(), start_time, units = "secs") > timeout) {
      stop("Transcription timed out")
    }

    Sys.sleep(poll_interval)
  }
}

#' Smooth numeric variables in a data frame
#'
#' @param x A data frame.
#' @param k Integer width of the rolling window. Default: \code{11}.
#' @param id The identification column. Default: \code{colname} of the first column of \code{x}.
#' @param colnum A \code{char} vector indicating the numeric colnames. If \code{NULL}, uses the columns of the \code{numeric} class.
#' @param mc.cores The number of cores to mclapply. By default uses \code{1}.
#' @return Vector of interpolated values with length near to \code{compact.to*length(x)}.
#' @importFrom dplyr %>%
#' @seealso \code{extract_features}
#' @examples
#' library(voice)
#'
#' # get path to audio file
#' path2wav <- list.files(system.file('extdata', package = 'wrassp'),
#' pattern = glob2rx('*.wav'), full.names = TRUE)
#'
#' # minimal usage
#' M <- extract_features(path2wav, features = c('f0', 'fmt'))
#' (Ms <- smooth_df(M[-(1:2)]))
#' dim(M)
#' dim(Ms)
#' @export
smooth_df <- function(x, k = 11, id = colnames(x)[1], colnum = NULL,
                      mc.cores = 1){

  # n
  n <- nrow(x)
  n_by_id <- table(x[,id])

  # n smoothed
  ns_fun <- function(x,k) {return(x-k+1)}
  ns_by_id <- sapply(n_by_id, ns_fun, k=k)
  ns <- sum(ns_by_id) # generalize

  # vector and length of distinct id's
  id_names <- names(n_by_id)
  n_id <- length(id_names)

  # beginning (beg0) and end (end0) of original objects
  beg0 <- c(0, cumsum(n_by_id)) + 1
  end0 <- beg0[-(n_id+1)] -1 + ns_by_id

  # beginning (beg1) and end (end1) of smoothed objects
  beg1 <- c(0, cumsum(ns_by_id)) + 1
  end1 <- cumsum(ns_by_id)

  # numeric columns
  if(is.null(colnum)){
    is.num <- sapply(x, class) %in% c('integer', 'numeric')
    colnum <- colnames(x[,is.num])
  }

  # non-numeric columns
  colnon <- base::setdiff(colnames(x), colnum)

  # split numeric columns by id
  snum <- split(x[,colnum], x[,id])

  # split non-numeric columns by id
  snon <- split(x[,colnon], x[,id])
  snon_df <- tibble::as_tibble(do.call(rbind, snon), .name_repair = 'unique')

  # smooth
  snum_li <- parallel::mclapply(snum, zoo::rollmean, k,
                     mc.cores = mc.cores)
  snum_df <- tibble::as_tibble(do.call(rbind, snum_li), .name_repair = 'unique')

  # binding non numeric columns to *x* *s*moothed
  xs <- suppressMessages(tibble::as_tibble(matrix(NA, nrow = ns,
                                 ncol = ncol(snon_df)+ncol(snum_df)),
                          .name_repair = 'unique'))
  colnames(xs) <- c(colnames(snon_df), colnames(snum_df))
  for(i in 1:n_id){
    fltr0 <- beg0[i]:end0[i]
    fltr1 <- beg1[i]:end1[i]
    xs[fltr1, ] <- dplyr::bind_cols(snon_df[fltr0,], snum_df[fltr1,])
  }

  # reordering columns
  xs <- xs %>%
    dplyr::select(colnames(x))

  return(xs)
}

---
title: '`voice` vignette'
subtitle: 'version `r packageVersion("voice")`'
author: 'Filipe J. Zabala'
date: "`r format(Sys.time(), '%Y-%m-%d')`"
# date: "`r format(Sys.time(), '%Y-%m-%d %H:%M:%S')`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{`voice` vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
ini0 <- Sys.time()
```

## 0. Basic installation
```{r, eval=FALSE}
# Development version from GitHub
install.packages('devtools')
devtools::install_github('filipezabala/voice')

# Stable version from CRAN
install.packages('voice')
```

### 0.1 For Windows Users
If you're compiling R packages from source, you may need to install [RTools](https://cran.r-project.org/bin/windows/Rtools/), a collection of Windows-specific build tools for R.

### 0.2 For macOS Users
If you're compiling packages, ensure you have [Xcode Command Line Tools](https://mac.install.guide/commandlinetools/) installed. You also may need [macOS tools](https://cran.r-project.org/bin/macosx/tools/).

```{bash, eval=FALSE}
# Install Xcode on MacOS
xcode-select --install
```
More details may be found at https://github.com/filipezabala/voice.


## 1. Extract features
### 1.1 Load packages and audio files
```{r, message=FALSE, warning=FALSE}
# packs
library(voice)
library(tidyverse)

# get path to audio file
wavDir <- list.files(system.file('extdata', package = 'wrassp'),
                     pattern = glob2rx('*.wav'), full.names = TRUE)
```

### 1.2 Extract features
```{r, message=FALSE, warning=FALSE}
# minimal usage
M <- voice::extract_features(wavDir)
glimpse(M)
```


## 2. Tag
```{r, message=FALSE, warning=FALSE}
# creating Extended synthetic data
E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3), wav_path = wavDir)
E

# minimal usage
voice::tag(E)

# canonical data
voice::tag(E, groupBy = 'subject_id')
```


## 3. Visualization
### 3.1 Get audio
```{r, message=FALSE, warning=FALSE, eval=FALSE}
url0 <- 'https://github.com/filipezabala/voiceAudios/raw/refs/heads/main/wav/doremi.wav'
download.file(url0, paste0(tempdir(), '/doremi.wav'), mode = 'wb')
voice::embed_audio(url0) # See https://github.com/mccarthy-m-g/embedr for more details.
```

### 3.2 Media data
```{r, message=FALSE, warning=FALSE, eval=FALSE}
M <- voice::extract_features(tempdir())
summary(M)
```

### 3.3 Plot
```{r, message=FALSE, warning=FALSE, fig.width=7.5, fig.height=4, eval=FALSE}
voice::piano_plot(M, 0) # f0
voice::piano_plot(M, 0:1) # f0 + f1
```
![](./img/plot1.png){width=95%}
![](./img/plot2.png){width=95%}

### 3.4 Assign notes
```{r, message=FALSE, warning=FALSE, eval=FALSE}
(f0_spn <- voice::assign_notes(M, fmt = 0, min_points = 22, min_percentile = .85)) # f0
(f1_spn <- voice::assign_notes(M, fmt = 1, min_points = 22, min_percentile = .85)) # f1
```

### 3.5 Sheet music
Must have [MuseScore](https://musescore.org/en) and [gm](https://flujoo.github.io/gm/articles/gm.html).

#### 3.5.1 Notes sequence of f0
```{r, message=FALSE, warning=FALSE, eval=FALSE}
library(gm)
line_0 <- gm::Line(as.character(f0_spn))
m0 <- gm::Music() +
  gm::Meter(4, 4) +
  line_0
gm::show(m0, to = c('score', 'audio'))
```
![](./img/sheet1.png){width=95%}

#### 3.5.2 Notes sequences of f0 and f1
```{r, message=FALSE, warning=FALSE, eval=FALSE}
line_0 <- gm::Line(as.character(f0_spn))
line_1 <- gm::Line(as.character(f1_spn))
m1 <- gm::Music() +
  gm::Meter(4, 4) +
  line_0 + line_1
gm::show(m1, to = c('score', 'audio'))
```
![](./img/sheet2.png){width=95%}

<!-- #### 3.6.3 Chords sequence using f0, f1 and f2  -->
<!-- ```{r, message=FALSE, warning=FALSE} -->
<!-- line_0 <- gm::Line(as.character(f0_spn)) -->
<!-- line_1 <- gm::Line(as.character(f1_spn)) -->
<!-- line_2 <- gm::Line(as.character(f2_spn)) -->
<!-- m2 <- gm::Music() +  -->
<!--   gm::Meter(4, 4) +  -->
<!--   line_0 + line_1 + line_2 -->
<!-- gm::show(m2, to = c('score', 'audio')) -->
<!-- ``` -->


## 4. Advanced installation
Python-based functions `diarize` and `extract_features` (when the latter is inferring `f0_praat` and `fmt_praat` features) require a configured Python environment.

### 4.1 Ubuntu
The following steps are used to fully configure `voice` on Ubuntu 24.04 LTS (Noble Numbat). Reports of inconsistencies are welcome.

<!-- > Without the following Python items 3 and 10, you may run all the functions except `diarize` and `extract_features` calling `f0_praat` and `fmt_praat` features. These functions run respectively [pyannote-audio](https://github.com/pyannote/pyannote-audio) and [Parselmouth](https://github.com/YannickJadoul/Parselmouth). -->

#### 4.1.1. [Curl](https://curl.se/)
Command line tool and library for transferring data with URLs.
```bash
# installing dependencies
sudo apt-get update
sudo apt-get install -y libssl-dev autoconf libtool make
# installing curl
sudo apt install curl
# verify installation
curl --version
```

<!-- #### 4.1.2. [Git](https://git-scm.com/) -->
<!-- Git is a free and open source distributed version control system. -->
<!-- ```bash -->
<!-- sudo apt-get update -->
<!-- sudo apt-get install git-all -->
<!-- ``` -->

<!-- #### 4.1.3. [pip](https://pypi.org/project/pip/) -->
<!-- pip is the package installer for Python.  -->
<!-- ```bash -->
<!-- sudo apt-get update -->
<!-- sudo apt-get install python3-pip -->
<!-- pip3 --version -->
<!-- ``` -->

#### 4.1.2. [ffmpeg](https://ffmpeg.org/)
ffmpeg is a cross-platform solution to record, convert and stream audio and video.
```bash
sudo apt-get update
sudo apt-get install ffmpeg
```

#### 4.1.3. Audio drivers and extra packages
```bash
sudo apt-get update
sudo apt-get install portaudio19-dev libasound2-dev libfontconfig1-dev libmagick++-dev libxml2-dev libharfbuzz-dev libfribidi-dev libgdal-dev cmake cmake-doc ninja-build
```

#### 4.1.4. [MuseScore](https://musescore.org/)
MuseScore is an open source notation software.

```bash
sudo add-apt-repository ppa:mscore-ubuntu/mscore-stable
sudo apt-get update
sudo apt-get install musescore
```

#### 4.1.5. [R](https://www.r-project.org)
R is a free software environment for statistical computing and graphics. To find out your Ubuntu distribution use `lsb_release -a` at terminal.    
```bash
sudo sh -c 'echo "deb https://cloud.r-project.org/bin/linux/ubuntu focal-cran40/" >> /etc/apt/sources.list.d/cran.list'
sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys E084DAB9 
sudo apt-key adv --keyserver keyserver.ubuntu.com --recv-keys 51716619E084DAB9
gpg -a --export E084DAB9 | sudo apt-key add -

sudo add-apt-repository ppa:c2d4u.team/c2d4u4.0+

sudo apt-get update && sudo apt-get upgrade
sudo apt-get install r-base r-base-dev
```

#### 4.1.6. [RStudio](https://posit.co/downloads)
RStudio is an Integrated Development Environment (IDE) for R. Check for updates [here](https://posit.co/downloads).
```bash
sudo apt-get update
sudo apt-get install gdebi-core
wget https://download1.rstudio.org/electron/jammy/amd64/rstudio-2025.05.0-496-amd64.deb
sudo gdebi rstudio-2025.05.0-496-amd64.deb
```

#### 4.1.9. R packages
"Packages are the fundamental units of reproducible R code." [Hadley Wickham and Jennifer Bryan](https://r-pkgs.org/). The installation may take several minutes. At terminal run:
```bash
sudo R
```

Running R as super user paste the following, row by row:
```r
packs <- c('audio','reticulate','R.utils','seewave','tidyverse','tuneR','wrassp')
install.packages(packs, dep = TRUE)
update.packages(ask = FALSE)
devtools::install_github('egenn/music')
devtools::install_github('flujoo/gm')
```
To configure the `gm` package.
```r
usethis::edit_r_environ()
```

Add the line `MUSESCORE_PATH=/usr/bin/mscore` to `/root/.Renviron` file. To exit use `:wq` at VI. Save and restart the R/RStudio session.


#### 4.1.10. [Miniconda](https://docs.conda.io/en/latest/miniconda.html)
Miniconda is a free minimal installer for [conda](https://docs.conda.io/), an open source package, dependency and environment management system for any language—Python, R, Ruby, Lua, Scala, Java, JavaScript, C/ C++, FORTRAN and more, that runs on Windows, macOS and Linux.   
Follow the instructions at https://docs.conda.io/en/latest/miniconda.html. 

At terminal:
```bash
cd ~/Downloads/
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-Linux-x86_64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-Linux-x86_64.sh
```
Do you accept the license terms? [yes|no] `yes`.

Miniconda3 will now be installed into this location: /home/user/miniconda3 [ENTER]

You can undo this by running `conda init --reverse $SHELL`? `yes`

Do you wish the installer to initialize Miniconda3 by running conda init? `yes`.

Close and reopen terminal.

```bash
conda update -n base -c defaults conda
```

The following packages will be INSTALLED/REMOVED/UPDATED/DOWNGRADED:... Proceed ([y]/n)? `y`

```bash
conda create -n pyvoice python=3.12
```

The following (NEW) packages will be downloaded/INSTALLED:... Proceed ([y]/n)? `y`

```bash
conda activate pyvoice
pip install -r https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt
```


### 4.2 MacOS
The following steps are used to fully configure `voice` on [MacOS Sonoma and Tahoe](https://www.apple.com/os/macos/). Reports of inconsistencies are welcome.

<!-- Hardware   -->
<!--  . MacBook Air (13-inch, 2020)   -->
<!--  . Chip Apple M1 -->
<!--  . Memory 8GB LPDDR4 -->

#### 4.2.1. [Homebrew](https://brew.sh/)
Install Homebrew, 'The Missing Package Manager for macOS (or Linux)' and remember to `brew doctor` eventually. At terminal (`command + space 'terminal'`) run:
```bash
/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"
```

#### 4.2.2. [wget](https://www.gnu.org/software/wget/)
GNU Wget is a free software package for retrieving files using HTTP, HTTPS, FTP and FTPS, the most widely used Internet protocols. It is a non-interactive commandline tool, so it may easily be called from scripts, cron jobs, terminals without X-Windows support, etc.
```bash
brew install wget
```

#### 4.2.3. [Python](https://www.python.org/)
Python is a programming language that integrate systems. According to [this](https://github.com/Homebrew/homebrew-core/issues/62911) post, it is recommended to install Python 3.8 and 3.9 and make it consistent.
```bash
brew install python@3.12
python3 --version 
pip3 --version
```

#### 4.2.4. [ffmpeg](https://ffmpeg.org/)
ffmpeg is a cross-platform solution to record, convert and stream audio and video. The installation may take several minutes.
```bash
brew install ffmpeg
```

#### 4.2.5. [XQuartz](https://www.xquartz.org/)
The XQuartz project is an open-source effort to develop a version of the [X.Org X Window System](https://www.x.org/wiki/) that runs on macOS.

 - Download and run https://github.com/XQuartz/XQuartz/releases/download/XQuartz-2.8.5/XQuartz-2.8.5.pkg
 - Will take around 320 MB of disk space   
 - Send XQuartz-2.8.5.dmg to Trash    


#### 4.2.6. [MacPorts](https://guide.macports.org/chunked/installing.macports.html)
Follow the instructions from https://guide.macports.org/chunked/installing.macports.html.

<!-- ```bash -->
<!-- curl -O https://distfiles.macports.org/MacPorts/MacPorts-2.10.7.tar.bz2 -->
<!-- tar xf MacPorts-2.10.7.tar.bz2 -->
<!-- cd MacPorts-2.10.7 -->
<!-- ./configure -->
<!-- make -->
<!-- sudo make install -->
<!-- cd ../; rm -rf MacPorts-2.10.7* -->
<!-- sudo nano /etc/paths -->
<!-- ``` -->
<!-- Add the following lines at the end of the file. -->
<!-- ```bash -->
<!-- /opt/local/bin -->
<!-- /opt/local/sbin -->
<!-- ``` -->
<!-- Use Ctrl+O and return to save the file and Ctrl+X to close nano editor. Reboot your terminal and use -->
<!-- ```bash -->
<!-- sudo port -v selfupdate -->
<!-- sudo port upgrade outdated -->
<!-- ``` -->

#### 4.2.7. [tcllib](https://ports.macports.org/port/tcllib/)

```bash
sudo port selfupdate && sudo port upgrade tcllib
sudo port install tcllib
```

#### 4.2.8. [MuseScore](https://musescore.org/)
MuseScore is an open source notation software.

 - Download and run https://musescore.org/en/download/musescore.dmg  
 - Drag MuseScore 4 to Applications folder   
 - Will take around 320 MB of disk space   
 - Unmount MuseScore-4.2.1 virtual disk and send MuseScore-4.2.1.240230937.dmg to Trash   


#### 4.2.9. [R](https://www.r-project.org)
R is a free software environment for statistical computing and graphics.  

 - Download and run the pkg file according to you architecture from https://cran.r-project.org/bin/macosx/
 - Will take around 180 MB of disk space   


#### 4.2.10. [RStudio](https://posit.co/downloads)
RStudio is an Integrated Development Environment (IDE) for R.

 - Download and run https://download1.rstudio.org/electron/macos/RStudio-2025.05.0-496.dmg
 - Drag RStudio to Applications folder   
 - Will take around 770 MB of disk space   
 - Unmount RStudio virtual disk and send RStudio-2025.05.0-496.dmg to Trash   
 - Type `command + space 'rstudio'`   
 - Tools > Global Options... > Appearance > Merbivore (Restart required)   


#### 4.2.11. R packages
"Packages are the fundamental units of reproducible R code." [Hadley Wickham and Jennifer Bryan](https://r-pkgs.org/). Type `command + space 'terminal'`   
```bash
sudo R
```

Running R as super user paste the following, one line at a time.
```r
packs <- c('audio','reticulate','R.utils','seewave','tidyverse','tuneR','wrassp')
install.packages(packs, dep = TRUE)
update.packages(ask = FALSE)
devtools::install_github('egenn/music')
devtools::install_github('flujoo/gm')
```

#### 4.2.12. [Miniconda](https://docs.conda.io/en/latest/miniconda.html)
Miniconda is a free minimal installer for [conda](https://docs.conda.io/), an open source package, dependency and environment management system for any language—Python, R, Ruby, Lua, Scala, Java, JavaScript, C/ C++, FORTRAN and more, that runs on Windows, macOS and Linux. 

For 64-bit version use

```bash
cd ~/Downloads
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-MacOSX-x86_64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-MacOSX-x86_64.sh
```

For M1 version use

```bash
cd ~/Downloads
wget -r -np -k https://repo.anaconda.com/miniconda/Miniconda3-latest-MacOSX-arm64.sh
cd repo.anaconda.com/miniconda/
bash Miniconda3-latest-MacOSX-arm64.sh
```

In order to continue the installation process, please review the license
agreement. Please, press ENTER to continue `ENTER`.

You can undo this by running `conda init --reverse $SHELL`? `yes`

Close and reopen terminal.

```bash
export PATH="~/miniconda3/bin:$PATH"
conda update -n base -c defaults conda
```

The following packages will be INSTALLED/REMOVED/UPDATED/DOWNGRADED:... Proceed ([y]/n)? `y`

```bash
conda create -n pyvoice python=3.12
```
The following (NEW) packages will be downloaded/INSTALLED:... Proceed ([y]/n)? `y`

Close and reopen terminal.

```bash
conda activate base
conda activate pyvoice
pip install -r https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt
```

## 5. Diarize

```{r, eval=FALSE}
# download
url0 <- 'https://github.com/filipezabala/voiceAudios/raw/main/wav/sherlock0.wav'
wavDir <- normalizePath(tempdir())
download.file(url0, paste0(wavDir, '/sherlock0.wav'), mode = 'wb')
```
Diarization can be performed to detect speaker segments (i.e., 'who spoke when').
```{r, eval=FALSE}
# diarize
voice::diarize(fromWav = wavDir, toRttm = wavDir, token = 'YOUR_TOKEN')
```

The `voice::diarize()` function creates Rich Transcription Time Marked (RTTM)[^rttm] files, space-delimited text files containing one turn per line defined by NIST - National Institute of Standards and Technology. The RTTM files can be read using `voice::read_rttm()`.

[^rttm]: See Appendix	C at https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf.

```{r, eval=FALSE}
# read_rttm
(rttm <- voice::read_rttm(wavDir))
```

Finally, the audio waves can be automatically segmented.
```{r, eval=FALSE}
# split audio wave
voice::splitw(fromWav = wavDir, fromRttm = wavDir, to = wavDir)
dir(wavDir, pattern = '.[Ww][Aa][Vv]$')
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/notes_freq.R
\name{notes_freq}
\alias{notes_freq}
\title{Frequencies on Scientific Pitch Notation (SPN)}
\usage{
notes_freq()
}
\value{
A tibble with frequencies for equal-tempered scale, A4 = 440 Hz.
}
\description{
Returns a tibble of frequencies on Scientific Pitch Notation (SPN) for equal-tempered scale, A4 = 440 Hz.
}
\details{
The symbol '#' is being used to represent a sharp note, the higher in pitch by one semitone. The SPN is also known as American Standard Pitch Notation (ASPN) or International Pitch Notation (IPN).
}
\examples{
library(voice)
notes_freq()
}
\references{
\url{https://en.wikipedia.org/wiki/Scientific_pitch_notation#Table_of_note_frequencies}
}
\seealso{
\code{notes}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_chords.R
\name{check_chords}
\alias{check_chords}
\title{Check chords}
\usage{
check_chords(x, window = 3, try_perm = FALSE)
}
\arguments{
\item{x}{A vector containing a sequence of musical notes.}

\item{window}{Size of window of notes to be checked. Default: \code{3}.}

\item{try_perm}{Logical. Must try all notes permutations of notes? Default: \code{FALSE}.}
}
\description{
Check the sequence of musical notes for chords.
}
\examples{
\dontrun{
library(voice)
check_chords(c('C','E','G'), window = 3, try_perm = FALSE)
check_chords(c('C','E','G'), window = 3, try_perm = TRUE)
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)
M <- extract_features(path2wav)
M$gain[is.na(M$f0)] <- NA
# assigning notes
f0_spn <- assign_notes(M, fmt = 0)
check_chords(f0_spn, window = 3, try_perm = FALSE)
check_chords(f0_spn, window = 3, try_perm = TRUE)
check_chords(f0_spn, window = 4, try_perm = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interp.R
\name{interp}
\alias{interp}
\title{Interpolate vectors}
\usage{
interp(
  y,
  compact.to,
  drop.zeros = FALSE,
  to.data.frame = FALSE,
  round.off = NULL,
  weight = NULL
)
}
\arguments{
\item{y}{A vector or time series.}

\item{compact.to}{Proportion of remaining points after compaction, between (including) 0 and 1. If equals to 1 and keep.zeros = TRUE, the original vector is presented.}

\item{drop.zeros}{Logical. Drop repeated zeros? Default: \code{FALSE}.}

\item{to.data.frame}{Logical. Convert to data frame? Default: \code{FALSE}.}

\item{round.off}{Number of decimal places of the interpolated \code{y} Default: \code{NULL}.}

\item{weight}{Vector of weights with same length of \code{y}. Default: \code{NULL}.}
}
\value{
A list of interpolated \code{x} and \code{y} values with length near to \code{compact.to*length(y)}.
}
\description{
Interpolate vactors, compressing to \code{compact.to} fraction. May remove zeros.
}
\examples{
library(voice)

v1 <- 1:100
(c1 <- interp(v1, compact.to = 0.2))
length(c1$y)
plot(1:100, type = 'l')
points(c1$x, c1$y, col='red')

# with weight
(c2 <- interp(v1, compact.to = 0.2, weight = rev(v1)))
plot(c1$y)
points(c2$y, col = 'red')

(v2 <- c(1:5, rep(0,10), 1:10, rep(0,5), 10:20, rep(0,10)))
length(v2)
interp(v2, 0.1, drop.zeros = TRUE, to.data.frame = FALSE)
interp(v2, 0.1, drop.zeros = TRUE, to.data.frame = TRUE)
interp(v2, 0.2, drop.zeros = TRUE)
interp(v2, 0.2, drop.zeros = FALSE)

(v3 <- c(rep(0,10), 1:20, rep(0,3)))
(c3 <- interp(v3, 1/3, drop.zeros = FALSE, to.data.frame = FALSE))
lapply(c3, length)
plot(v3, type = 'l')
points(c3$x, c3$y, col = 'red')

(v4 <- c(rnorm(1:100)))
(c4 <- interp(v4, 1/4, round.off = 3))
}
\seealso{
\code{rm0}, \code{interp_mc}, \code{interp_df}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bit.R
\name{get_bit}
\alias{get_bit}
\title{Get bit rate}
\usage{
get_bit(x)
}
\arguments{
\item{x}{Wave object from `tuneR::readWave`.}
}
\value{
Integer indicating the bit rate from a WAV file.
}
\description{
Get bit rate from WAV file.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern <- glob2rx('*.wav'), full.names = TRUE)

rw <- tuneR::readWave(path2wav[1])
voice::get_bit(rw)

rwl <- lapply(path2wav, tuneR::readWave)
sapply(rwl, voice::get_bit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smooth_df.R
\name{smooth_df}
\alias{smooth_df}
\title{Smooth numeric variables in a data frame}
\usage{
smooth_df(x, k = 11, id = colnames(x)[1], colnum = NULL, mc.cores = 1)
}
\arguments{
\item{x}{A data frame.}

\item{k}{Integer width of the rolling window. Default: \code{11}.}

\item{id}{The identification column. Default: \code{colname} of the first column of \code{x}.}

\item{colnum}{A \code{char} vector indicating the numeric colnames. If \code{NULL}, uses the columns of the \code{numeric} class.}

\item{mc.cores}{The number of cores to mclapply. By default uses \code{1}.}
}
\value{
Vector of interpolated values with length near to \code{compact.to*length(x)}.
}
\description{
Smooth numeric variables in a data frame
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

# minimal usage
M <- extract_features(path2wav, features = c('f0', 'fmt'))
(Ms <- smooth_df(M[-(1:2)]))
dim(M)
dim(Ms)
}
\seealso{
\code{extract_features}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/splitw.R
\name{splitw}
\alias{splitw}
\title{Split Wave}
\usage{
splitw(
  fromWav,
  slice_duration = NULL,
  fromRttm = NULL,
  toSplit = NULL,
  autoDir = FALSE,
  subDir = FALSE,
  output = "wave",
  filesRange = NULL,
  full.names = TRUE,
  recursive = FALSE,
  silence.gap = 0.5
)
}
\arguments{
\item{fromWav}{Either WAV file or directory containing WAV files.}

\item{slice_duration}{The slices duration in seconds. Default: \code{NULL}, so RTTM file(s) must be given.}

\item{fromRttm}{Either RTTM file or directory containing RTTM files. Default: \code{NULL}.}

\item{toSplit}{A directory to write generated files. Default: \code{NULL}.}

\item{autoDir}{Logical. Must the directories tree be created? Default: \code{FALSE}. See 'Details'.}

\item{subDir}{Logical. Must the splitted files be placed in subdirectories? Default: \code{FALSE}.}

\item{output}{Character string, the class of the object to return, either 'wave' or 'list'.}

\item{filesRange}{The desired range of directory files (default: \code{NULL}, i.e., all files). Must be TRUE only if \code{fromWav} is a directory.}

\item{full.names}{Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (default: \code{TRUE}) Used by \code{base::list.files}.}

\item{recursive}{Logical. Should the listing recursively into directories? (default: \code{FALSE}) Used by \code{base::list.files}. Inactive if \code{fromWav} is a file.}

\item{silence.gap}{The silence gap (in seconds) between adjacent words in a keyword. Rows with \code{tdur <= silence.gap} are removed. (default: \code{0.5})}
}
\value{
Splited audio files according to the correspondent RTTM file(s). See '\code{voice::diarize}'.
}
\description{
Split WAV files either in \code{fromWav} directory or using (same names) RTTM files/subdirectories as guidance.
}
\details{
When \code{autoDir = TRUE}, the following directories are created: \code{'../mp3'},\code{'../rttm'}, \code{'../split'} and \code{'../musicxml'}. Use \code{getwd()} to find the parent directory \code{'../'}.
}
\examples{
\dontrun{
library(voice)

# Using RTTM files as refernece
urlWav <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/wav/sherlock0.wav'
destWav <- paste0(tempdir(), '/sherlock0.wav')
download.file(urlWav, destfile = destWav)

urlRttm <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
destRttm <- paste0(tempdir(), '/sherlock0.rttm')
download.file(urlRttm, destfile = destRttm)

splitDir <- paste0(tempdir(), '/split')
dir.create(splitDir)

# Splitting via RTTM
splitw(destWav, fromRttm = destRttm, toSplit = splitDir)
dir(splitDir)

# Splitting with slice_duration of 5 seconds
splitw(destWav, slice_duration = 5, toSplit = splitDir)
splitw(destWav, slice_duration = 2, toSplit = splitDir)

}
}
\seealso{
\code{voice::diarize}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is.audio}
\alias{is.audio}
\title{Match string for audio suffix}
\usage{
is.audio(x)
}
\arguments{
\item{x}{A character vector.}
}
\description{
Given a character vector, returns a logical vector indicating
which elements have a valid audio file extension.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cut_audio.R
\name{cut_audio}
\alias{cut_audio}
\title{Cut audio vectors}
\usage{
cut_audio(x, byvar = x)
}
\arguments{
\item{x}{A vector containing the feature to be cut by \code{byvar}.}

\item{byvar}{A vector containing the variable to cut by.}
}
\description{
Cut vectors
}
\examples{
library(voice)
# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)
# Media dataset
M <- extract_features(path2wav)
cut_audio(M$f0)
cut_audio(M$gain, M$f0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/notes.R
\name{notes}
\alias{notes}
\title{Assign notes to frequencies}
\usage{
notes(x, method = "spn", moving.average = FALSE, k = 11)
}
\arguments{
\item{x}{Numeric vector of frequencies in Hz.}

\item{method}{Method of specifying musical pitch. (Default: \code{spn}, i.e., Scientific Pitch Notation).}

\item{moving.average}{Logical. Must apply moving average? (Default: \code{FALSE}).}

\item{k}{Integer width of the rolling window used if moving.average is TRUE. (Default: \code{11}).}
}
\value{
A vector containing the notes for equal-tempered scale, A4 = 440 Hz. When `method = 'spn'` the vector is of class 'ordered factor'. When `method = 'octave'` the vector is of class 'factor'.  When `method = 'midi'` the vector is of class 'integer'.
}
\description{
Returns a vector of notes for equal-tempered scale, A4 = 440 Hz.
}
\details{
The symbol '#' is being used to represent a sharp note, the higher
in pitch by one semitone on Scientific Pitch Notation (SPN).
}
\examples{
library(voice)
notes(c(220,440,880))
notes(c(220,440,880), method = 'octave')
notes(c(220,440,880), method = 'midi')
}
\references{
\url{https://en.wikipedia.org/wiki/Scientific_pitch_notation#Table_of_note_frequencies}
}
\seealso{
\code{notes_freq}
}

\name{mozilla_id_path}
\alias{mozilla_id_path}
\title{Sample IDs and paths}
\description{
  A dataset containing sample IDs and paths from Ardila et al (2019)
  'Common voice: A massively-multilingual speech corpus',
  used in Zabala (2023) 'voice: new approaches to audio analysis'.
  The considered sample contains 34,425 rows associated with
  838 IDs (p_s = 2.4\%).
}
\usage{
mozilla_id_path
}
\seealso{
  \code{\link{extract_features}}.
}
\examples{
library(voice)
mozilla_id_path
}
\references{Ardila R, Branson M, Davis K, Henretty M, Kohler M, Meyer J, Morais R, Saunders L, Tyers FM, Weber G (2019). "Common voice: A massively-multilingual speech corpus." arXiv preprint \href{https://arxiv.org/abs/1912.06670v2}{arXiv:1912.06670}. URL \url{https://arxiv.org/abs/1912.06670}.}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/media_files.R
\docType{data}
\name{media-files}
\alias{media-files}
\alias{mp3}
\alias{mp4}
\alias{png}
\title{These functions are sourced from the `embedr` package by Michael McCarthy, under
MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
See https://github.com/mccarthy-m-g/embedr for more details.}
\format{
An object of class \code{character} of length 1.
}
\usage{
mp3
}
\description{
Example Media Files
}
\details{
Example media files included with embedr.

 - `mp3`: MP3 audio
 - `mp4`: MP4 video
 - `png`: PNG thumbnail
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spn2abc.R
\name{spn2abc}
\alias{spn2abc}
\title{Convert SPN to ABC}
\usage{
spn2abc(x, to_lower = FALSE, spacing = TRUE)
}
\arguments{
\item{x}{A vector containing a note in SPN (Scientific Pitch Notation).}

\item{to_lower}{Logical. Should the string be lower case? Default: \code{FALSE}.}

\item{spacing}{Logical. Should the strin return spaces between notes? Default: \code{TRUE}.}
}
\description{
Convert SPN to standard octave.
}
\examples{
library(voice)
spn2abc('C4')
spn2abc('C5')
spn2abc('C4', to_lower = TRUE)
spn2abc(c('C4','D#7','E2'))
spn2abc(c('C4','D#7','E2'), to_lower = TRUE)
spn2abc(c('C4','D#7','E2'), spacing = FALSE)
spn2abc(c('C4','D#7','E2'), to_lower = TRUE, spacing = FALSE)
}
\references{
https://en.wikipedia.org/wiki/Scientific_pitch_notation

https://en.wikipedia.org/wiki/ABC_notation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duration.R
\name{duration}
\alias{duration}
\title{Duration of sequences}
\usage{
duration(x, windowShift = 5)
}
\arguments{
\item{x}{A vector containing symbols and \code{NA}.}

\item{windowShift}{Window shift to duration in ms (default: 5.0).}
}
\value{
A data frame with duration in number of lines/ocurrences (\code{dur_line}), milliseconds (\code{dur_ms}) and proportional (\code{dur_prop}).
}
\description{
Duration of sequences
}
\examples{
library(voice)
duration(letters)
duration(c('a','a','a',letters,'z'))

nts <- c('NA','NA','A3','A3','A3','A3','A#3','B3','B3','C4','C4','C4','C4',
'C4','C4','C#4','C4','C4','C4','B3','A#3','NA','NA','NA','NA','NA','NA','NA',
'NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','NA','D4','D4','D4','C#4',
'C#4','C#4','C4','C4','B3','B3','A#3','A#3','A3','A3','G3','G#3','G3','F#3')
duration(nts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diarize.R
\name{diarize}
\alias{diarize}
\title{Who spoke when?}
\usage{
diarize(
  fromWav,
  toRttm = NULL,
  autoDir = FALSE,
  pycall = "~/miniconda3/envs/pyvoice/bin/python",
  token = NULL
)
}
\arguments{
\item{fromWav}{Either a file or a directory containing WAV files.}

\item{toRttm}{A directory to write RTTM files. If the default \code{toRttm = NULL} is used, \code{'./voiceAudios/rttm'} is created and used.}

\item{autoDir}{Logical. Must the directories tree be created? Default: \code{FALSE}. See 'Details'.}

\item{pycall}{Python call. See \url{https://github.com/filipezabala/voice} for details.}

\item{token}{Access token needed to instantiate pretrained speaker diarization
pipeline from pyannote.audio. #1 Install pyannote.audio 3.1 with pip install
pyannote.audio (already listed @ https://raw.githubusercontent.com/filipezabala/voice/master/requirements.txt).
#2. Accept \url{https://hf.co/pyannote/segmentation-3.0} user conditions.
#3 Accept \url{https://huggingface.co/pyannote/speaker-diarization-3.1} user conditions.
#4. Create access token in \url{https://hf.co/settings/tokens}.}
}
\value{
RTTM files in NIST standard. See 'voice::read_rttm'.
}
\description{
Diarization of WAV audios.
}
\details{
When \code{autoDir = TRUE}, the following directories are created: \code{'../mp3'},\code{'../rttm'}, \code{'../split'} and \code{'../musicxml'}. Use \code{getwd()} to find the parent directory \code{'../'}.
}
\examples{
\dontrun{
library(voice)

wavDir <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

voice::diarize(fromWav = unique(dirname(wavDir)),
toRttm = tempdir(),
token = NULL) # Must enter a token! See documentation.

(rttm <- dir(tempdir(), '.[Rr][Tt][Tt][Mm]$', full.names = TRUE))
file.info(rttm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_right.R
\name{get_right}
\alias{get_right}
\title{Get right channel}
\usage{
get_right(x)
}
\arguments{
\item{x}{Wave object from `tuneR::readWave`.}
}
\value{
Numeric vector indicating the right channel from a WAV file.
}
\description{
Get right channel from WAV file.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern <- glob2rx('*.wav'), full.names = TRUE)

rw <- tuneR::readWave(path2wav[1])
r <- voice::get_right(rw)
head(r)
length(r)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/embed_media.R
\name{embed_audio}
\alias{embed_audio}
\title{These functions are sourced from the `embedr` package by Michael McCarthy, under
MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
See https://github.com/mccarthy-m-g/embedr for more details.}
\usage{
embed_audio(
  src,
  type = c("mpeg", "ogg", "wav"),
  attribute = c("controls", "autoplay", "loop", "muted", "preload", "none"),
  id = "",
  placeholder = ""
)
}
\arguments{
\item{src}{A path or URL to the media file.}

\item{type}{The type of media file specified in `src`.}

\item{attribute}{A character vector specifying which attributes to use.
"none" can be used if no attributes are desired.}

\item{id}{A character string specifying a unique ID for the element.
Can be used by CSS or JavaScript to perform certain tasks for
the element with the specific ID.}

\item{placeholder}{The placeholder text to use when the output format is
not HTML.}
}
\value{
If `knitr::is_html_output()` is `TRUE`, returns HTML `<audio>` code.
  If `knitr::is_html_output()` is `FALSE`, returns placeholder text.
}
\description{
Embed audio in R Markdown documents
}
\details{
`embed_audio()` provides a standard way to embed audio in R Markdown
documents when the output format is HTML, and to print placeholder text
when the output format is not HTML.

`embed_audio()` is a wrapper for the HTML5 `<audio>` element that prints
HTML `<audio>` code in HTML documents built by R Markdown and placeholder
text in non-HTML documents built by R Markdown. This function may be useful
for conditional output that depends on the output format. For example, you
may embed audio in an R Markdown document when the output format is HTML,
and print placeholder text when the output format is LaTeX.

The function determines output format using [knitr::is_html_output()]. By
default, these formats are considered as HTML formats: `c('markdown',
'epub', 'html', 'html5', 'revealjs', 's5', 'slideous', 'slidy')`.
}
\note{
This function is supposed to be used in R code chunks or inline R code
  expressions. You are recommended to use forward slashes (/) as path
  separators instead of backslashes in the file paths.
}
\examples{
# By default, embed_audio() embeds an audio element with playback controls
embed_audio(mp3)

# To change the attributes of the audio element, use `attribute`
embed_audio(mp3, attribute = c("controls", "loop"))

# To add placeholder text for non-HTML documents, use `placeholder`
embed_audio(mp3, placeholder = "This is placeholder text.")

\dontrun{
# embed_audio() is intended to be used in R Markdown code chunks or inline
# expressions. The following creates and knits an R Markdown document to
# HTML and PDF in your current working directory for you to inspect:
library(rmarkdown)
writeLines(c("# Hello embedr!",
"```{r embed-audio, echo=TRUE}",
"embed_audio(mp3, placeholder = 'This is placeholder text.')",
"```"), "test.Rmd")
render("test.Rmd", output_format = c('html_document', 'pdf_document'))

# Delete test files created by example code
unlink(c("test.Rmd", "test.html", "test.pdf"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{url.exists}
\alias{url.exists}
\title{These functions are sourced from the `embedr` package by Michael McCarthy, under
MIT License: https://github.com/mccarthy-m-g/embedr/blob/master/LICENSE.md
This inclusion is temporary and will be discontinued once `embedr` is available on CRAN.
See https://github.com/mccarthy-m-g/embedr for more details.}
\usage{
url.exists(x)
}
\arguments{
\item{x}{A character vector.}
}
\value{
`TRUE` if the URL responds without error, otherwise `FALSE`.
}
\description{
Check if URL exists
}
\details{
Given a character string, returns a logical vector indicating
whether a request for a specific URL responds without error.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_rttm.R
\name{read_rttm}
\alias{read_rttm}
\title{Read RTTM files}
\usage{
read_rttm(fromRttm)
}
\arguments{
\item{fromRttm}{A directory/folder containing RTTM files.}
}
\value{
A list containing data frames obtained from standard RTTM files. See 'Details'.
}
\description{
Read Rich Transcription Time Marked (RTTM) files in \code{fromRttm} directory.
}
\details{
The Rich Transcription Time Marked (RTTM) files are space-delimited text files containing one turn per line defined by NIST - National Institute of Standards and Technology. Each line containing ten fields:

\code{type} Type: segment type; should always by SPEAKER.

\code{file} File ID: file name; basename of the recording minus extension (e.g., rec1_a).

\code{chnl} Channel ID: channel (1-indexed) that turn is on; should always be 1.

\code{tbeg} Turn Onset -- onset of turn in seconds from beginning of recording.

\code{tdur} Turn Duration -- duration of turn in seconds.

\code{ortho} Orthography Field -- should always by <NA>.

\code{stype} Speaker Type -- should always be <NA>.

\code{name} Speaker Name -- name of speaker of turn; should be unique within scope of each file.

\code{conf} Confidence Score -- system confidence (probability) that information is correct; should always be <NA>.

\code{slat} Signal Lookahead Time -- should always be <NA>.
}
\examples{
library(voice)

url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
download.file(url0, destfile = paste0(tempdir(), '/sherlock0.rttm'))
url1 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock1.rttm'
download.file(url0, destfile = paste0(tempdir(), '/sherlock1.rttm'))

(rttm <- voice::read_rttm(tempdir()))
class(rttm)
lapply(rttm, class)
}
\references{
\url{https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf}
}
\seealso{
\code{voice::enrich_rttm}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_left.R
\name{get_left}
\alias{get_left}
\title{Get left channel}
\usage{
get_left(x)
}
\arguments{
\item{x}{Wave object from `tuneR::readWave`.}
}
\value{
Numeric vector indicating the left channel from a WAV file.
}
\description{
Get left channel from WAV file.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern <- glob2rx('*.wav'), full.names = TRUE)

rw <- tuneR::readWave(path2wav[1])
l <- voice::get_left(rw)
head(l)
length(l)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_model.R
\name{expand_model}
\alias{expand_model}
\title{Expand model}
\usage{
expand_model(y, x, k)
}
\arguments{
\item{y}{The Y variable.}

\item{x}{The X variables.}

\item{k}{Number of additive components.}
}
\value{
A \code{char} vector containing the expanded models.
}
\description{
Expand model given \code{y} and \code{x} variables.
}
\examples{
library(voice)

expand_model('y', LETTERS[1:4], 1)
expand_model('y', LETTERS[1:4], 2)
expand_model('y', LETTERS[1:4], 3)
expand_model('y', LETTERS[1:4], 4)

# multiple models using apply functions
nx <- 10 # number of X variables to be used
models <- lapply(1:nx, expand_model, y = 'y', x = LETTERS[1:nx])
names(models) <- 1:nx
models
sum(sapply(models, length)) # total of models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich_rttm.R
\name{enrich_rttm}
\alias{enrich_rttm}
\title{Enrich RTTM files}
\usage{
enrich_rttm(listRttm, silence.gap = 0.5, as.tibble = TRUE)
}
\arguments{
\item{listRttm}{A list containing RTTM files.}

\item{silence.gap}{The silence gap (in seconds) between adjacent words in a keyword. Rows with \code{tdur <= silence.gap} are removed. (default: \code{0.5})}

\item{as.tibble}{Logical. Should it return a tibble?}
}
\value{
A list containing either data frames or tibbles obtained from standard RTTM files. See 'voice::read_rttm'.
}
\description{
Enrich Rich Transcription Time Marked (RTTM) files obtained from '\code{voice::read_rttm}'.
}
\examples{
\donttest{
library(voice)

url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
destfile0 <- paste0(tempdir(), '/sherlock0.rttm')
download.file(url0, destfile = destfile0)
url1 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock1.rttm'
destfile1 <- paste0(tempdir(), '/sherlock1.rttm')
download.file(url0, destfile = destfile1)

rttm <- voice::read_rttm(dirname(destfile0))
(er <- voice::enrich_rttm(rttm))
class(er)
lapply(er, class)
}
}
\references{
\url{https://www.nist.gov/system/files/documents/itl/iad/mig/KWS15-evalplan-v05.pdf}
}
\seealso{
\code{voice::read_rttm}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_list.R
\name{write_list}
\alias{write_list}
\title{Writes a list to a path}
\usage{
write_list(x, path)
}
\arguments{
\item{x}{A list.}

\item{path}{A full path to file.}
}
\value{
A file named `list.txt` in `path`.
}
\description{
Writes a list to a path
}
\examples{
\dontrun{
library(voice)

pts <- list(x = cars[,1], y = cars[,2])
listFile <- paste0(tempdir(), '/list.txt')
voice::write_list(pts, listFile)
file.info(listFile)
system(paste0('head ', listFile))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_dur.R
\name{get_dur}
\alias{get_dur}
\title{Time duration}
\usage{
get_dur(x)
}
\arguments{
\item{x}{Wave object from `tuneR::readWave`.}
}
\value{
Numeric indicating the time duration in seconds from a WAV file.
}
\description{
Get time duration from WAV file.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern <- glob2rx('*.wav'), full.names = TRUE)

rw <- tuneR::readWave(path2wav[1])
voice::get_dur(rw)

rwl <- lapply(path2wav, tuneR::readWave)
sapply(rwl, voice::get_dur)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assign_notes.R
\name{assign_notes}
\alias{assign_notes}
\title{Assign musical notes}
\usage{
assign_notes(
  x,
  fmt = 0,
  min_points = 4,
  min_percentile = 0.75,
  max_na_prop = 1
)
}
\arguments{
\item{x}{Media dataset from voice::extract_features().}

\item{fmt}{Either F0 or formant frequency (in Hz). Default: \code{fmt = 0}.}

\item{min_points}{Minimum number of points for audio section. Default: \code{min_points = 4}.}

\item{min_percentile}{Minimum percentile value of gain to be included on the average of \code{fmt}. Default: \code{min_percentile = 0.75}.}

\item{max_na_prop}{Maximum proportion os NAs on gain sector. Default: \code{max_na_prop = 1}.}
}
\description{
Assign musical notes in Scientific Pitch Notation or other variant. See \code{voice::notes()}. The notes are cut considering f0 to ensure alignment.
}
\examples{
library(voice)
# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)
# Media dataset
M <- extract_features(path2wav)
assign_notes(M, fmt = 0) # f0
assign_notes(M, fmt = 1) # f1
assign_notes(M, fmt = 2) # f2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_mono.R
\name{is_mono}
\alias{is_mono}
\title{Verify if an audio is mono}
\usage{
is_mono(x)
}
\arguments{
\item{x}{Path to WAV audio file.}
}
\value{
Logical. `TRUE` indicates a mono (one-channel) file. `FALSE` indicates a non-mono (two-channel) file.
}
\description{
Verify if an audio is mono
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

is_mono(path2wav[1])
sapply(path2wav, is_mono)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interp_mc.R
\name{interp_mc}
\alias{interp_mc}
\title{Interpolate vectors using multicore}
\usage{
interp_mc(
  y,
  compact.to,
  drop.zeros = FALSE,
  to.data.frame = FALSE,
  round.off = NULL,
  weight = NULL,
  mc.cores = 1
)
}
\arguments{
\item{y}{A numeric vector, matrix or data frame.}

\item{compact.to}{Proportion of remaining points after compression. If equals to 1 and keep.zeros = TRUE, the original vector is presented.}

\item{drop.zeros}{Logical. Drop repeated zeros? Default: \code{FALSE}.}

\item{to.data.frame}{Logical. Convert to data frame? Default: \code{FALSE}.}

\item{round.off}{Number of decimal places of the interpolated \code{y}. Default: \code{NULL}.}

\item{weight}{Vector of weights with same length of \code{y}. Default: \code{NULL}.}

\item{mc.cores}{The number of cores to mclapply. Default: \code{1}.}
}
\value{
A list of x and y convoluted values with length near to \code{compact.to*length(y)}.
}
\description{
Interpolate vectors using multicore
}
\examples{
library(voice)
# Same result of interp() function if x is a vector
interp(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = FALSE)
interp_mc(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = FALSE)

interp(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = TRUE)
interp_mc(1:100, compact.to = 0.1, drop.zeros = TRUE, to.data.frame = TRUE)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

\donttest{
# getting Media data frame
M <- voice::extract_features(dirname(path2wav), mc.cores = 1, verbose = FALSE)

M.num <- M[,-(1:3)]
nrow(M.num)
cm1 <- interp_mc(M.num, compact.to = 0.1, drop.zeros = TRUE,
to.data.frame = FALSE, mc.cores = 1)
names(cm1)
lapply(cm1$f0, length)
}
}
\seealso{
\code{rm0}, \code{interp}, \code{interp_df}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_features.R
\name{extract_features}
\alias{extract_features}
\title{Extract audio features}
\usage{
extract_features(
  x,
  features = c("f0", "fmt", "gain"),
  filesRange = NULL,
  sex = "u",
  windowShift = 10,
  numFormants = 8,
  numcep = 12,
  dcttype = c("t2", "t1", "t3", "t4"),
  fbtype = c("mel", "htkmel", "fcmel", "bark"),
  resolution = 40,
  usecmp = FALSE,
  mc.cores = 1,
  full.names = TRUE,
  recursive = FALSE,
  check.mono = FALSE,
  stereo2mono = FALSE,
  overwrite = FALSE,
  freq = 44100,
  round.to = NULL,
  verbose = FALSE,
  pycall = "~/miniconda3/envs/pyvoice/bin/python"
)
}
\arguments{
\item{x}{A vector containing either files or directories of audio files in WAV format.}

\item{features}{Vector of features to be extracted. (Default: \code{'f0','fmt','gain'}). Available features: \code{'f0','f0_mhs','f0_praat','fmt','fmt_praat','zcr','rms','gain','rfc','ac','cep','dft','css','lps','mfcc','df','pf','rf','rcf','rpf'}.}

\item{filesRange}{The desired range of directory files (Default: \code{NULL}, i.e., all files). Should only be used when all the WAV files are in the same folder.}

\item{sex}{\code{= <code>} set sex specific parameters where <code> = \code{'f'}[emale], \code{'m'}[ale] or \code{'u'}[nknown] (Default: \code{'u'}). Used as 'gender' by \code{wrassp::ksvF0}, \code{wrassp::forest} and \code{wrassp::mhsF0}.}

\item{windowShift}{\code{= <dur>} set analysis window shift to <dur>ation in ms (Default: \code{5.0}). Used by \code{wrassp::ksvF0}, \code{wrassp::forest}, \code{wrassp::mhsF0}, \code{wrassp::zcrana}, \code{wrassp::rfcana}, \code{wrassp::acfana}, \code{wrassp::cepstrum}, \code{wrassp::dftSpectrum}, \code{wrassp::cssSpectrum} and \code{wrassp::lpsSpectrum}.}

\item{numFormants}{\code{= <num>} <num>ber of formants (Default: \code{8}). Used by \code{wrassp::forest}.}

\item{numcep}{Number of Mel-frequency cepstral coefficients (cepstra) to return (Default: \code{12}). Used by \code{tuneR::melfcc}.}

\item{dcttype}{Type of DCT used. \code{'t1'} or \code{'t2'}, \code{'t3'} for HTK \code{'t4'} for feacalc (Default: \code{'t2'}). Used by \code{tuneR::melfcc}.}

\item{fbtype}{Auditory frequency scale to use: \code{'mel'}, \code{'bark'}, \code{'htkmel'}, \code{'fcmel'} (Default: \code{'mel'}). Used by \code{tuneR::melfcc}.}

\item{resolution}{\code{= <freq>} set FFT length to the smallest value which results in a frequency resolution of <freq> Hz or better (Default: \code{40.0}). Used by \code{wrassp::cssSpectrum}, \code{wrassp::dftSpectrum} and \code{wrassp::lpsSpectrum}.}

\item{usecmp}{Logical. Apply equal-loudness weighting and cube-root compression (PLP instead of LPC) (Default: \code{FALSE}). Used by \code{tuneR::melfcc}.}

\item{mc.cores}{Number of cores to be used in parallel processing. (Default: \code{1})}

\item{full.names}{Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (Default: \code{TRUE}) Used by \code{base::list.files}.}

\item{recursive}{Logical. Should the listing recursively into directories? (Default: \code{FALSE}) Used by \code{base::list.files}.}

\item{check.mono}{Logical. Check if the WAV file is mono. (Default: \code{TRUE})}

\item{stereo2mono}{(Experimental) Logical. Should files be converted from stereo to mono? (Default: \code{TRUE})}

\item{overwrite}{(Experimental) Logical. Should converted files be overwritten? If not, the file gets the suffix \code{_mono}. (Default: \code{FALSE})}

\item{freq}{Frequency in Hz to write the converted files when \code{stereo2mono=TRUE}. (Default: \code{44100})}

\item{round.to}{Number of decimal places to round to. (Default: \code{NULL})}

\item{verbose}{Logical. Should the running status be showed? (Default: \code{FALSE})}

\item{pycall}{Python call. See \url{https://github.com/filipezabala/voice} for details.}
}
\value{
A Media data frame containing the selected features.
}
\description{
Extracts features from WAV audio files.
}
\details{
The feature 'df' corresponds to 'formant dispersion' (df2:df8) by
Fitch (1997), 'pf' to formant position' (pf1:pf8) by Puts, Apicella & Cárdena
(2011), 'rf' to 'formant removal' (rf1:rf8) by Zabala (2025), 'rcf' to
'formant cumulated removal' (rcf2:rcf8) by Zabala (2025) and 'rpf' to
'formant position removal' (rpf2:rpf8) by Zabala (2025).
The \code{'fmt_praat'} feature may take long time processing. The following
features may contain a variable number of columns: \code{'cep'}, \code{'dft'},
\code{'css'} and \code{'lps'}.
On Windows, this function may occasionally cause the RStudio session to crash.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

# minimal usage
M1 <- extract_features(path2wav)
M2 <- extract_features(dirname(path2wav))
identical(M1,M2)
table(basename(M1$wav_path))

# limiting filesRange
M3 <- extract_features(path2wav, filesRange = 3:6)
table(basename(M3$wav_path))
}
\references{
Levinson N. (1946). The Wiener (root mean square) error criterion in filter design and prediction. Journal of Mathematics and Physics, 25(1-4), 261–278. (\doi{10.1002/SAPM1946251261})

Durbin J. (1960). “The fitting of time-series models.” Revue de l’Institut International de Statistique, pp. 233–244. (\url{https://www.jstor.org/stable/1401322})

Cooley J.W., Tukey J.W. (1965). “An algorithm for the machine calculation of complex Fourier series.” Mathematics of computation, 19(90), 297–301. (\url{https://www.ams.org/journals/mcom/1965-19-090/S0025-5718-1965-0178586-1/})

Wasson D., Donaldson R. (1975). “Speech amplitude and zero crossings for automated identification of human speakers.” IEEE Transactions on Acoustics, Speech, and Signal Processing, 23(4), 390–392. (\url{https://ieeexplore.ieee.org/document/1162690})

Allen J. (1977). “Short term spectral analysis, synthesis, and modification by discrete Fourier transform.” IEEE Transactions on Acoustics, Speech, and Signal Processing, 25(3), 235– 238. (\url{https://ieeexplore.ieee.org/document/1162950})

Schäfer-Vincent K. (1982). "Significant points: Pitch period detection as a problem of segmentation." Phonetica, 39(4-5), 241–253. (\doi{10.1159/000261665} )

Schäfer-Vincent K. (1983). "Pitch period detection and chaining: Method and evaluation." Phonetica, 40(3), 177–202. (\doi{10.1159/000261691})

Ephraim Y., Malah D. (1984). “Speech enhancement using a minimum-mean square error short-time spectral amplitude estimator.” IEEE Transactions on acoustics, speech, and signal processing, 32(6), 1109–1121. (\url{https://ieeexplore.ieee.org/document/1164453})

Delsarte P., Genin Y. (1986). “The split Levinson algorithm.” IEEE transactions on acoustics, speech, and signal processing, 34(3), 470–478. (\url{https://ieeexplore.ieee.org/document/1164830})

Jackson J.C. (1995). "The Harmonic Sieve: A Novel Application of Fourier Analysis to Machine Learning Theory and Practice." Technical report, Carnegie-Mellon University Pittsburgh PA Schoo; of Computer Science.

Fitch, W.T. (1997) "Vocal tract length and formant frequency dispersion correlate with body size in rhesus macaques." J. Acoust. Soc. Am. 102, 1213 – 1222. (\doi{10.1121/1.421048})

Boersma P., van Heuven V. (2001). Praat, a system for doing phonetics by computer. Glot. Int., 5(9/10), 341–347. (\url{https://www.fon.hum.uva.nl/paul/papers/speakUnspeakPraat_glot2001.pdf})

Ellis DPW (2005). “PLP and RASTA (and MFCC, and inversion) in Matlab.” Online web resource.

Puts, D.A., Apicella, C.L., Cardenas, R.A. (2012) "Masculine voices signal men's threat potential in forager and industrial societies." Proc. R. Soc. B Biol. Sci. 279, 601–609. (\doi{10.1098/rspb.2011.0829})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transcribe.R
\name{transcribe}
\alias{transcribe}
\title{Transcribe audio to text}
\usage{
transcribe(
  file,
  server = "http://localhost:8080",
  wait = TRUE,
  poll_interval = 2,
  timeout = 300
)
}
\arguments{
\item{file}{Audio file to trinscribe.}

\item{server}{Default: http://localhost:8080.}

\item{wait}{Default: \code{TRUE}.}

\item{poll_interval}{Default: 2.}

\item{timeout}{Default: 300.}
}
\description{
Transcribe audio to text.
}
\references{
https://github.com/rishikanthc/scriberr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is.hosted}
\alias{is.hosted}
\title{Return strings with a URL scheme}
\usage{
is.hosted(x)
}
\arguments{
\item{x}{A character vector.}
}
\description{
Given a character vector, returns a logical indicating whether the
URLs in the vector respond without error.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag.R
\name{tag}
\alias{tag}
\title{Tag a data frame with media information}
\usage{
tag(
  x,
  groupBy = "wav_path",
  wavPath = unique(x$wav_path),
  wavPathName = "wav_path",
  tags = c("feat_summary"),
  sortByGroupBy = TRUE,
  filesRange = NULL,
  features = "f0",
  sex = "u",
  windowShift = 5,
  numFormants = 8,
  numcep = 12,
  dcttype = c("t2", "t1", "t3", "t4"),
  fbtype = c("mel", "htkmel", "fcmel", "bark"),
  resolution = 40,
  usecmp = FALSE,
  mc.cores = 1,
  full.names = TRUE,
  recursive = FALSE,
  check.mono = FALSE,
  stereo2mono = FALSE,
  overwrite = FALSE,
  freq = 44100,
  round.to = 4,
  verbose = FALSE
)
}
\arguments{
\item{x}{An Extended data frame to be tagged with media information. See references.}

\item{groupBy}{A variable to group the summary measures. The argument must be a character vector. (Default: \code{groupBy = 'wav_path'}).}

\item{wavPath}{A vector containing the path(s) to WAV files. May be both as \code{dirname} or \code{basename} formats.}

\item{wavPathName}{A string containing the WAV path name. (Default: \code{wavPathName = 'wav_path'}).}

\item{tags}{Tags to be added to \code{x}. See Details. (Default: \code{'feat_summary'}).}

\item{sortByGroupBy}{Logical. Should the function sort the Extended data frame \code{x} by \code{gropuBy}? (Default: \code{sortByGroupBy = TRUE}).}

\item{filesRange}{The desired range of directory files. Should only be used when all the WAV files are in the same folder. (Default: \code{NULL}, i.e., all files).}

\item{features}{Vector of features to be extracted. (Default: \code{'f0'}).}

\item{sex}{\code{= <code>} set sex specific parameters where <code> = \code{'f'}[emale], \code{'m'}[ale] or \code{'u'}[nknown] (default: \code{'u'}). Used as 'gender' by \code{wrassp::ksvF0}, \code{wrassp::forest} and \code{wrassp::mhsF0}.}

\item{windowShift}{\code{= <dur>} set analysis window shift to <dur>ation in ms (default: 5.0). Used by \code{wrassp::ksvF0}, \code{wrassp::forest}, \code{wrassp::mhsF0}, \code{wrassp::zcrana}, \code{wrassp::rfcana}, \code{wrassp::acfana}, \code{wrassp::cepstrum}, \code{wrassp::dftSpectrum}, \code{wrassp::cssSpectrum} and \code{wrassp::lpsSpectrum}.}

\item{numFormants}{\code{= <num>} <num>ber of formants (Default: \code{8}). Used by \code{wrassp::forest}.}

\item{numcep}{Number of Mel-frequency cepstral coefficients (cepstra) to return (Default: \code{12}). Used by \code{tuneR::melfcc}.}

\item{dcttype}{Type of DCT used. \code{'t1'} or \code{'t2'}, \code{'t3'} for HTK \code{'t4'} for feacalc (Default: \code{'t2'}). Used by \code{tuneR::melfcc}.}

\item{fbtype}{Auditory frequency scale to use: \code{'mel'}, \code{'bark'}, \code{'htkmel'}, \code{'fcmel'} (Default: \code{'mel'}). Used by \code{tuneR::melfcc}.}

\item{resolution}{\code{= <freq>} set FFT length to the smallest value which results in a frequency resolution of <freq> Hz or better (Default: \code{40.0}). Used by \code{wrassp::cssSpectrum}, \code{wrassp::dftSpectrum} and \code{wrassp::lpsSpectrum}.}

\item{usecmp}{Logical. Apply equal-loudness weighting and cube-root compression (PLP instead of LPC) (Default: \code{FALSE}). Used by \code{tuneR::melfcc}.}

\item{mc.cores}{Number of cores to be used in parallel processing. (Default: \code{1})}

\item{full.names}{Logical. If \code{TRUE}, the directory path is prepended to the file names to give a relative file path. If \code{FALSE}, the file names (rather than paths) are returned. (Default: \code{TRUE}) Used by \code{base::list.files}.}

\item{recursive}{Logical. Should the listing recursively into directories? (Default: \code{FALSE}) Used by \code{base::list.files}.}

\item{check.mono}{Logical. Check if the WAV file is mono. (Default: \code{TRUE})}

\item{stereo2mono}{(Experimental) Logical. Should files be converted from stereo to mono? (Default: \code{TRUE})}

\item{overwrite}{(Experimental) Logical. Should converted files be overwritten? If not, the file gets the suffix \code{_mono}. (Default: \code{FALSE})}

\item{freq}{Frequency in Hz to write the converted files when \code{stereo2mono=TRUE}. (Default: \code{44100})}

\item{round.to}{Number of decimal places to round to. (Default: \code{NULL})}

\item{verbose}{Logical. Should the running status be showed? (Default: \code{FALSE})}
}
\value{
A tibble data frame containing summarized numeric columns using (1) mean, (2) standard deviation, (3) variation coefficient, (4) median, (5) interquartile range and (6) median absolute deviation.
}
\description{
Tag a data frame with media information
}
\details{
\code{filesRange} should only be used when all the WAV files are in the same folder.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

# creating Extended synthetic data
E <- dplyr::tibble(subject_id = c(1,1,1,2,2,2,3,3,3),
wav_path = path2wav)
E

# minimal usage
tag(E)

# canonical data
tag(E, groupBy = 'subject_id')

# limiting filesRange
tag(E, filesRange = 3:6)

# more features
Et <- tag(E, features = c('f0', 'fmt', 'rf', 'rcf', 'rpf', 'rfc', 'mfcc'),
groupBy = 'subject_id')
Et
str(Et)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interp_df.R
\name{interp_df}
\alias{interp_df}
\title{Inperpolate data frames}
\usage{
interp_df(
  x,
  compact.to,
  id = colnames(x)[1],
  colnum = NULL,
  drop.x = TRUE,
  drop.zeros = FALSE,
  to.data.frame = TRUE,
  round.off = NULL,
  weight = NULL,
  mc.cores = 1
)
}
\arguments{
\item{x}{A data frame.}

\item{compact.to}{Proportion of remaining points after interpolation. If equals to 1 and keep.zeros = TRUE, the original vector is presented.}

\item{id}{The identification column. Default: \code{colname} of the first column of \code{x}.}

\item{colnum}{A \code{char} vector indicating the numeric colnames. If \code{NULL}, uses the columns of the \code{numeric} class.}

\item{drop.x}{Logical. Drop columns containing .x? Default: \code{TRUE}.}

\item{drop.zeros}{Logical. Drop repeated zeros or keep 1 zero per null set? Default: \code{FALSE}.}

\item{to.data.frame}{Logical. Should return a data frame? If \code{FALSE} returns a list. Default: \code{TRUE}.}

\item{round.off}{Number of decimal places of the interpolated \code{y}. Default: \code{NULL}.}

\item{weight}{Vector of weights with same length of \code{y}. Default: \code{NULL}.}

\item{mc.cores}{The number of cores to mclapply. Default: \code{1}.}
}
\value{
A data frame of interpolated values with nrow near to \code{compact.to*length(x)}.
}
\description{
Interpolate data frames using multicore, compressing to \code{compact.to} fraction. May remove zeros.
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)

# getting Media data frame via lean call
M <- extract_features(dirname(path2wav), features = c('f0','fmt'),
mc.cores = 1, verbose = FALSE)

\donttest{
(cM.df <- interp_df(M[,-(1:2)], 0.1, mc.cores = 1))
(cM.df2 <- interp_df(M[,-(1:2)], 0.1, drop.x = FALSE, mc.cores = 1))

dim(M)
dim(cM.df)
dim(cM.df2)
(cM.list <- interp_df(M[,-(1:2)], 0.1, to.data.frame = FALSE, mc.cores = 1))
}
}
\seealso{
\code{interp}, \code{interp_mc}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tdur.R
\name{get_tdur}
\alias{get_tdur}
\title{Time duration}
\usage{
get_tdur(x)
}
\arguments{
\item{x}{A data frame in RTTM standard. See 'voice::read_rttm'.}
}
\value{
Numeric vector containing the time duration in seconds.
}
\description{
Get time duration from a data frame in RTTM standard.
}
\examples{
library(voice)

url0 <- 'https://raw.githubusercontent.com/filipezabala/voiceAudios/main/rttm/sherlock0.rttm'
download.file(url0, destfile = paste0(tempdir(), '/sherlock0.rttm'))

rttm <- voice::read_rttm(tempdir())
(gtd <- voice::get_tdur(rttm$sherlock0.rttm))
class(gtd)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audio_time.R
\name{audio_time}
\alias{audio_time}
\title{Returns the total time of audio files in seconds}
\usage{
audio_time(x, filesRange = NULL, recursive = FALSE)
}
\arguments{
\item{x}{Either a WAV file or a directory containing WAV files.}

\item{filesRange}{The desired range of directory files (default: \code{NULL}, i.e., all files).}

\item{recursive}{Logical. Should the listing recursively into directories? (default: \code{FALSE}) Used by \code{base::list.files}.}
}
\value{
A tibble containing file name <chr> and audio time <dbl> in seconds.
}
\description{
Returns the total time of audio files in seconds
}
\examples{
library(voice)

# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern <- glob2rx('*.wav'), full.names = TRUE)

# Tibble containing file name and audio time
(at <- voice::audio_time(unique(dirname(path2wav))))
str(at)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rm0.R
\name{rm0}
\alias{rm0}
\title{Compress zeros.}
\usage{
rm0(y)
}
\arguments{
\item{y}{A vector or time series.}
}
\value{
Vector with n zeros.
}
\description{
Transforms \code{n} sets of \code{m>n} zeros (alternated with sets of non zeros) into \code{n} sets of \code{n} zeros.
}
\examples{
library(voice)

(v0 <- c(1:20,rep(0,10)))
(r0 <- rm0(v0))
length(v0)
length(r0)
sum(v0 == 0)

(v1 <- c(rep(0,10),1:20))
(r1 <- rm0(v1))
length(r1)

(v2 <- rep(0,10))
(r2 <- rm0(v2))
length(r2)

(v3 <- c(0:10))
(r3 <- rm0(v3))
length(r3)

(v4 <- c(rep(0,10), 1:10, rep(0,5), 10:20, rep(0,10)))
(r4 <- rm0(v4))
length(r4)
sum(v4 == 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is.local}
\alias{is.local}
\title{Return strings without a URL scheme}
\usage{
is.local(x)
}
\arguments{
\item{x}{A character vector.}
}
\description{
Given a character vector, returns a logical indicating whether the
paths in the vector point to existing local files.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piano_plot.R
\name{piano_plot}
\alias{piano_plot}
\title{Piano plot}
\usage{
piano_plot(dat, num_fmt = 0, log_freq = TRUE, base = exp(1), color = "slice")
}
\arguments{
\item{dat}{Data frame or tibble containing the desired frequencies to be plotted.}

\item{num_fmt}{Number of the desired formant (includes f0 for simplicity). Default: \code{num_fmt = 0}.}

\item{log_freq}{Logical. Must plot log(frequency)?}

\item{base}{Logarithm base. Default: \code{exp(1)}.}

\item{color}{Must the graph be colored overall or by slice? Default: \code{slice}.}
}
\description{
Piano plot showing the notes in Scientific Pitch Notation.
}
\examples{
library(voice)
# get path to audio file
path2wav <- list.files(system.file('extdata', package = 'wrassp'),
pattern = glob2rx('*.wav'), full.names = TRUE)
# Media dataset
M <- extract_features(path2wav[1])
piano_plot(M, 0)
piano_plot(M, 0, color = 'overall')
}
\references{
https://en.wikipedia.org/wiki/12_equal_temperament

https://en.wikipedia.org/wiki/Scientific_pitch_notation
}
